@@ -27,11 +27,16 @@ import (
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/bootloader"
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/gadget"
+	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/snapfile"
+	"github.com/snapcore/snapd/strutil"
 )
 
+var sealKeyToModeenv = sealKeyToModeenvImpl
+
 // BootableSet represents the boot snaps of a system to be made bootable.
 type BootableSet struct {
 	Base       *snap.Info
@@ -49,7 +54,7 @@ type BootableSet struct {
 
 	UnpackedGadgetDir string
 
-	// Recover is set when making the recovery partition bootable.
+	// Recovery is set when making the recovery partition bootable.
 	Recovery bool
 }
 
@@ -75,15 +80,25 @@ func MakeBootableImage(model *asserts.Model, rootdir string, bootWith *BootableS
 	if !bootWith.Recovery {
 		return fmt.Errorf("internal error: MakeBootableImage called at runtime, use MakeRunnableSystem instead")
 	}
-	return makeBootable20(rootdir, bootWith, bootFlags)
+	return makeBootable20(model, rootdir, bootWith, bootFlags)
+}
+
+// MakeBootablePartition configures a partition mounted on rootdir
+// using information from bootWith and bootFlags. Contrarily to
+// MakeBootableImage this happens in a live system.
+func MakeBootablePartition(partDir string, opts *bootloader.Options, bootWith *BootableSet, bootMode string, bootFlags []string) error {
+	if bootWith.RecoverySystemDir != "" {
+		return fmt.Errorf("internal error: RecoverySystemDir unexpectedly set for MakeBootablePartition")
+	}
+	return configureBootloader(partDir, opts, bootWith, bootMode, bootFlags)
 }
 
 // makeBootable16 setups the image filesystem for boot with UC16
 // and UC18 models. This entails:
-//  - installing the bootloader configuration from the gadget
-//  - creating symlinks for boot snaps from seed to the runtime blob dir
-//  - setting boot env vars pointing to the revisions of the boot snaps to use
-//  - extracting kernel assets as needed by the bootloader
+//   - installing the bootloader configuration from the gadget
+//   - creating symlinks for boot snaps from seed to the runtime blob dir
+//   - setting boot env vars pointing to the revisions of the boot snaps to use
+//   - extracting kernel assets as needed by the bootloader
 func makeBootable16(model *asserts.Model, rootdir string, bootWith *BootableSet) error {
 	opts := &bootloader.Options{
 		PrepareImageTime: true,
@@ -155,20 +170,7 @@ func makeBootable16(model *asserts.Model, rootdir string, bootWith *BootableSet)
 	return nil
 }
 
-func makeBootable20(rootdir string, bootWith *BootableSet, bootFlags []string) error {
-	// we can only make a single recovery system bootable right now
-	recoverySystems, err := filepath.Glob(filepath.Join(rootdir, "systems/*"))
-	if err != nil {
-		return fmt.Errorf("cannot validate recovery systems: %v", err)
-	}
-	if len(recoverySystems) > 1 {
-		return fmt.Errorf("cannot make multiple recovery systems bootable yet")
-	}
-
-	if bootWith.RecoverySystemLabel == "" {
-		return fmt.Errorf("internal error: recovery system label unset")
-	}
-
+func configureBootloader(rootdir string, opts *bootloader.Options, bootWith *BootableSet, bootMode string, bootFlags []string) error {
 	blVars := make(map[string]string, 3)
 	if len(bootFlags) != 0 {
 		if err := setImageBootFlags(bootFlags, blVars); err != nil {
@@ -176,12 +178,6 @@ func makeBootable20(rootdir string, bootWith *BootableSet, bootFlags []string) e
 		}
 	}
 
-	opts := &bootloader.Options{
-		PrepareImageTime: true,
-		// setup the recovery bootloader
-		Role: bootloader.RoleRecovery,
-	}
-
 	// install the bootloader configuration from the gadget
 	if err := bootloader.InstallBootConfig(bootWith.UnpackedGadgetDir, rootdir, opts); err != nil {
 		return err
@@ -193,20 +189,48 @@ func makeBootable20(rootdir string, bootWith *BootableSet, bootFlags []string) e
 		return fmt.Errorf("internal error: cannot find bootloader: %v", err)
 	}
 
-	// record which recovery system is to be used on the bootloader, note
-	// that this goes on the main bootloader environment, and not on the
-	// recovery system bootloader environment, for example for grub
-	// bootloader, this env var is set on the ubuntu-seed root grubenv, and
-	// not on the recovery system grubenv in the systems/20200314/ subdir on
-	// ubuntu-seed
-	blVars["snapd_recovery_system"] = bootWith.RecoverySystemLabel
-	// always set the mode as install
-	blVars["snapd_recovery_mode"] = ModeInstall
+	blVars["snapd_recovery_mode"] = bootMode
+	if bootWith.RecoverySystemLabel != "" {
+		// record which recovery system is to be used on the bootloader, note
+		// that this goes on the main bootloader environment, and not on the
+		// recovery system bootloader environment, for example for grub
+		// bootloader, this env var is set on the ubuntu-seed root grubenv, and
+		// not on the recovery system grubenv in the systems/20200314/ subdir on
+		// ubuntu-seed
+		blVars["snapd_recovery_system"] = bootWith.RecoverySystemLabel
+	}
+
 	if err := bl.SetBootVars(blVars); err != nil {
 		return fmt.Errorf("cannot set recovery environment: %v", err)
 	}
 
-	return MakeRecoverySystemBootable(rootdir, bootWith.RecoverySystemDir, &RecoverySystemBootableSet{
+	return nil
+}
+
+func makeBootable20(model *asserts.Model, rootdir string, bootWith *BootableSet, bootFlags []string) error {
+	// we can only make a single recovery system bootable right now
+	recoverySystems, err := filepath.Glob(filepath.Join(rootdir, "systems/*"))
+	if err != nil {
+		return fmt.Errorf("cannot validate recovery systems: %v", err)
+	}
+	if len(recoverySystems) > 1 {
+		return fmt.Errorf("cannot make multiple recovery systems bootable yet")
+	}
+
+	if bootWith.RecoverySystemLabel == "" {
+		return fmt.Errorf("internal error: recovery system label unset")
+	}
+
+	opts := &bootloader.Options{
+		PrepareImageTime: true,
+		// setup the recovery bootloader
+		Role: bootloader.RoleRecovery,
+	}
+	if err := configureBootloader(rootdir, opts, bootWith, ModeInstall, bootFlags); err != nil {
+		return fmt.Errorf("cannot install bootloader: %v", err)
+	}
+
+	return MakeRecoverySystemBootable(model, rootdir, bootWith.RecoverySystemDir, &RecoverySystemBootableSet{
 		Kernel:           bootWith.Kernel,
 		KernelPath:       bootWith.KernelPath,
 		GadgetSnapOrDir:  bootWith.UnpackedGadgetDir,
@@ -227,7 +251,7 @@ type RecoverySystemBootableSet struct {
 
 // MakeRecoverySystemBootable prepares a recovery system under a path relative
 // to recovery bootloader's rootdir for booting.
-func MakeRecoverySystemBootable(rootdir string, relativeRecoverySystemDir string, bootWith *RecoverySystemBootableSet) error {
+func MakeRecoverySystemBootable(model *asserts.Model, rootdir string, relativeRecoverySystemDir string, bootWith *RecoverySystemBootableSet) error {
 	opts := &bootloader.Options{
 		// XXX: this is only needed by LK, it is unclear whether LK does
 		// too much when extracting recovery kernel assets, in the end
@@ -277,7 +301,13 @@ func MakeRecoverySystemBootable(rootdir string, relativeRecoverySystemDir string
 		"snapd_recovery_kernel": filepath.Join("/", kernelPath),
 	}
 	if _, ok := bl.(bootloader.TrustedAssetsBootloader); ok {
-		recoveryCmdlineArgs, err := bootVarsForTrustedCommandLineFromGadget(bootWith.GadgetSnapOrDir)
+		// Look at gadget default values for system.kernel.*cmdline-append options
+		cmdlineAppend, err := buildOptionalKernelCommandLine(model, bootWith.GadgetSnapOrDir)
+		if err != nil {
+			return fmt.Errorf("while retrieving system.kernel.*cmdline-append defaults: %v", err)
+		}
+		// to set cmdlineAppend.
+		recoveryCmdlineArgs, err := bootVarsForTrustedCommandLineFromGadget(bootWith.GadgetSnapOrDir, cmdlineAppend)
 		if err != nil {
 			return fmt.Errorf("cannot obtain recovery system command line: %v", err)
 		}
@@ -293,44 +323,66 @@ func MakeRecoverySystemBootable(rootdir string, relativeRecoverySystemDir string
 }
 
 type makeRunnableOptions struct {
+	Standalone     bool
 	AfterDataReset bool
+	SeedDir        string
+}
+
+func copyBootSnap(orig string, dstInfo *snap.Info, dstSnapBlobDir string) error {
+	// if the source path is a symlink, don't copy the symlink, copy the
+	// target file instead of copying the symlink, as the initramfs won't
+	// follow the symlink when it goes to mount the base and kernel snaps by
+	// design as the initramfs should only be using trusted things from
+	// ubuntu-data to boot in run mode
+	if osutil.IsSymlink(orig) {
+		link, err := os.Readlink(orig)
+		if err != nil {
+			return err
+		}
+		orig = link
+	}
+	// note that we need to use the "Filename()" here because unasserted
+	// snaps will have names like pc-kernel_5.19.4.snap but snapd expects
+	// "pc-kernel_x1.snap"
+	dst := filepath.Join(dstSnapBlobDir, dstInfo.Filename())
+	if err := osutil.CopyFile(orig, dst, osutil.CopyFlagPreserveAll|osutil.CopyFlagSync); err != nil {
+		return err
+	}
+	return nil
 }
 
 func makeRunnableSystem(model *asserts.Model, bootWith *BootableSet, sealer *TrustedAssetsInstallObserver, makeOpts makeRunnableOptions) error {
 	if model.Grade() == asserts.ModelGradeUnset {
 		return fmt.Errorf("internal error: cannot make pre-UC20 system runnable")
 	}
+	if bootWith.RecoverySystemDir != "" {
+		return fmt.Errorf("internal error: RecoverySystemDir unexpectedly set for MakeRunnableSystem")
+	}
+
 	// TODO:UC20:
 	// - figure out what to do for uboot gadgets, currently we require them to
 	//   install the boot.sel onto ubuntu-boot directly, but the file should be
 	//   managed by snapd instead
 
 	// copy kernel/base/gadget into the ubuntu-data partition
-	snapBlobDir := dirs.SnapBlobDirUnder(InstallHostWritableDir)
+	snapBlobDir := dirs.SnapBlobDirUnder(InstallHostWritableDir(model))
 	if err := os.MkdirAll(snapBlobDir, 0755); err != nil {
 		return err
 	}
-	for _, fn := range []string{bootWith.BasePath, bootWith.KernelPath, bootWith.GadgetPath} {
-		dst := filepath.Join(snapBlobDir, filepath.Base(fn))
-		// if the source filename is a symlink, don't copy the symlink, copy the
-		// target file instead of copying the symlink, as the initramfs won't
-		// follow the symlink when it goes to mount the base and kernel snaps by
-		// design as the initramfs should only be using trusted things from
-		// ubuntu-data to boot in run mode
-		if osutil.IsSymlink(fn) {
-			link, err := os.Readlink(fn)
-			if err != nil {
-				return err
-			}
-			fn = link
-		}
-		if err := osutil.CopyFile(fn, dst, osutil.CopyFlagPreserveAll|osutil.CopyFlagSync); err != nil {
+	for _, origDest := range []struct {
+		orig     string
+		destInfo *snap.Info
+	}{
+		{orig: bootWith.BasePath, destInfo: bootWith.Base},
+		{orig: bootWith.KernelPath, destInfo: bootWith.Kernel},
+		{orig: bootWith.GadgetPath, destInfo: bootWith.Gadget}} {
+		if err := copyBootSnap(origDest.orig, origDest.destInfo, snapBlobDir); err != nil {
 			return err
 		}
 	}
 
 	// replicate the boot assets cache in host's writable
-	if err := CopyBootAssetsCacheToRoot(InstallHostWritableDir); err != nil {
+	if err := CopyBootAssetsCacheToRoot(InstallHostWritableDir(model)); err != nil {
 		return fmt.Errorf("cannot replicate boot assets cache: %v", err)
 	}
 
@@ -340,7 +392,7 @@ func makeRunnableSystem(model *asserts.Model, bootWith *BootableSet, sealer *Tru
 		currentTrustedBootAssets = sealer.currentTrustedBootAssetsMap()
 		currentTrustedRecoveryBootAssets = sealer.currentTrustedRecoveryBootAssetsMap()
 	}
-	recoverySystemLabel := filepath.Base(bootWith.RecoverySystemDir)
+	recoverySystemLabel := bootWith.RecoverySystemLabel
 	// write modeenv on the ubuntu-data partition
 	modeenv := &Modeenv{
 		Mode:           "run",
@@ -355,8 +407,7 @@ func makeRunnableSystem(model *asserts.Model, bootWith *BootableSet, sealer *Tru
 		// installed
 		CurrentKernelCommandLines: nil,
 		// keep this comment to make gofmt 1.9 happy
-		Base:           filepath.Base(bootWith.BasePath),
-		Gadget:         filepath.Base(bootWith.GadgetPath),
+		Gadget:         bootWith.Gadget.Filename(),
 		CurrentKernels: []string{bootWith.Kernel.Filename()},
 		BrandID:        model.BrandID(),
 		Model:          model.Model(),
@@ -365,6 +416,11 @@ func makeRunnableSystem(model *asserts.Model, bootWith *BootableSet, sealer *Tru
 		Grade:          string(model.Grade()),
 		ModelSignKeyID: model.SignKeyID(),
 	}
+	// Note on classic systems there is no boot base, the system boots
+	// from debs.
+	if !model.Classic() {
+		modeenv.Base = bootWith.Base.Filename()
+	}
 
 	// get the ubuntu-boot bootloader and extract the kernel there
 	opts := &bootloader.Options{
@@ -441,7 +497,12 @@ func makeRunnableSystem(model *asserts.Model, bootWith *BootableSet, sealer *Tru
 		}
 		modeenv.CurrentKernelCommandLines = bootCommandLines{cmdline}
 
-		cmdlineVars, err := bootVarsForTrustedCommandLineFromGadget(bootWith.UnpackedGadgetDir)
+		// Look at gadget default values for system.kernel.*cmdline-append options
+		cmdlineAppend, err := buildOptionalKernelCommandLine(model, bootWith.UnpackedGadgetDir)
+		if err != nil {
+			return fmt.Errorf("while retrieving system.kernel.*cmdline-append defaults: %v", err)
+		}
+		cmdlineVars, err := bootVarsForTrustedCommandLineFromGadget(bootWith.UnpackedGadgetDir, cmdlineAppend)
 		if err != nil {
 			return fmt.Errorf("cannot prepare bootloader variables for kernel command line: %v", err)
 		}
@@ -452,13 +513,23 @@ func makeRunnableSystem(model *asserts.Model, bootWith *BootableSet, sealer *Tru
 
 	// all fields that needed to be set in the modeenv must have been set by
 	// now, write modeenv to disk
-	if err := modeenv.WriteTo(InstallHostWritableDir); err != nil {
+	if err := modeenv.WriteTo(InstallHostWritableDir(model)); err != nil {
 		return fmt.Errorf("cannot write modeenv: %v", err)
 	}
 
 	if sealer != nil {
+		hasHook, err := HasFDESetupHook(bootWith.Kernel)
+		if err != nil {
+			return fmt.Errorf("cannot check for fde-setup hook: %v", err)
+		}
+
 		flags := sealKeyToModeenvFlags{
-			FactoryReset: makeOpts.AfterDataReset,
+			HasFDESetupHook: hasHook,
+			FactoryReset:    makeOpts.AfterDataReset,
+			SeedDir:         makeOpts.SeedDir,
+		}
+		if makeOpts.Standalone {
+			flags.SnapsDir = snapBlobDir
 		}
 		// seal the encryption key to the parameters specified in modeenv
 		if err := sealKeyToModeenv(sealer.dataEncryptionKey, sealer.saveEncryptionKey, model, modeenv, flags); err != nil {
@@ -474,6 +545,48 @@ func makeRunnableSystem(model *asserts.Model, bootWith *BootableSet, sealer *Tru
 	return nil
 }
 
+func buildOptionalKernelCommandLine(model *asserts.Model, gadgetSnapOrDir string) (string, error) {
+	sf, err := snapfile.Open(gadgetSnapOrDir)
+	if err != nil {
+		return "", fmt.Errorf("cannot open gadget snap: %v", err)
+	}
+	gadgetInfo, err := gadget.ReadInfoFromSnapFile(sf, nil)
+	if err != nil {
+		return "", fmt.Errorf("cannot read gadget data: %v", err)
+	}
+
+	defaults := gadget.SystemDefaults(gadgetInfo.Defaults)
+
+	var cmdlineAppend, cmdlineAppendDangerous string
+
+	if cmdlineAppendIf, ok := defaults["system.kernel.cmdline-append"]; ok {
+		cmdlineAppend, ok = cmdlineAppendIf.(string)
+		if !ok {
+			return "", fmt.Errorf("system.kernel.cmdline-append is not a string")
+		}
+	}
+
+	if cmdlineAppendIf, ok := defaults["system.kernel.dangerous-cmdline-append"]; ok {
+		cmdlineAppendDangerous, ok = cmdlineAppendIf.(string)
+		if !ok {
+			return "", fmt.Errorf("system.kernel.dangerous-cmdline-append is not a string")
+		}
+		if model.Grade() != asserts.ModelDangerous {
+			// Print a warning and ignore
+			logger.Noticef("WARNING: system.kernel.dangerous-cmdline-append ignored by non-dangerous models")
+			return "", nil
+		}
+	}
+
+	if cmdlineAppend != "" {
+		// TODO perform validation against what is allowed by the gadget
+	}
+
+	cmdlineAppend = strutil.JoinNonEmpty([]string{cmdlineAppend, cmdlineAppendDangerous}, " ")
+
+	return cmdlineAppend, nil
+}
+
 // MakeRunnableSystem is like MakeBootableImage in that it sets up a system to
 // be able to boot, but is unique in that it is intended to be called from UC20
 // install mode and makes the run system bootable (hence it is called
@@ -484,7 +597,32 @@ func makeRunnableSystem(model *asserts.Model, bootWith *BootableSet, sealer *Tru
 // setting up a run system and actually transitioning to it, with hooks, etc.
 // running in between.
 func MakeRunnableSystem(model *asserts.Model, bootWith *BootableSet, sealer *TrustedAssetsInstallObserver) error {
-	return makeRunnableSystem(model, bootWith, sealer, makeRunnableOptions{})
+	return makeRunnableSystem(model, bootWith, sealer, makeRunnableOptions{
+		SeedDir: dirs.SnapSeedDir,
+	})
+}
+
+// MakeRunnableStandaloneSystem operates like MakeRunnableSystem but does
+// assume that the run system being set up is related to the current
+// system. This is appropriate e.g when installing from a classic installer.
+func MakeRunnableStandaloneSystem(model *asserts.Model, bootWith *BootableSet, sealer *TrustedAssetsInstallObserver) error {
+	// TODO consider merging this back into MakeRunnableSystem but need
+	// to consider the properties of the different input used for sealing
+	return makeRunnableSystem(model, bootWith, sealer, makeRunnableOptions{
+		Standalone: true,
+		SeedDir:    dirs.SnapSeedDir,
+	})
+}
+
+// MakeRunnableStandaloneSystemFromInitrd is the same as MakeRunnableStandaloneSystem
+// but uses seed dir path expected in initrd.
+func MakeRunnableStandaloneSystemFromInitrd(model *asserts.Model, bootWith *BootableSet, sealer *TrustedAssetsInstallObserver) error {
+	// TODO consider merging this back into MakeRunnableSystem but need
+	// to consider the properties of the different input used for sealing
+	return makeRunnableSystem(model, bootWith, sealer, makeRunnableOptions{
+		Standalone: true,
+		SeedDir:    filepath.Join(InitramfsRunMntDir, "ubuntu-seed"),
+	})
 }
 
 // MakeRunnableSystemAfterDataReset sets up the system to be able to boot, but it is
@@ -493,5 +631,6 @@ func MakeRunnableSystem(model *asserts.Model, bootWith *BootableSet, sealer *Tru
 func MakeRunnableSystemAfterDataReset(model *asserts.Model, bootWith *BootableSet, sealer *TrustedAssetsInstallObserver) error {
 	return makeRunnableSystem(model, bootWith, sealer, makeRunnableOptions{
 		AfterDataReset: true,
+		SeedDir:        dirs.SnapSeedDir,
 	})
 }
