@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2014-2020 Canonical Ltd
+ * Copyright (C) 2014-2021 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -163,6 +163,31 @@ type ExtractedRunKernelImageBootloader interface {
 	DisableTryKernel() error
 }
 
+// ComamndLineComponents carries the components of the kernel command line. The
+// bootloader is expected to combine the provided components, optionally
+// including its built-in static set of arguments, and produce a command line
+// that will be passed to the kernel during boot.
+type CommandLineComponents struct {
+	// Argument related to mode selection.
+	ModeArg string
+	// Argument related to recovery system selection, relevant for given
+	// mode argument.
+	SystemArg string
+	// Extra arguments requested by the system.
+	ExtraArgs string
+	// A complete set of arguments that overrides both the built-in static
+	// set and ExtraArgs. Note that, it is an error if extra and full
+	// arguments are non-empty.
+	FullArgs string
+}
+
+func (c *CommandLineComponents) Validate() error {
+	if c.ExtraArgs != "" && c.FullArgs != "" {
+		return fmt.Errorf("cannot use both full and extra components of command line")
+	}
+	return nil
+}
+
 // TrustedAssetsBootloader has boot assets that take part in the secure boot
 // process and need to be tracked, while other boot assets (typically boot
 // config) are managed by snapd.
@@ -176,14 +201,15 @@ type TrustedAssetsBootloader interface {
 	// the bootloader. Returns true when assets were updated.
 	UpdateBootConfig() (bool, error)
 	// CommandLine returns the kernel command line composed of mode and
-	// system arguments, built-in bootloader specific static arguments
-	// corresponding to the on-disk boot asset edition, followed by any
-	// extra arguments. The command line may be different when using a
-	// recovery bootloader.
-	CommandLine(modeArg, systemArg, extraArgs string) (string, error)
+	// system arguments, followed by either a built-in bootloader specific
+	// static arguments corresponding to the on-disk boot asset edition, and
+	// any extra arguments or a separate set of arguments provided in the
+	// components. The command line may be different when using a recovery
+	// bootloader.
+	CommandLine(pieces CommandLineComponents) (string, error)
 	// CandidateCommandLine is similar to CommandLine, but uses the current
 	// edition of managed built-in boot assets as reference.
-	CandidateCommandLine(modeArg, systemArg, extraArgs string) (string, error)
+	CandidateCommandLine(pieces CommandLineComponents) (string, error)
 
 	// TrustedAssets returns the list of relative paths to assets inside the
 	// bootloader's rootdir that are measured in the boot process in the
@@ -200,6 +226,27 @@ type TrustedAssetsBootloader interface {
 	BootChain(runBl Bootloader, kernelPath string) ([]BootFile, error)
 }
 
+// NotScriptableBootloader cannot change the bootloader environment
+// because it supports no scripting or cannot do any writes. This
+// applies to piboot for the moment.
+type NotScriptableBootloader interface {
+	Bootloader
+
+	// Sets boot variables from initramfs - this is needed in
+	// addition to SetBootVars() to prevent side effects like
+	// re-writing the bootloader configuration.
+	SetBootVarsFromInitramfs(values map[string]string) error
+}
+
+// RebootBootloader needs arguments to the reboot syscall when snaps
+// are being updated.
+type RebootBootloader interface {
+	Bootloader
+
+	// GetRebootArguments returns the needed reboot arguments
+	GetRebootArguments() (string, error)
+}
+
 func genericInstallBootConfig(gadgetFile, systemFile string) error {
 	if err := os.MkdirAll(filepath.Dir(systemFile), 0755); err != nil {
 		return err
@@ -268,6 +315,7 @@ var (
 		newGrub,
 		newAndroidBoot,
 		newLk,
+		newPiboot,
 	}
 )
 
@@ -281,7 +329,8 @@ var (
 //
 // The rootdir option is useful for image creation operations. It
 // can also be used to find the recovery bootloader, e.g. on uc20:
-//   bootloader.Find("/run/mnt/ubuntu-seed")
+//
+//	bootloader.Find("/run/mnt/ubuntu-seed")
 func Find(rootdir string, opts *Options) (Bootloader, error) {
 	if err := opts.validate(); err != nil {
 		return nil, err
