@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2014-2020 Canonical Ltd
+ * Copyright (C) 2014-2021 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -25,13 +25,14 @@ import (
 	"path/filepath"
 	"strings"
 
+	"github.com/snapcore/snapd/arch"
 	"github.com/snapcore/snapd/bootloader/assets"
 	"github.com/snapcore/snapd/bootloader/grubenv"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/snap"
 )
 
-// sanity - grub implements the required interfaces
+// grub implements the required interfaces
 var (
 	_ Bootloader                        = (*grub)(nil)
 	_ RecoveryAwareBootloader           = (*grub)(nil)
@@ -47,6 +48,7 @@ type grub struct {
 	uefiRunKernelExtraction bool
 	recovery                bool
 	nativePartitionLayout   bool
+	prepareImageTime        bool
 }
 
 // newGrub create a new Grub bootloader object
@@ -60,6 +62,7 @@ func newGrub(rootdir string, opts *Options) Bootloader {
 		g.uefiRunKernelExtraction = opts.Role == RoleRunMode
 		g.recovery = opts.Role == RoleRecovery
 		g.nativePartitionLayout = opts.NoSlashBoot || g.recovery
+		g.prepareImageTime = opts.PrepareImageTime
 	}
 	if g.nativePartitionLayout {
 		g.basedir = "EFI/ubuntu"
@@ -81,13 +84,13 @@ func (g *grub) dir() string {
 	return filepath.Join(g.rootdir, g.basedir)
 }
 
-func (g *grub) installManagedRecoveryBootConfig(gadgetDir string) error {
+func (g *grub) installManagedRecoveryBootConfig() error {
 	assetName := g.Name() + "-recovery.cfg"
 	systemFile := filepath.Join(g.rootdir, "/EFI/ubuntu/grub.cfg")
 	return genericSetBootConfigFromAsset(systemFile, assetName)
 }
 
-func (g *grub) installManagedBootConfig(gadgetDir string) error {
+func (g *grub) installManagedBootConfig() error {
 	assetName := g.Name() + ".cfg"
 	systemFile := filepath.Join(g.rootdir, "/EFI/ubuntu/grub.cfg")
 	return genericSetBootConfigFromAsset(systemFile, assetName)
@@ -96,11 +99,11 @@ func (g *grub) installManagedBootConfig(gadgetDir string) error {
 func (g *grub) InstallBootConfig(gadgetDir string, opts *Options) error {
 	if opts != nil && opts.Role == RoleRecovery {
 		// install managed config for the recovery partition
-		return g.installManagedRecoveryBootConfig(gadgetDir)
+		return g.installManagedRecoveryBootConfig()
 	}
 	if opts != nil && opts.Role == RoleRunMode {
 		// install managed boot config that can handle kernel.efi
-		return g.installManagedBootConfig(gadgetDir)
+		return g.installManagedBootConfig()
 	}
 
 	gadgetFile := filepath.Join(gadgetDir, g.Name()+".conf")
@@ -360,13 +363,24 @@ func (g *grub) ManagedAssets() []string {
 	}
 }
 
-func (g *grub) commandLineForEdition(edition uint, modeArg, systemArg, extraArgs string) (string, error) {
+func (g *grub) commandLineForEdition(edition uint, pieces CommandLineComponents) (string, error) {
 	assetName := "grub.cfg"
 	if g.recovery {
 		assetName = "grub-recovery.cfg"
 	}
-	staticCmdline := staticCommandLineForGrubAssetEdition(assetName, edition)
-	args, err := osutil.KernelCommandLineSplit(staticCmdline + " " + extraArgs)
+
+	if err := pieces.Validate(); err != nil {
+		return "", err
+	}
+
+	var nonSnapdCmdline string
+	if pieces.FullArgs == "" {
+		staticCmdline := staticCommandLineForGrubAssetEdition(assetName, edition)
+		nonSnapdCmdline = staticCmdline + " " + pieces.ExtraArgs
+	} else {
+		nonSnapdCmdline = pieces.FullArgs
+	}
+	args, err := osutil.KernelCommandLineSplit(nonSnapdCmdline)
 	if err != nil {
 		return "", fmt.Errorf("cannot use badly formatted kernel command line: %v", err)
 	}
@@ -375,23 +389,24 @@ func (g *grub) commandLineForEdition(edition uint, modeArg, systemArg, extraArgs
 	// arguments are separated by a single space, the space after last is
 	// replaced with terminating NULL
 	snapdArgs := make([]string, 0, 2)
-	if modeArg != "" {
-		snapdArgs = append(snapdArgs, modeArg)
+	if pieces.ModeArg != "" {
+		snapdArgs = append(snapdArgs, pieces.ModeArg)
 	}
-	if systemArg != "" {
-		snapdArgs = append(snapdArgs, systemArg)
+	if pieces.SystemArg != "" {
+		snapdArgs = append(snapdArgs, pieces.SystemArg)
 	}
 	return strings.Join(append(snapdArgs, args...), " "), nil
 }
 
 // CommandLine returns the kernel command line composed of mode and
-// system arguments, built-in bootloader specific static arguments
-// corresponding to the on-disk boot asset edition, followed by any
-// extra arguments. The command line may be different when using a
-// recovery bootloader.
+// system arguments, followed by either a built-in bootloader specific
+// static arguments corresponding to the on-disk boot asset edition, and
+// any extra arguments or a separate set of arguments provided in the
+// components. The command line may be different when using a recovery
+// bootloader.
 //
 // Implements TrustedAssetsBootloader for the grub bootloader.
-func (g *grub) CommandLine(modeArg, systemArg, extraArgs string) (string, error) {
+func (g *grub) CommandLine(pieces CommandLineComponents) (string, error) {
 	currentBootConfig := filepath.Join(g.dir(), "grub.cfg")
 	edition, err := editionFromDiskConfigAsset(currentBootConfig)
 	if err != nil {
@@ -405,14 +420,14 @@ func (g *grub) CommandLine(modeArg, systemArg, extraArgs string) (string, error)
 		// used before we started writing out the files ourselves
 		edition = 1
 	}
-	return g.commandLineForEdition(edition, modeArg, systemArg, extraArgs)
+	return g.commandLineForEdition(edition, pieces)
 }
 
 // CandidateCommandLine is similar to CommandLine, but uses the current
 // edition of managed built-in boot assets as reference.
 //
 // Implements TrustedAssetsBootloader for the grub bootloader.
-func (g *grub) CandidateCommandLine(modeArg, systemArg, extraArgs string) (string, error) {
+func (g *grub) CandidateCommandLine(pieces CommandLineComponents) (string, error) {
 	assetName := "grub.cfg"
 	if g.recovery {
 		assetName = "grub-recovery.cfg"
@@ -421,7 +436,7 @@ func (g *grub) CandidateCommandLine(modeArg, systemArg, extraArgs string) (strin
 	if err != nil {
 		return "", err
 	}
-	return g.commandLineForEdition(edition, modeArg, systemArg, extraArgs)
+	return g.commandLineForEdition(edition, pieces)
 }
 
 // staticCommandLineForGrubAssetEdition fetches a static command line for given
@@ -434,19 +449,54 @@ func staticCommandLineForGrubAssetEdition(asset string, edition uint) string {
 	return string(cmdline)
 }
 
-var (
-	grubRecoveryModeTrustedAssets = []string{
-		// recovery mode shim EFI binary
-		"EFI/boot/bootx64.efi",
-		// recovery mode grub EFI binary
-		"EFI/boot/grubx64.efi",
-	}
+// grubBootAssetPath contains the paths for assets in the boot chain.
+type grubBootAssetPath struct {
+	shimBinary string
+	grubBinary string
+}
 
-	grubRunModeTrustedAssets = []string{
-		// run mode grub EFI binary
-		"EFI/boot/grubx64.efi",
+// grubBootAssetsForArch contains the paths for assets for different
+// architectures in a map
+var grubBootAssetsForArch = map[string]grubBootAssetPath{
+	"amd64": {
+		shimBinary: filepath.Join("EFI/boot/", "bootx64.efi"),
+		grubBinary: filepath.Join("EFI/boot/", "grubx64.efi")},
+	"arm64": {
+		shimBinary: filepath.Join("EFI/boot/", "bootaa64.efi"),
+		grubBinary: filepath.Join("EFI/boot/", "grubaa64.efi")},
+}
+
+func (g *grub) getGrubBootAssetsForArch() (*grubBootAssetPath, error) {
+	if g.prepareImageTime {
+		return nil, fmt.Errorf("internal error: retrieving boot assets at prepare image time")
 	}
-)
+	archi := arch.DpkgArchitecture()
+	assets, ok := grubBootAssetsForArch[archi]
+	if !ok {
+		return nil, fmt.Errorf("cannot find grub assets for %q", archi)
+	}
+	return &assets, nil
+}
+
+// getGrubRecoveryModeTrustedAssets returns the assets for recovery mode,
+// which are shim and grub from the seed partition.
+func (g *grub) getGrubRecoveryModeTrustedAssets() ([]string, error) {
+	assets, err := g.getGrubBootAssetsForArch()
+	if err != nil {
+		return nil, err
+	}
+	return []string{assets.shimBinary, assets.grubBinary}, nil
+}
+
+// getGrubRunModeTrustedAssets returns the assets for run mode, which is
+// grub from the boot partition.
+func (g *grub) getGrubRunModeTrustedAssets() ([]string, error) {
+	assets, err := g.getGrubBootAssetsForArch()
+	if err != nil {
+		return nil, err
+	}
+	return []string{assets.grubBinary}, nil
+}
 
 // TrustedAssets returns the list of relative paths to assets inside
 // the bootloader's rootdir that are measured in the boot process in the
@@ -456,9 +506,9 @@ func (g *grub) TrustedAssets() ([]string, error) {
 		return nil, fmt.Errorf("internal error: trusted assets called without native host-partition layout")
 	}
 	if g.recovery {
-		return grubRecoveryModeTrustedAssets, nil
+		return g.getGrubRecoveryModeTrustedAssets()
 	}
-	return grubRunModeTrustedAssets, nil
+	return g.getGrubRunModeTrustedAssets()
 }
 
 // RecoveryBootChain returns the load chain for recovery modes.
@@ -469,8 +519,12 @@ func (g *grub) RecoveryBootChain(kernelPath string) ([]BootFile, error) {
 	}
 
 	// add trusted assets to the recovery chain
-	chain := make([]BootFile, 0, len(grubRecoveryModeTrustedAssets)+1)
-	for _, ta := range grubRecoveryModeTrustedAssets {
+	assets, err := g.getGrubRecoveryModeTrustedAssets()
+	if err != nil {
+		return nil, err
+	}
+	chain := make([]BootFile, 0, len(assets)+1)
+	for _, ta := range assets {
 		chain = append(chain, NewBootFile("", ta, RoleRecovery))
 	}
 	// add recovery kernel to the recovery chain
@@ -491,11 +545,19 @@ func (g *grub) BootChain(runBl Bootloader, kernelPath string) ([]BootFile, error
 	}
 
 	// add trusted assets to the recovery chain
-	chain := make([]BootFile, 0, len(grubRecoveryModeTrustedAssets)+len(grubRunModeTrustedAssets)+1)
-	for _, ta := range grubRecoveryModeTrustedAssets {
+	recoveryModeAssets, err := g.getGrubRecoveryModeTrustedAssets()
+	if err != nil {
+		return nil, err
+	}
+	runModeAssets, err := g.getGrubRunModeTrustedAssets()
+	if err != nil {
+		return nil, err
+	}
+	chain := make([]BootFile, 0, len(recoveryModeAssets)+len(runModeAssets)+1)
+	for _, ta := range recoveryModeAssets {
 		chain = append(chain, NewBootFile("", ta, RoleRecovery))
 	}
-	for _, ta := range grubRunModeTrustedAssets {
+	for _, ta := range runModeAssets {
 		chain = append(chain, NewBootFile("", ta, RoleRunMode))
 	}
 	// add kernel to the boot chain
