@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2014-2015 Canonical Ltd
+ * Copyright (C) 2014-2021 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -29,6 +29,8 @@ import (
 	"github.com/mvo5/goconfigparser"
 	. "gopkg.in/check.v1"
 
+	"github.com/snapcore/snapd/arch"
+	"github.com/snapcore/snapd/arch/archtest"
 	"github.com/snapcore/snapd/bootloader"
 	"github.com/snapcore/snapd/bootloader/assets"
 	"github.com/snapcore/snapd/bootloader/grubenv"
@@ -53,6 +55,14 @@ func (s *grubTestSuite) SetUpTest(c *C) {
 	bootloader.MockGrubFiles(c, s.rootdir)
 
 	s.bootdir = filepath.Join(s.rootdir, "boot")
+	// By default assume amd64 in the tests: there are specialized
+	// tests for other arches
+	s.AddCleanup(archtest.MockArchitecture("amd64"))
+	snippets := []assets.ForEditions{
+		{FirstEdition: 1, Snippet: []byte("console=ttyS0 console=tty1 panic=-1")},
+	}
+	s.AddCleanup(assets.MockSnippetsForEdition("grub.cfg:static-cmdline", snippets))
+	s.AddCleanup(assets.MockSnippetsForEdition("grub-recovery.cfg:static-cmdline", snippets))
 }
 
 // grubEditenvCmd finds the right grub{,2}-editenv command
@@ -612,6 +622,26 @@ func (s *grubTestSuite) TestKernelExtractionRunImageKernelNoSlashBoot(c *C) {
 	c.Check(exists, Equals, false)
 }
 
+func (s *grubTestSuite) TestListTrustedAssetsNotForArch(c *C) {
+	oldArch := arch.DpkgArchitecture()
+	defer arch.SetArchitecture(arch.ArchitectureType(oldArch))
+	arch.SetArchitecture("non-existing-architecture")
+
+	s.makeFakeGrubEFINativeEnv(c, []byte(`this is
+some random boot config`))
+
+	opts := &bootloader.Options{NoSlashBoot: true}
+	g := bootloader.NewGrub(s.rootdir, opts)
+	c.Assert(g, NotNil)
+
+	tg, ok := g.(bootloader.TrustedAssetsBootloader)
+	c.Assert(ok, Equals, true)
+
+	ta, err := tg.TrustedAssets()
+	c.Check(err, ErrorMatches, `cannot find grub assets for "non-existing-architecture"`)
+	c.Check(ta, HasLen, 0)
+}
+
 func (s *grubTestSuite) TestListManagedAssets(c *C) {
 	s.makeFakeGrubEFINativeEnv(c, []byte(`this is
 some random boot config`))
@@ -838,14 +868,21 @@ func (s *grubTestSuite) TestCommandLineNotManaged(c *C) {
 	opts := &bootloader.Options{NoSlashBoot: true}
 	mg := bootloader.NewGrub(s.rootdir, opts).(bootloader.TrustedAssetsBootloader)
 
-	args, err := mg.CommandLine("snapd_recovery_mode=run", "", "extra")
+	args, err := mg.CommandLine(bootloader.CommandLineComponents{
+		ModeArg:   "snapd_recovery_mode=run",
+		ExtraArgs: "extra",
+	})
 	c.Assert(err, IsNil)
 	c.Check(args, Equals, "snapd_recovery_mode=run static=1 extra")
 
 	optsRecovery := &bootloader.Options{NoSlashBoot: true, Role: bootloader.RoleRecovery}
 	mgr := bootloader.NewGrub(s.rootdir, optsRecovery).(bootloader.TrustedAssetsBootloader)
 
-	args, err = mgr.CommandLine("snapd_recovery_mode=recover", "snapd_recovery_system=1234", "extra")
+	args, err = mgr.CommandLine(bootloader.CommandLineComponents{
+		ModeArg:   "snapd_recovery_mode=recover",
+		SystemArg: "snapd_recovery_system=1234",
+		ExtraArgs: "extra",
+	})
 	c.Assert(err, IsNil)
 	c.Check(args, Equals, "snapd_recovery_mode=recover snapd_recovery_system=1234 static=1 recovery extra")
 }
@@ -877,19 +914,28 @@ boot script
 	c.Assert(ok, Equals, true)
 
 	extraArgs := `extra_arg=1  extra_foo=-1   panic=3 baz="more  spaces"`
-	args, err := tg.CommandLine("snapd_recovery_mode=run", "", extraArgs)
+	args, err := tg.CommandLine(bootloader.CommandLineComponents{
+		ModeArg:   "snapd_recovery_mode=run",
+		ExtraArgs: extraArgs,
+	})
 	c.Assert(err, IsNil)
 	c.Check(args, Equals, `snapd_recovery_mode=run arg1 foo=123 panic=-1 arg2="with spaces " extra_arg=1 extra_foo=-1 panic=3 baz="more  spaces"`)
 
 	// empty mode/system do not produce confusing results
-	args, err = tg.CommandLine("", "", extraArgs)
+	args, err = tg.CommandLine(bootloader.CommandLineComponents{
+		ExtraArgs: extraArgs,
+	})
 	c.Assert(err, IsNil)
 	c.Check(args, Equals, `arg1 foo=123 panic=-1 arg2="with spaces " extra_arg=1 extra_foo=-1 panic=3 baz="more  spaces"`)
 
 	// now check the recovery bootloader
 	optsRecovery := &bootloader.Options{NoSlashBoot: true, Role: bootloader.RoleRecovery}
 	mrg := bootloader.NewGrub(s.rootdir, optsRecovery).(bootloader.TrustedAssetsBootloader)
-	args, err = mrg.CommandLine("snapd_recovery_mode=recover", "snapd_recovery_system=20200202", extraArgs)
+	args, err = mrg.CommandLine(bootloader.CommandLineComponents{
+		ModeArg:   "snapd_recovery_mode=recover",
+		SystemArg: "snapd_recovery_system=20200202",
+		ExtraArgs: extraArgs,
+	})
 	c.Assert(err, IsNil)
 	// static command line from recovery asset
 	c.Check(args, Equals, `snapd_recovery_mode=recover snapd_recovery_system=20200202 recovery config panic=-1 extra_arg=1 extra_foo=-1 panic=3 baz="more  spaces"`)
@@ -902,9 +948,28 @@ boot script
 	tg = bootloader.NewGrub(s.rootdir, optsNoSlashBoot).(bootloader.TrustedAssetsBootloader)
 	c.Assert(g, NotNil)
 	extraArgs = `extra_arg=1`
-	args, err = tg.CommandLine("snapd_recovery_mode=run", "", extraArgs)
+	args, err = tg.CommandLine(bootloader.CommandLineComponents{
+		ModeArg:   "snapd_recovery_mode=run",
+		ExtraArgs: extraArgs,
+	})
 	c.Assert(err, IsNil)
 	c.Check(args, Equals, `snapd_recovery_mode=run edition=3 static args extra_arg=1`)
+
+	// full args set overrides static arguments
+	args, err = tg.CommandLine(bootloader.CommandLineComponents{
+		ModeArg:  "snapd_recovery_mode=run",
+		FullArgs: "full for run mode",
+	})
+	c.Assert(err, IsNil)
+	c.Check(args, Equals, `snapd_recovery_mode=run full for run mode`)
+	args, err = mrg.CommandLine(bootloader.CommandLineComponents{
+		ModeArg:   "snapd_recovery_mode=recover",
+		SystemArg: "snapd_recovery_system=20200202",
+		FullArgs:  "full for recover mode",
+	})
+	c.Assert(err, IsNil)
+	c.Check(args, Equals, `snapd_recovery_mode=recover snapd_recovery_system=20200202 full for recover mode`)
+
 }
 
 func (s *grubTestSuite) TestCandidateCommandLineMocked(c *C) {
@@ -940,10 +1005,17 @@ boot script
 	optsRecovery := &bootloader.Options{NoSlashBoot: true, Role: bootloader.RoleRecovery}
 	recoverymg := bootloader.NewGrub(s.rootdir, optsRecovery).(bootloader.TrustedAssetsBootloader)
 
-	args, err := mg.CandidateCommandLine("snapd_recovery_mode=run", "", "extra=1")
+	args, err := mg.CandidateCommandLine(bootloader.CommandLineComponents{
+		ModeArg:   "snapd_recovery_mode=run",
+		ExtraArgs: "extra=1",
+	})
 	c.Assert(err, IsNil)
 	c.Check(args, Equals, `snapd_recovery_mode=run edition=1 extra=1`)
-	args, err = recoverymg.CandidateCommandLine("snapd_recovery_mode=recover", "snapd_recovery_system=20200202", "extra=1")
+	args, err = recoverymg.CandidateCommandLine(bootloader.CommandLineComponents{
+		ModeArg:   "snapd_recovery_mode=recover",
+		SystemArg: "snapd_recovery_system=20200202",
+		ExtraArgs: "extra=1",
+	})
 	c.Assert(err, IsNil)
 	c.Check(args, Equals, `snapd_recovery_mode=recover snapd_recovery_system=20200202 recovery edition=1 extra=1`)
 
@@ -952,10 +1024,17 @@ boot script
 	restore = assets.MockInternal("grub-recovery.cfg", edition3)
 	defer restore()
 
-	args, err = mg.CandidateCommandLine("snapd_recovery_mode=run", "", "extra=1")
+	args, err = mg.CandidateCommandLine(bootloader.CommandLineComponents{
+		ModeArg:   "snapd_recovery_mode=run",
+		ExtraArgs: "extra=1",
+	})
 	c.Assert(err, IsNil)
 	c.Check(args, Equals, `snapd_recovery_mode=run edition=3 extra=1`)
-	args, err = recoverymg.CandidateCommandLine("snapd_recovery_mode=recover", "snapd_recovery_system=20200202", "extra=1")
+	args, err = recoverymg.CandidateCommandLine(bootloader.CommandLineComponents{
+		ModeArg:   "snapd_recovery_mode=recover",
+		SystemArg: "snapd_recovery_system=20200202",
+		ExtraArgs: "extra=1",
+	})
 	c.Assert(err, IsNil)
 	c.Check(args, Equals, `snapd_recovery_mode=recover snapd_recovery_system=20200202 recovery edition=3 extra=1`)
 
@@ -963,13 +1042,29 @@ boot script
 	restore = assets.MockInternal("grub-recovery.cfg", edition4)
 	defer restore()
 	// boot bootloader unchanged
-	args, err = mg.CandidateCommandLine("snapd_recovery_mode=run", "", "extra=1")
+	args, err = mg.CandidateCommandLine(bootloader.CommandLineComponents{
+		ModeArg:   "snapd_recovery_mode=run",
+		ExtraArgs: "extra=1",
+	})
 	c.Assert(err, IsNil)
 	c.Check(args, Equals, `snapd_recovery_mode=run edition=3 extra=1`)
 	// recovery uses a new edition
-	args, err = recoverymg.CandidateCommandLine("snapd_recovery_mode=recover", "snapd_recovery_system=20200202", "extra=1")
+	args, err = recoverymg.CandidateCommandLine(bootloader.CommandLineComponents{
+		ModeArg:   "snapd_recovery_mode=recover",
+		SystemArg: "snapd_recovery_system=20200202",
+		ExtraArgs: "extra=1",
+	})
 	c.Assert(err, IsNil)
 	c.Check(args, Equals, `snapd_recovery_mode=recover snapd_recovery_system=20200202 recovery edition=4up extra=1`)
+
+	// the static snippet is ignored when using full arg set
+	args, err = recoverymg.CandidateCommandLine(bootloader.CommandLineComponents{
+		ModeArg:   "snapd_recovery_mode=recover",
+		SystemArg: "snapd_recovery_system=20200202",
+		FullArgs:  "full args set",
+	})
+	c.Assert(err, IsNil)
+	c.Check(args, Equals, `snapd_recovery_mode=recover snapd_recovery_system=20200202 full args set`)
 }
 
 func (s *grubTestSuite) TestCommandLineReal(c *C) {
@@ -986,17 +1081,90 @@ boot script
 	c.Assert(ok, Equals, true)
 
 	extraArgs := "foo bar baz=1"
-	args, err := tg.CommandLine("snapd_recovery_mode=run", "", extraArgs)
+	args, err := tg.CommandLine(bootloader.CommandLineComponents{
+		ModeArg:   "snapd_recovery_mode=run",
+		ExtraArgs: extraArgs,
+	})
 	c.Assert(err, IsNil)
 	c.Check(args, Equals, `snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1 foo bar baz=1`)
+	// with full args the static part is not used
+	args, err = tg.CommandLine(bootloader.CommandLineComponents{
+		ModeArg:  "snapd_recovery_mode=run",
+		FullArgs: "full for run mode",
+	})
+	c.Assert(err, IsNil)
+	c.Check(args, Equals, `snapd_recovery_mode=run full for run mode`)
 
 	// now check the recovery bootloader
 	opts = &bootloader.Options{NoSlashBoot: true, Role: bootloader.RoleRecovery}
 	mrg := bootloader.NewGrub(s.rootdir, opts).(bootloader.TrustedAssetsBootloader)
-	args, err = mrg.CommandLine("snapd_recovery_mode=recover", "snapd_recovery_system=20200202", extraArgs)
+	args, err = mrg.CommandLine(bootloader.CommandLineComponents{
+		ModeArg:   "snapd_recovery_mode=recover",
+		SystemArg: "snapd_recovery_system=20200202",
+		ExtraArgs: extraArgs,
+	})
 	c.Assert(err, IsNil)
 	// static command line from recovery asset
 	c.Check(args, Equals, `snapd_recovery_mode=recover snapd_recovery_system=20200202 console=ttyS0 console=tty1 panic=-1 foo bar baz=1`)
+	// similarly, when passed full args, the static part is not used
+	args, err = mrg.CommandLine(bootloader.CommandLineComponents{
+		ModeArg:   "snapd_recovery_mode=recover",
+		SystemArg: "snapd_recovery_system=20200202",
+		FullArgs:  "full for recover mode",
+	})
+	c.Assert(err, IsNil)
+	c.Check(args, Equals, `snapd_recovery_mode=recover snapd_recovery_system=20200202 full for recover mode`)
+}
+
+func (s *grubTestSuite) TestCommandLineComponentsValidate(c *C) {
+	grubCfg := `# Snapd-Boot-Config-Edition: 1
+boot script
+`
+	// native EFI/ubuntu setup
+	s.makeFakeGrubEFINativeEnv(c, []byte(grubCfg))
+
+	opts := &bootloader.Options{NoSlashBoot: true}
+	g := bootloader.NewGrub(s.rootdir, opts)
+	c.Assert(g, NotNil)
+	tg, ok := g.(bootloader.TrustedAssetsBootloader)
+	c.Assert(ok, Equals, true)
+
+	args, err := tg.CommandLine(bootloader.CommandLineComponents{
+		ModeArg:   "snapd_recovery_mode=run",
+		ExtraArgs: "extra is set",
+		FullArgs:  "full is set",
+	})
+	c.Assert(err, ErrorMatches, "cannot use both full and extra components of command line")
+	c.Check(args, Equals, "")
+	// invalid for the candidate command line too
+	args, err = tg.CandidateCommandLine(bootloader.CommandLineComponents{
+		ModeArg:   "snapd_recovery_mode=run",
+		ExtraArgs: "extra is set",
+		FullArgs:  "full is set",
+	})
+	c.Assert(err, ErrorMatches, "cannot use both full and extra components of command line")
+	c.Check(args, Equals, "")
+
+	// now check the recovery bootloader
+	opts = &bootloader.Options{NoSlashBoot: true, Role: bootloader.RoleRecovery}
+	mrg := bootloader.NewGrub(s.rootdir, opts).(bootloader.TrustedAssetsBootloader)
+	args, err = mrg.CommandLine(bootloader.CommandLineComponents{
+		ModeArg:   "snapd_recovery_mode=recover",
+		SystemArg: "snapd_recovery_system=20200202",
+		ExtraArgs: "extra is set",
+		FullArgs:  "full is set",
+	})
+	c.Assert(err, ErrorMatches, "cannot use both full and extra components of command line")
+	c.Check(args, Equals, "")
+	// candidate recovery command line is checks validity of the components too
+	args, err = mrg.CandidateCommandLine(bootloader.CommandLineComponents{
+		ModeArg:   "snapd_recovery_mode=recover",
+		SystemArg: "snapd_recovery_system=20200202",
+		ExtraArgs: "extra is set",
+		FullArgs:  "full is set",
+	})
+	c.Assert(err, ErrorMatches, "cannot use both full and extra components of command line")
+	c.Check(args, Equals, "")
 }
 
 func (s *grubTestSuite) TestTrustedAssetsNativePartitionLayout(c *C) {
@@ -1040,6 +1208,26 @@ func (s *grubTestSuite) TestTrustedAssetsRoot(c *C) {
 	c.Check(ta, IsNil)
 }
 
+func (s *grubTestSuite) TestTrustedAssetsFailAtPrepareImageTime(c *C) {
+	// native EFI/ubuntu setup
+	s.makeFakeGrubEFINativeEnv(c, []byte("grub.cfg"))
+
+	opts := []bootloader.Options{
+		{NoSlashBoot: true, PrepareImageTime: true},
+		{NoSlashBoot: true, PrepareImageTime: true, Role: bootloader.RoleRecovery}}
+	for _, opt := range opts {
+		g := bootloader.NewGrub(s.rootdir, &opt)
+		c.Assert(g, NotNil)
+
+		tab, ok := g.(bootloader.TrustedAssetsBootloader)
+		c.Assert(ok, Equals, true)
+
+		ta, err := tab.TrustedAssets()
+		c.Assert(err, ErrorMatches, "internal error: retrieving boot assets at prepare image time")
+		c.Check(ta, IsNil)
+	}
+}
+
 func (s *grubTestSuite) TestRecoveryBootChains(c *C) {
 	s.makeFakeGrubEFINativeEnv(c, nil)
 	g := bootloader.NewGrub(s.rootdir, &bootloader.Options{Role: bootloader.RoleRecovery})
@@ -1083,6 +1271,26 @@ func (s *grubTestSuite) TestBootChains(c *C) {
 	})
 }
 
+func (s *grubTestSuite) TestBootChainsArm64(c *C) {
+	s.makeFakeGrubEFINativeEnv(c, nil)
+	r := archtest.MockArchitecture("arm64")
+	defer r()
+	g := bootloader.NewGrub(s.rootdir, &bootloader.Options{Role: bootloader.RoleRecovery})
+	tab, ok := g.(bootloader.TrustedAssetsBootloader)
+	c.Assert(ok, Equals, true)
+
+	g2 := bootloader.NewGrub(s.rootdir, &bootloader.Options{Role: bootloader.RoleRunMode})
+
+	chain, err := tab.BootChain(g2, "kernel.snap")
+	c.Assert(err, IsNil)
+	c.Assert(chain, DeepEquals, []bootloader.BootFile{
+		{Path: "EFI/boot/bootaa64.efi", Role: bootloader.RoleRecovery},
+		{Path: "EFI/boot/grubaa64.efi", Role: bootloader.RoleRecovery},
+		{Path: "EFI/boot/grubaa64.efi", Role: bootloader.RoleRunMode},
+		{Snap: "kernel.snap", Path: "kernel.efi", Role: bootloader.RoleRunMode},
+	})
+}
+
 func (s *grubTestSuite) TestBootChainsNotRecoveryBootloader(c *C) {
 	s.makeFakeGrubEnv(c)
 	g := bootloader.NewGrub(s.rootdir, nil)
