@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2015-2016 Canonical Ltd
+ * Copyright (C) 2015-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -20,30 +20,35 @@
 package client
 
 import (
-	"encoding/json"
 	"errors"
 	"fmt"
 	"net/url"
 	"strings"
 	"time"
 
+	"golang.org/x/xerrors"
+
 	"github.com/snapcore/snapd/snap"
 )
 
 // Snap holds the data for a snap as obtained from snapd.
 type Snap struct {
-	ID               string        `json:"id"`
-	Title            string        `json:"title,omitempty"`
-	Summary          string        `json:"summary"`
-	Description      string        `json:"description"`
-	DownloadSize     int64         `json:"download-size,omitempty"`
-	Icon             string        `json:"icon,omitempty"`
-	InstalledSize    int64         `json:"installed-size,omitempty"`
-	InstallDate      time.Time     `json:"install-date,omitempty"`
-	Name             string        `json:"name"`
+	ID            string             `json:"id"`
+	Title         string             `json:"title,omitempty"`
+	Summary       string             `json:"summary"`
+	Description   string             `json:"description"`
+	DownloadSize  int64              `json:"download-size,omitempty"`
+	Icon          string             `json:"icon,omitempty"`
+	InstalledSize int64              `json:"installed-size,omitempty"`
+	InstallDate   *time.Time         `json:"install-date,omitempty"`
+	Name          string             `json:"name"`
+	Publisher     *snap.StoreAccount `json:"publisher,omitempty"`
+	StoreURL      string             `json:"store-url,omitempty"`
+	// Developer is also the publisher's username for historic reasons.
 	Developer        string        `json:"developer"`
 	Status           string        `json:"status"`
 	Type             string        `json:"type"`
+	Base             string        `json:"base,omitempty"`
 	Version          string        `json:"version"`
 	Channel          string        `json:"channel"`
 	TrackingChannel  string        `json:"tracking-channel,omitempty"`
@@ -56,38 +61,42 @@ type Snap struct {
 	TryMode          bool          `json:"trymode,omitempty"`
 	Apps             []AppInfo     `json:"apps,omitempty"`
 	Broken           string        `json:"broken,omitempty"`
-	Contact          string        `json:"contact"`
 	License          string        `json:"license,omitempty"`
+	CommonIDs        []string      `json:"common-ids,omitempty"`
+	MountedFrom      string        `json:"mounted-from,omitempty"`
+	CohortKey        string        `json:"cohort-key,omitempty"`
 
-	Prices      map[string]float64 `json:"prices,omitempty"`
-	Screenshots []Screenshot       `json:"screenshots,omitempty"`
+	Links map[string][]string `json:"links,omitempy"`
+
+	// legacy fields before we had links
+	Contact string `json:"contact"`
+	Website string `json:"website,omitempty"`
+
+	Prices      map[string]float64    `json:"prices,omitempty"`
+	Screenshots []snap.ScreenshotInfo `json:"screenshots,omitempty"`
+	Media       snap.MediaInfos       `json:"media,omitempty"`
+	Categories  []snap.CategoryInfo   `json:"categories,omitempty"`
 
 	// The flattended channel map with $track/$risk
 	Channels map[string]*snap.ChannelSnapInfo `json:"channels,omitempty"`
 
 	// The ordered list of tracks that contains channels
 	Tracks []string `json:"tracks,omitempty"`
+
+	Health *SnapHealth `json:"health,omitempty"`
+
+	// Hold is the time until which the snap's refreshes are held by the user.
+	Hold *time.Time `json:"hold,omitempty"`
+	// GatingHold is the time until which the snap's refreshes are held by a snap.
+	GatingHold *time.Time `json:"gating-hold,omitempty"`
 }
 
-func (s *Snap) MarshalJSON() ([]byte, error) {
-	type auxSnap Snap // use auxiliary type so that Go does not call Snap.MarshalJSON()
-	// separate type just for marshalling
-	m := struct {
-		auxSnap
-		InstallDate *time.Time `json:"install-date,omitempty"`
-	}{
-		auxSnap: auxSnap(*s),
-	}
-	if !s.InstallDate.IsZero() {
-		m.InstallDate = &s.InstallDate
-	}
-	return json.Marshal(&m)
-}
-
-type Screenshot struct {
-	URL    string `json:"url"`
-	Width  int64  `json:"width,omitempty"`
-	Height int64  `json:"height,omitempty"`
+type SnapHealth struct {
+	Revision  snap.Revision `json:"revision"`
+	Timestamp time.Time     `json:"timestamp"`
+	Status    string        `json:"status"`
+	Message   string        `json:"message,omitempty"`
+	Code      string        `json:"code,omitempty"`
 }
 
 // Statuses and types a snap may have.
@@ -117,11 +126,19 @@ type ResultInfo struct {
 // - Private: return snaps that are private
 // - Query: only return snaps that match the query string
 type FindOptions struct {
-	Refresh bool
-	Private bool
-	Prefix  bool
-	Query   string
+	// Query is a term to search by or a prefix (if Prefix is true)
+	Query  string
+	Prefix bool
+
+	CommonID string
+
+	Category string
+	// Section is deprecated, use Category instead.
 	Section string
+	Private bool
+	Scope   string
+
+	Refresh bool
 }
 
 var ErrNoSnapsInstalled = errors.New("no snaps installed")
@@ -130,6 +147,11 @@ type ListOptions struct {
 	All bool
 }
 
+// Information about a category
+type Category struct {
+	Name string `json:"name"`
+}
+
 // List returns the list of all snaps installed on the system
 // with names in the given list; if the list is empty, all snaps.
 func (client *Client) List(names []string, opts *ListOptions) ([]*Snap, error) {
@@ -158,15 +180,27 @@ func (client *Client) List(names []string, opts *ListOptions) ([]*Snap, error) {
 }
 
 // Sections returns the list of existing snap sections in the store
+// This is deprecated, use Categories() instead.
 func (client *Client) Sections() ([]string, error) {
 	var sections []string
 	_, err := client.doSync("GET", "/v2/sections", nil, nil, nil, &sections)
 	if err != nil {
-		return nil, fmt.Errorf("cannot get snap sections: %s", err)
+		fmt := "cannot get snap sections: %w"
+		return nil, xerrors.Errorf(fmt, err)
 	}
 	return sections, nil
 }
 
+// Categories returns the list of existing snap categories in the store
+func (client *Client) Categories() ([]*Category, error) {
+	var categories []*Category
+	_, err := client.doSync("GET", "/v2/categories", nil, nil, nil, &categories)
+	if err != nil {
+		return nil, fmt.Errorf("cannot get snap categories: %w", err)
+	}
+	return categories, nil
+}
+
 // Find returns a list of snaps available for install from the
 // store for this system and that match the query
 func (client *Client) Find(opts *FindOptions) ([]*Snap, *ResultInfo, error) {
@@ -178,8 +212,14 @@ func (client *Client) Find(opts *FindOptions) ([]*Snap, *ResultInfo, error) {
 	if opts.Prefix {
 		q.Set("name", opts.Query+"*")
 	} else {
-		q.Set("q", opts.Query)
+		if opts.CommonID != "" {
+			q.Set("common-id", opts.CommonID)
+		}
+		if opts.Query != "" {
+			q.Set("q", opts.Query)
+		}
 	}
+
 	switch {
 	case opts.Refresh && opts.Private:
 		return nil, nil, fmt.Errorf("cannot specify refresh and private together")
@@ -188,9 +228,15 @@ func (client *Client) Find(opts *FindOptions) ([]*Snap, *ResultInfo, error) {
 	case opts.Private:
 		q.Set("select", "private")
 	}
+	if opts.Category != "" {
+		q.Set("category", opts.Category)
+	}
 	if opts.Section != "" {
 		q.Set("section", opts.Section)
 	}
+	if opts.Scope != "" {
+		q.Set("scope", opts.Scope)
+	}
 
 	return client.snapsFromPath("/v2/find", q)
 }
@@ -201,7 +247,8 @@ func (client *Client) FindOne(name string) (*Snap, *ResultInfo, error) {
 
 	snaps, ri, err := client.snapsFromPath("/v2/find", q)
 	if err != nil {
-		return nil, nil, fmt.Errorf("cannot find snap %q: %s", name, err)
+		fmt := "cannot find snap %q: %w"
+		return nil, nil, xerrors.Errorf(fmt, name, err)
 	}
 
 	if len(snaps) == 0 {
@@ -214,8 +261,12 @@ func (client *Client) FindOne(name string) (*Snap, *ResultInfo, error) {
 func (client *Client) snapsFromPath(path string, query url.Values) ([]*Snap, *ResultInfo, error) {
 	var snaps []*Snap
 	ri, err := client.doSync("GET", path, query, nil, nil, &snaps)
+	if e, ok := err.(*Error); ok {
+		return nil, nil, e
+	}
 	if err != nil {
-		return nil, nil, fmt.Errorf("cannot list snaps: %s", err)
+		fmt := "cannot list snaps: %w"
+		return nil, nil, xerrors.Errorf(fmt, err)
 	}
 	return snaps, ri, nil
 }
@@ -227,7 +278,8 @@ func (client *Client) Snap(name string) (*Snap, *ResultInfo, error) {
 	path := fmt.Sprintf("/v2/snaps/%s", name)
 	ri, err := client.doSync("GET", path, nil, nil, nil, &snap)
 	if err != nil {
-		return nil, nil, fmt.Errorf("cannot retrieve snap %q: %s", name, err)
+		fmt := "cannot retrieve snap %q: %w"
+		return nil, nil, xerrors.Errorf(fmt, name, err)
 	}
 	return snap, ri, nil
 }
