@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016-2017 Canonical Ltd
+ * Copyright (C) 2016-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -57,6 +57,10 @@ type SnapOptions struct {
 	Purge            bool            `json:"purge,omitempty"`
 	Amend            bool            `json:"amend,omitempty"`
 	Transaction      TransactionType `json:"transaction,omitempty"`
+	QuotaGroupName   string          `json:"quota-group,omitempty"`
+	ValidationSets   []string        `json:"validation-sets,omitempty"`
+	Time             string          `json:"time,omitempty"`
+	HoldLevel        string          `json:"hold-level,omitempty"`
 
 	Users []string `json:"users,omitempty"`
 }
@@ -103,6 +107,11 @@ func (opts *SnapOptions) writeOptionFields(mw *multipart.Writer) error {
 			return err
 		}
 	}
+	if opts.QuotaGroupName != "" {
+		if err := mw.WriteField("quota-group", opts.QuotaGroupName); err != nil {
+			return err
+		}
+	}
 	return writeFields(mw, fields)
 }
 
@@ -114,11 +123,15 @@ type actionData struct {
 }
 
 type multiActionData struct {
-	Action        string          `json:"action"`
-	Snaps         []string        `json:"snaps,omitempty"`
-	Users         []string        `json:"users,omitempty"`
-	Transaction   TransactionType `json:"transaction,omitempty"`
-	IgnoreRunning bool            `json:"ignore-running,omitempty"`
+	Action         string          `json:"action"`
+	Snaps          []string        `json:"snaps,omitempty"`
+	Users          []string        `json:"users,omitempty"`
+	Transaction    TransactionType `json:"transaction,omitempty"`
+	IgnoreRunning  bool            `json:"ignore-running,omitempty"`
+	Purge          bool            `json:"purge,omitempty"`
+	ValidationSets []string        `json:"validation-sets,omitempty"`
+	Time           string          `json:"time,omitempty"`
+	HoldLevel      string          `json:"hold-level,omitempty"`
 }
 
 // Install adds the snap with the given name from the given channel (or
@@ -150,6 +163,22 @@ func (client *Client) RefreshMany(names []string, options *SnapOptions) (changeI
 	return client.doMultiSnapAction("refresh", names, options)
 }
 
+func (client *Client) HoldRefreshes(name string, options *SnapOptions) (changeID string, err error) {
+	return client.doSnapAction("hold", name, options)
+}
+
+func (client *Client) HoldRefreshesMany(names []string, options *SnapOptions) (changeID string, err error) {
+	return client.doMultiSnapAction("hold", names, options)
+}
+
+func (client *Client) UnholdRefreshes(name string, options *SnapOptions) (changeID string, err error) {
+	return client.doSnapAction("unhold", name, options)
+}
+
+func (client *Client) UnholdRefreshesMany(names []string, options *SnapOptions) (changeID string, err error) {
+	return client.doMultiSnapAction("unhold", names, options)
+}
+
 func (client *Client) Enable(name string, options *SnapOptions) (changeID string, err error) {
 	return client.doSnapAction("enable", name, options)
 }
@@ -225,6 +254,10 @@ func (client *Client) doMultiSnapActionFull(actionName string, snaps []string, o
 		action.Users = options.Users
 		action.Transaction = options.Transaction
 		action.IgnoreRunning = options.IgnoreRunning
+		action.Purge = options.Purge
+		action.ValidationSets = options.ValidationSets
+		action.Time = options.Time
+		action.HoldLevel = options.HoldLevel
 	}
 
 	data, err := json.Marshal(&action)
