@@ -39,22 +39,18 @@
 #include <unistd.h>
 
 #include "../libsnap-confine-private/cgroup-freezer-support.h"
+#include "../libsnap-confine-private/cgroup-support.h"
 #include "../libsnap-confine-private/classic.h"
 #include "../libsnap-confine-private/cleanup-funcs.h"
+#include "../libsnap-confine-private/feature.h"
+#include "../libsnap-confine-private/infofile.h"
 #include "../libsnap-confine-private/locking.h"
 #include "../libsnap-confine-private/mountinfo.h"
 #include "../libsnap-confine-private/string-utils.h"
 #include "../libsnap-confine-private/tool.h"
 #include "../libsnap-confine-private/utils.h"
 #include "user-support.h"
-
-/*!
- * The void directory.
- *
- * Snap confine moves to that directory in case it cannot retain the current
- * working directory across the pivot_root call.
- **/
-#define SC_VOID_DIR "/var/lib/snapd/void"
+#include "mount-support.h"
 
 /**
  * Directory where snap-confine keeps namespace files.
@@ -97,7 +93,7 @@ void sc_reassociate_with_pid1_mount_ns(void)
 	if (readlinkat(init_mnt_fd, "", init_buf, sizeof init_buf) < 0) {
 		if (errno == ENOENT) {
 			// According to namespaces(7) on a pre 3.8 kernel the namespace
-			// files are hardlinks, not sylinks. If that happens readlinkat
+			// files are hardlinks, not symlinks. If that happens readlinkat
 			// fails with ENOENT. As a quick workaround for this special-case
 			// functionality, just bail out and do nothing without raising an
 			// error.
@@ -124,16 +120,20 @@ void sc_reassociate_with_pid1_mount_ns(void)
 	}
 }
 
-void sc_initialize_mount_ns(void)
+void sc_initialize_mount_ns(unsigned int experimental_features)
 {
+	debug("unsharing snap namespace directory");
+
 	/* Ensure that /run/snapd/ns is a directory. */
+	sc_identity old = sc_set_effective_identity(sc_root_group_identity());
 	if (sc_nonfatal_mkpath(sc_ns_dir, 0755) < 0) {
 		die("cannot create directory %s", sc_ns_dir);
 	}
+	(void)sc_set_effective_identity(old);
 
 	/* Read and analyze the mount table. We need to see whether /run/snapd/ns
 	 * is a mount point with private event propagation. */
-	struct sc_mountinfo *info SC_CLEANUP(sc_cleanup_mountinfo) = NULL;
+	sc_mountinfo *info SC_CLEANUP(sc_cleanup_mountinfo) = NULL;
 	info = sc_parse_mountinfo(NULL);
 	if (info == NULL) {
 		die("cannot parse /proc/self/mountinfo");
@@ -141,7 +141,7 @@ void sc_initialize_mount_ns(void)
 
 	bool is_mnt = false;
 	bool is_private = false;
-	for (struct sc_mountinfo_entry * entry = sc_first_mountinfo_entry(info);
+	for (sc_mountinfo_entry * entry = sc_first_mountinfo_entry(info);
 	     entry != NULL; entry = sc_next_mountinfo_entry(entry)) {
 		/* Find /run/snapd/ns */
 		if (!sc_streq(entry->mount_dir, sc_ns_dir)) {
@@ -167,6 +167,14 @@ void sc_initialize_mount_ns(void)
 			die("cannot change propagation type to MS_PRIVATE in %s", sc_ns_dir);
 		}
 	}
+
+	/* code that follows is experimental */
+	if (experimental_features & SC_FEATURE_PARALLEL_INSTANCES) {
+		// Ensure that SNAP_MOUNT_DIR and /var/snap are shared mount points
+		debug
+		    ("(experimental) ensuring snap mount and data directories are mount points");
+		sc_ensure_snap_dir_shared_mounts();
+	}
 }
 
 struct sc_mount_ns {
@@ -239,13 +247,13 @@ static dev_t find_base_snap_device(const char *base_snap_name,
 	sc_must_snprintf(base_squashfs_path,
 			 sizeof base_squashfs_path, "%s/%s/%s",
 			 SNAP_MOUNT_DIR, base_snap_name, base_snap_rev);
-	struct sc_mountinfo *mi SC_CLEANUP(sc_cleanup_mountinfo) = NULL;
+	sc_mountinfo *mi SC_CLEANUP(sc_cleanup_mountinfo) = NULL;
 	mi = sc_parse_mountinfo(NULL);
 	if (mi == NULL) {
 		die("cannot parse mountinfo of the current process");
 	}
 	bool found = false;
-	for (struct sc_mountinfo_entry * mie =
+	for (sc_mountinfo_entry * mie =
 	     sc_first_mountinfo_entry(mi); mie != NULL;
 	     mie = sc_next_mountinfo_entry(mie)) {
 		if (sc_streq(mie->mount_dir, base_squashfs_path)) {
@@ -265,74 +273,197 @@ static dev_t find_base_snap_device(const char *base_snap_name,
 	return base_snap_dev;
 }
 
-static bool should_discard_current_ns(dev_t base_snap_dev)
+static bool base_snap_device_changed(sc_mountinfo *mi, dev_t base_snap_dev)
 {
-	// Inspect the namespace and check if we should discard it.
-	//
-	// The namespace may become "stale" when the rootfs is not the same
-	// device we found above. This will happen whenever the base snap is
-	// refreshed since the namespace was first created.
-	struct sc_mountinfo_entry *mie;
-	struct sc_mountinfo *mi SC_CLEANUP(sc_cleanup_mountinfo) = NULL;
+	sc_mountinfo_entry *mie;
+
+	/* We are looking for a mount entry matching the device ID of the base
+	 * snap. We need to take these cases into account:
+	 * 1) In the typical case, this will be mounted on the "/" directory.
+	 * 2) If the root directory is a tmpfs, the base snap would be mounted
+	 *    under /usr.
+	 * 3) If the snap has a layout that adds directories or files directly
+	 *    under /usr, a writable mimic will be created: /usr will be a tmpfs,
+	 *    with all of the original directory entries inside of /usr being
+	 *    bind-mounted onto mount-points created into the tmpfs.
+	 * In light of the above, we do ignore all tmpfs entries and accept that
+	 * our base snap might be mounted under /, /usr, or anywhere under /usr.
+	 */
+	for (mie = sc_first_mountinfo_entry(mi); mie != NULL;
+	     mie = sc_next_mountinfo_entry(mie)) {
+		if (sc_streq(mie->fs_type, "tmpfs")) {
+			continue;
+		}
+
+		if (base_snap_dev == makedev(mie->dev_major, mie->dev_minor) &&
+		    (sc_streq(mie->mount_dir, "/") ||
+		     sc_streq(mie->mount_dir, "/usr") ||
+		     sc_startswith(mie->mount_dir, "/usr/"))) {
+			debug("found base snap device %d:%d on %s",
+			      mie->dev_major, mie->dev_minor, mie->mount_dir);
+			return false;
+		}
+	}
+	debug("base snap device %d:%d not found in existing mount ns",
+	      major(base_snap_dev), minor(base_snap_dev));
+	return true;
+}
+
+static bool homedirs_are_mounted(sc_mountinfo *mi, char **homedirs, int num_homedirs)
+{
+	if (num_homedirs == 0) {
+		return true;
+	}
+
+	/* We know that the number of homedirs is not going to be huge, so let's
+	 * just allocare this vector on the stack */
+	bool homedir_seen[num_homedirs];
+	for (int i = 0; i < num_homedirs; i++) {
+		homedir_seen[i] = false;
+	}
+
+	sc_mountinfo_entry *mie;
+	for (mie = sc_first_mountinfo_entry(mi); mie != NULL;
+	     mie = sc_next_mountinfo_entry(mie)) {
+		for (int i = 0; i < num_homedirs; i++) {
+			if (sc_streq(mie->mount_dir, homedirs[i])) {
+				homedir_seen[i] = true;
+			}
+		}
+	}
+
+	bool all_seen = true;
+	for (int i = 0; i < num_homedirs; i++) {
+		if (!homedir_seen[i]) {
+			debug("Homedir %s missing from namespace", homedirs[i]);
+			all_seen = false;
+			break;
+		}
+	}
+	return all_seen;
+}
+
+// Inspect the namespace and check if we should discard it.
+static bool should_discard_current_ns(const struct sc_invocation *inv,
+                                      dev_t base_snap_dev)
+{
+	sc_mountinfo *mi SC_CLEANUP(sc_cleanup_mountinfo) = NULL;
 
 	mi = sc_parse_mountinfo(NULL);
 	if (mi == NULL) {
 		die("cannot parse mountinfo of the current process");
 	}
-	for (mie = sc_first_mountinfo_entry(mi); mie != NULL;
-	     mie = sc_next_mountinfo_entry(mie)) {
-		if (!sc_streq(mie->mount_dir, "/")) {
-			continue;
-		}
-		// NOTE: we want the initial rootfs just in case overmount
-		// was used to do something weird. The initial rootfs was
-		// set up by snap-confine and that is the one we want to
-		// measure.
-		debug("block device of the root filesystem is %d:%d",
-		      mie->dev_major, mie->dev_minor);
-		return base_snap_dev != makedev(mie->dev_major, mie->dev_minor);
+
+	// The namespace may become "stale" when the rootfs is not the same
+	// device we found above. This will happen whenever the base snap is
+	// refreshed since the namespace was first created.
+	if (base_snap_device_changed(mi, base_snap_dev)) {
+		return true;
 	}
-	die("cannot find mount entry of the root filesystem");
+
+	// Another reason for becoming stale is if the homedirs configuration has
+	// changed: so this code will check that all homedirs are mounted in the
+	// namespace.
+	if (!homedirs_are_mounted(mi, inv->homedirs, inv->num_homedirs)) {
+		return true;
+	}
+
+	return false;
 }
 
 enum sc_discard_vote {
+	/**
+	 * SC_DISCARD_NO denotes that the mount namespace doesn't have to be
+	 * discarded. This happens when the base snap has not changed.
+	 **/
 	SC_DISCARD_NO = 1,
-	SC_DISCARD_YES = 2,
+	/**
+	 * SC_DISCARD_SHOULD indicates that the mount namespace should be discarded
+	 * but may be reused if it is still inhabited by processes. This only
+	 * happens when the base snap revision changes but the name of the base
+	 * snap is the same as before.
+	 **/
+	SC_DISCARD_SHOULD = 2,
+	/**
+	 * SC_DISCARD_MUST indicates that the mount namespace must be discarded
+	 * even if it still inhabited by processes. This only happens when the name
+	 * of the base snap changes.
+	 **/
+	SC_DISCARD_MUST = 3,
 };
 
+/**
+ * is_base_transition returns true if a base transition is occurring.
+ *
+ * The function inspects /run/snapd/ns/snap.$SNAP_INSTANCE_NAME.info as well
+ * as the invocation parameters of snap-confine. If the base snap name, as
+ * encoded in the info file and as described by the invocation parameters
+ * differ then a base transition is occurring. If the info file is absent or
+ * does not record the name of the base snap then transition cannot be
+ * detected.
+**/
+static bool is_base_transition(const sc_invocation * inv)
+{
+	char info_path[PATH_MAX] = { 0 };
+	sc_must_snprintf(info_path,
+			 sizeof info_path,
+			 "/run/snapd/ns/snap.%s.info", inv->snap_instance);
+
+	FILE *stream SC_CLEANUP(sc_cleanup_file) = NULL;
+	stream = fopen(info_path, "r");
+	if (stream == NULL && errno == ENOENT) {
+		// If the info file is absent then we cannot decide if a transition had
+		// occurred. For people upgrading from snap-confine without the info
+		// file, that is the best we can do.
+		return false;
+	}
+	if (stream == NULL) {
+		die("cannot open %s", info_path);
+	}
+
+	char *base_snap_name SC_CLEANUP(sc_cleanup_string) = NULL;
+	sc_error *err = NULL;
+	if (sc_infofile_get_key
+	    (stream, "base-snap-name", &base_snap_name, &err) < 0) {
+		sc_die_on_error(err);
+	}
+
+	if (base_snap_name == NULL) {
+		// If the info file doesn't record the name of the base snap then,
+		// again, we cannot decide if a transition had occurred.
+		return false;
+	}
+
+	return !sc_streq(inv->orig_base_snap_name, base_snap_name);
+}
+
+static bool sc_is_mount_ns_in_use(const char *snap_instance);
+
 // The namespace may be stale. To check this we must actually switch into it
 // but then we use up our setns call (the kernel misbehaves if we setns twice).
 // To work around this we'll fork a child and use it to probe. The child will
 // inspect the namespace and send information back via eventfd and then exit
 // unconditionally.
 static int sc_inspect_and_maybe_discard_stale_ns(int mnt_fd,
-						 const char *snap_name,
-						 const char *base_snap_name,
+						 const sc_invocation * inv,
 						 int snap_discard_ns_fd)
 {
 	char base_snap_rev[PATH_MAX] = { 0 };
-	char fname[PATH_MAX] = { 0 };
 	dev_t base_snap_dev;
 	int event_fd SC_CLEANUP(sc_cleanup_close) = -1;
 
 	// Read the revision of the base snap by looking at the current symlink.
-	sc_must_snprintf(fname, sizeof fname, "%s/%s/current",
-			 SNAP_MOUNT_DIR, base_snap_name);
-	if (readlink(fname, base_snap_rev, sizeof base_snap_rev) < 0) {
-		die("cannot read current revision of snap %s", snap_name);
+	if (readlink(inv->rootfs_dir, base_snap_rev, sizeof base_snap_rev) < 0) {
+		die("cannot read current revision of snap %s",
+		    inv->snap_instance);
 	}
 	if (base_snap_rev[sizeof base_snap_rev - 1] != '\0') {
 		die("cannot read current revision of snap %s: value too long",
-		    snap_name);
+		    inv->snap_instance);
 	}
 	// Find the device that is backing the current revision of the base snap.
-	base_snap_dev = find_base_snap_device(base_snap_name, base_snap_rev);
-
-	// Check if we are running in normal mode with pivot root. Do this here
-	// because once on the inside of the transformed mount namespace we can no
-	// longer tell.
-	bool is_normal_mode =
-	    sc_should_use_normal_mode(sc_classify_distro(), base_snap_name);
+	base_snap_dev =
+	    find_base_snap_device(inv->base_snap_name, base_snap_rev);
 
 	// Store the PID of this process. This is done instead of calls to
 	// getppid() below because then we can reliably track the PID of the
@@ -381,21 +512,32 @@ static int sc_inspect_and_maybe_discard_stale_ns(int mnt_fd,
 			die("cannot join preserved mount namespace");
 		}
 		// Check if the namespace needs to be discarded.
-		//
+		eventfd_t value = SC_DISCARD_NO;
+		const char *value_str = "no";
+
 		// TODO: enable this for core distributions. This is complex because on
 		// core the rootfs is mounted in initrd and is _not_ changed (no
 		// pivot_root) and the base snap is again mounted (2nd time) by
 		// systemd. This makes us end up in a situation where the outer base
 		// snap will never match the rootfs inside the mount namespace.
-		bool should_discard =
-		    is_normal_mode ? should_discard_current_ns(base_snap_dev) :
-		    false;
-
-		// Send this back to the parent: 2 - discard, 1 - keep.
+		if (inv->is_normal_mode
+		    && should_discard_current_ns(inv, base_snap_dev)) {
+			value = SC_DISCARD_SHOULD;
+			value_str = "should";
+		}
+		// If the base snap changed, we must discard the mount namespace and
+		// start over to allow the newly started process to see the requested
+		// base snap. Due to the TODO above always perform explicit transition
+		// check to protect against LP:#1819875 and LP:#1861901
+		if (is_base_transition(inv)) {
+			// The base snap has changed. We must discard ...
+			value = SC_DISCARD_MUST;
+			value_str = "must";
+		}
+		// Send this back to the parent: 3 - force discard 2 - prefer discard, 1 - keep.
 		// Note that we cannot just use 0 and 1 because of the semantics of eventfd(2).
-		if (eventfd_write(event_fd, should_discard ?
-				  SC_DISCARD_YES : SC_DISCARD_NO) < 0) {
-			die("cannot send information to %s preserved mount namespace", should_discard ? "discard" : "keep");
+		if (eventfd_write(event_fd, value) < 0) {
+			die("cannot send information to %s preserved mount namespace", value_str);
 		}
 		// Exit, we're done.
 		exit(0);
@@ -422,20 +564,26 @@ static int sc_inspect_and_maybe_discard_stale_ns(int mnt_fd,
 		die("support process for mount namespace inspection exited abnormally");
 	}
 	// If the namespace is up-to-date then we are done.
-	if (value == SC_DISCARD_NO) {
-		debug("preserved mount namespace can be reused");
+	switch (value) {
+	case SC_DISCARD_NO:
+		debug("preserved mount is not stale, reusing");
 		return 0;
+	case SC_DISCARD_SHOULD:
+		if (sc_is_mount_ns_in_use(inv->snap_instance)) {
+			// Some processes are still using the namespace so we cannot discard it
+			// as that would fracture the view that the set of processes inside
+			// have on what is mounted.
+			debug
+			    ("preserved mount namespace is stale but occupied, reusing");
+			return 0;
+		}
+		break;
+	case SC_DISCARD_MUST:
+		debug
+		    ("preserved mount namespace is stale and base snap has changed, discarding");
+		break;
 	}
-	// The namespace is stale, let's check if we can discard it.
-	if (sc_cgroup_freezer_occupied(snap_name)) {
-		// Some processes are still using the namespace so we cannot discard it
-		// as that would fracture the view that the set of processes inside
-		// have on what is mounted.
-		debug("preserved mount namespace is stale but occupied");
-		return 0;
-	}
-	// The namespace is both stale and empty. We can discard it now.
-	sc_call_snap_discard_ns(snap_discard_ns_fd, snap_name);
+	sc_call_snap_discard_ns(snap_discard_ns_fd, inv->snap_instance);
 	return EAGAIN;
 }
 
@@ -447,8 +595,8 @@ static void helper_capture_ns(struct sc_mount_ns *group, pid_t parent);
 static void helper_capture_per_user_ns(struct sc_mount_ns *group, pid_t parent);
 
 int sc_join_preserved_ns(struct sc_mount_ns *group, struct sc_apparmor
-			 *apparmor, const char *base_snap_name,
-			 const char *snap_name, int snap_discard_ns_fd)
+			 *apparmor, const sc_invocation * inv,
+			 int snap_discard_ns_fd)
 {
 	// Open the mount namespace file.
 	char mnt_fname[PATH_MAX] = { 0 };
@@ -457,7 +605,10 @@ int sc_join_preserved_ns(struct sc_mount_ns *group, struct sc_apparmor
 	// NOTE: There is no O_EXCL here because the file can be around but
 	// doesn't have to be a mounted namespace.
 	mnt_fd = openat(group->dir_fd, mnt_fname,
-			O_CREAT | O_RDONLY | O_CLOEXEC | O_NOFOLLOW, 0600);
+			O_RDONLY | O_CLOEXEC | O_NOFOLLOW, 0600);
+	if (mnt_fd < 0 && errno == ENOENT) {
+		return ESRCH;
+	}
 	if (mnt_fd < 0) {
 		die("cannot open preserved mount namespace %s", group->name);
 	}
@@ -486,32 +637,15 @@ int sc_join_preserved_ns(struct sc_mount_ns *group, struct sc_apparmor
 
 		// Inspect and perhaps discard the preserved mount namespace.
 		if (sc_inspect_and_maybe_discard_stale_ns
-		    (mnt_fd, snap_name, base_snap_name,
-		     snap_discard_ns_fd) == EAGAIN) {
+		    (mnt_fd, inv, snap_discard_ns_fd) == EAGAIN) {
 			return ESRCH;
 		}
-		// Remember the vanilla working directory so that we may attempt to restore it later.
-		char *vanilla_cwd SC_CLEANUP(sc_cleanup_string) = NULL;
-		vanilla_cwd = get_current_dir_name();
-		if (vanilla_cwd == NULL) {
-			die("cannot get the current working directory");
-		}
 		// Move to the mount namespace of the snap we're trying to start.
 		if (setns(mnt_fd, CLONE_NEWNS) < 0) {
 			die("cannot join preserved mount namespace %s",
 			    group->name);
 		}
 		debug("joined preserved mount namespace %s", group->name);
-
-		// Try to re-locate back to vanilla working directory. This can fail
-		// because that directory is no longer present.
-		if (chdir(vanilla_cwd) != 0) {
-			debug("cannot enter %s, moving to void", vanilla_cwd);
-			if (chdir(SC_VOID_DIR) != 0) {
-				die("cannot change directory to %s",
-				    SC_VOID_DIR);
-			}
-		}
 		return 0;
 	}
 	return ESRCH;
@@ -527,7 +661,10 @@ int sc_join_preserved_per_user_ns(struct sc_mount_ns *group,
 
 	int mnt_fd SC_CLEANUP(sc_cleanup_close) = -1;
 	mnt_fd = openat(group->dir_fd, mnt_fname,
-			O_CREAT | O_RDONLY | O_CLOEXEC | O_NOFOLLOW, 0600);
+			O_RDONLY | O_CLOEXEC | O_NOFOLLOW, 0600);
+	if (mnt_fd < 0 && errno == ENOENT) {
+		return ESRCH;
+	}
 	if (mnt_fd < 0) {
 		die("cannot open preserved mount namespace %s", group->name);
 	}
@@ -545,24 +682,11 @@ int sc_join_preserved_per_user_ns(struct sc_mount_ns *group,
 #endif
 	if (ns_statfs_buf.f_type == NSFS_MAGIC
 	    || ns_statfs_buf.f_type == PROC_SUPER_MAGIC) {
-		// TODO: refactor the cwd workflow across all of snap-confine.
-		char *vanilla_cwd SC_CLEANUP(sc_cleanup_string) = NULL;
-		vanilla_cwd = get_current_dir_name();
-		if (vanilla_cwd == NULL) {
-			die("cannot get the current working directory");
-		}
 		if (setns(mnt_fd, CLONE_NEWNS) < 0) {
 			die("cannot join preserved per-user mount namespace %s",
 			    group->name);
 		}
 		debug("joined preserved mount namespace %s", group->name);
-		if (chdir(vanilla_cwd) != 0) {
-			debug("cannot enter %s, moving to void", vanilla_cwd);
-			if (chdir(SC_VOID_DIR) != 0) {
-				die("cannot change directory to %s",
-				    SC_VOID_DIR);
-			}
-		}
 		return 0;
 	}
 	return ESRCH;
@@ -650,8 +774,24 @@ static void helper_main(struct sc_mount_ns *group, struct sc_apparmor *apparmor,
 	if (kill(parent, 0) < 0) {
 		switch (errno) {
 		case ESRCH:
+			// When snap-confine executes it will fork a helper process. That
+			// process establishes an elaborate dance to ensure both itself and
+			// the parent are operating exactly as specified, so that no
+			// processes are left behind for unbound amount of time. As a part
+			// of that dance the child pings the parent to ensure it is still
+			// alive after establishing a notification signal to be sent in
+			// case the parent dies. This is a race avoidance mechanism, we set
+			// up the notification and then check if the parent is alive by the
+			// time we are done.
+			//
+			// In the case when the parent does go away we used to call
+			// abort(). On some distributions this would trigger an unclean
+			// process termination error report to be sent. One such example is
+			// the Ubuntu error tracker. Since the parent process can be
+			// legitimately interrupted and killed, this should not generate an
+			// error report. As such, perform clean exit in this specific case.
 			debug("parent process has terminated");
-			abort();
+			exit(0);
 		default:
 			die("cannot confirm that parent process is alive");
 			break;
@@ -710,6 +850,7 @@ static void helper_capture_ns(struct sc_mount_ns *group, pid_t parent)
 		die("cannot create file %s", dst);
 	}
 	close(fd);
+
 	if (mount(src, dst, NULL, MS_BIND, NULL) < 0) {
 		die("cannot preserve mount namespace of process %d as %s",
 		    (int)parent, dst);
@@ -727,6 +868,14 @@ static void helper_capture_per_user_ns(struct sc_mount_ns *group, pid_t parent)
 	debug("capturing per-snap, per-user mount namespace");
 	sc_must_snprintf(src, sizeof src, "/proc/%d/ns/mnt", (int)parent);
 	sc_must_snprintf(dst, sizeof dst, "%s.%d.mnt", group->name, (int)uid);
+
+	/* Ensure the bind mount destination exists. */
+	int fd = open(dst, O_CREAT | O_CLOEXEC | O_NOFOLLOW | O_RDONLY, 0600);
+	if (fd < 0) {
+		die("cannot create file %s", dst);
+	}
+	close(fd);
+
 	if (mount(src, dst, NULL, MS_BIND, NULL) < 0) {
 		die("cannot preserve per-user mount namespace of process %d as %s", (int)parent, dst);
 	}
@@ -800,3 +949,49 @@ void sc_wait_for_helper(struct sc_mount_ns *group)
 	sc_message_capture_helper(group, HELPER_CMD_EXIT);
 	sc_wait_for_capture_helper(group);
 }
+
+void sc_store_ns_info(const sc_invocation * inv)
+{
+	FILE *stream SC_CLEANUP(sc_cleanup_file) = NULL;
+	char info_path[PATH_MAX] = { 0 };
+	sc_must_snprintf(info_path, sizeof info_path,
+			 "/run/snapd/ns/snap.%s.info", inv->snap_instance);
+	int fd = -1;
+	fd = open(info_path,
+		  O_WRONLY | O_CREAT | O_TRUNC | O_CLOEXEC | O_NOFOLLOW, 0644);
+	if (fd < 0) {
+		die("cannot open %s", info_path);
+	}
+	if (fchown(fd, 0, 0) < 0) {
+		die("cannot chown %s to root.root", info_path);
+	}
+	// The stream now owns the file descriptor.
+	stream = fdopen(fd, "w");
+	if (stream == NULL) {
+		die("cannot get stream from file descriptor");
+	}
+	fprintf(stream, "base-snap-name=%s\n", inv->orig_base_snap_name);
+	if (ferror(stream) != 0) {
+		die("I/O error when writing to %s", info_path);
+	}
+	if (fflush(stream) == EOF) {
+		die("cannot flush %s", info_path);
+	}
+	debug("saved mount namespace meta-data to %s", info_path);
+}
+
+bool sc_is_mount_ns_in_use(const char *snap_instance)
+{
+	// perform an indirect check of whether the mount namespace is occupied,
+	// with cgroups v1, each snap process is attached to a group under the
+	// freezer controller, however with cgroups v2, we must check for any groups
+	// tracking the snap
+	bool occupied = false;
+	if (sc_cgroup_is_v2()) {
+		// cgroup v2 must consult the tracking groups
+		occupied = sc_cgroup_v2_is_tracking_snap(snap_instance);
+	} else {
+		occupied = sc_cgroup_freezer_occupied(snap_instance);
+	}
+	return occupied;
+}
