@@ -25,16 +25,19 @@ import (
 	"os"
 	"os/exec"
 	"path/filepath"
+	"strings"
+	"syscall"
 	"testing"
 
 	. "gopkg.in/check.v1"
 
+	snapExec "github.com/snapcore/snapd/cmd/snap-exec"
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/snaptest"
 	"github.com/snapcore/snapd/testutil"
-
-	snapExec "github.com/snapcore/snapd/cmd/snap-exec"
 )
 
 // Hook up check.v1 into the "go test" runner
@@ -61,10 +64,12 @@ apps:
   command: run-app cmd-arg1 $SNAP_DATA
   stop-command: stop-app
   post-stop-command: post-stop-app
+  completer: you/complete/me
   environment:
    BASE_PATH: /some/path
    LD_LIBRARY_PATH: ${BASE_PATH}/lib
    MY_PATH: $PATH
+   TEST_PATH: /custom
  app2:
   command: run-app2
   stop-command: stop-app2
@@ -74,6 +79,8 @@ apps:
   command: nostop
 `)
 
+var mockClassicYaml = append([]byte("confinement: classic\n"), mockYaml...)
+
 var mockHookYaml = []byte(`name: snapname
 version: 1.0
 hooks:
@@ -157,6 +164,12 @@ func (s *snapExecSuite) TestSnapExecAppIntegration(c *C) {
 	})
 	defer restore()
 
+	// FIXME: TEST_PATH was meant to be just PATH but this uncovers another
+	// bug in the test suite where mocking binaries misbehaves.
+	oldPath := os.Getenv("TEST_PATH")
+	os.Setenv("TEST_PATH", "/vanilla")
+	defer os.Setenv("TEST_PATH", oldPath)
+
 	// launch and verify its run the right way
 	err := snapExec.ExecApp("snapname.app", "42", "stop", []string{"arg1", "arg2"})
 	c.Assert(err, IsNil)
@@ -165,6 +178,53 @@ func (s *snapExecSuite) TestSnapExecAppIntegration(c *C) {
 	c.Check(execEnv, testutil.Contains, "BASE_PATH=/some/path")
 	c.Check(execEnv, testutil.Contains, "LD_LIBRARY_PATH=/some/path/lib")
 	c.Check(execEnv, testutil.Contains, fmt.Sprintf("MY_PATH=%s", os.Getenv("PATH")))
+	// TEST_PATH is properly handled and we only see one value, /custom, defined
+	// as an app-specific override.
+	// See also https://bugs.launchpad.net/snapd/+bug/1860369
+	c.Check(execEnv, Not(testutil.Contains), "TEST_PATH=/vanilla")
+	c.Check(execEnv, testutil.Contains, "TEST_PATH=/custom")
+
+	// ensure that CUPS_SERVER is absent since we didn't mock the /var/cups dir
+	c.Check(execEnv, Not(testutil.Contains), "CUPS_SERVER=/var/cups")
+}
+
+func (s *snapExecSuite) TestSnapExecAppIntegrationCupsServerWorkaround(c *C) {
+	dir := c.MkDir()
+	dirs.SetRootDir(dir)
+	snaptest.MockSnap(c, string(mockYaml), &snap.SideInfo{
+		Revision: snap.R("42"),
+	})
+
+	// mock the /var/cups dir is a bind-mount
+	restore := snapExec.MockSyscallStat(func(p string, st *syscall.Stat_t) error {
+		if strings.HasSuffix(p, "/var/cups/") {
+			st.Dev = 2
+		} else {
+			st.Dev = 1
+		}
+		return nil
+	})
+	defer restore()
+
+	execArgv0 := ""
+	execArgs := []string{}
+	execEnv := []string{}
+	restore = snapExec.MockSyscallExec(func(argv0 string, argv []string, env []string) error {
+		execArgv0 = argv0
+		execArgs = argv
+		execEnv = env
+		return nil
+	})
+	defer restore()
+
+	// launch and verify its run the right way
+	err := snapExec.ExecApp("snapname.app", "42", "stop", []string{"arg1", "arg2"})
+	c.Assert(err, IsNil)
+	c.Check(execArgv0, Equals, fmt.Sprintf("%s/snapname/42/stop-app", dirs.SnapMountDir))
+	c.Check(execArgs, DeepEquals, []string{execArgv0, "arg1", "arg2"})
+
+	// ensure that CUPS_SERVER is now set since we did mock the /var/cups dir
+	c.Check(execEnv, testutil.Contains, "CUPS_SERVER=/var/cups/cups.sock")
 }
 
 func (s *snapExecSuite) TestSnapExecAppCommandChainIntegration(c *C) {
@@ -461,7 +521,146 @@ func (s *snapExecSuite) TestSnapExecExpandEnvCmdArgs(c *C) {
 			expected: []string{"foo", "bar", "baz"},
 		},
 	} {
-		c.Check(snapExec.ExpandEnvCmdArgs(t.args, t.env), DeepEquals, t.expected)
-
+		env := osutil.Environment(t.env)
+		c.Check(snapExec.ExpandEnvCmdArgs(t.args, env), DeepEquals, t.expected)
 	}
 }
+
+func (s *snapExecSuite) TestSnapExecCompleteError(c *C) {
+	dirs.SetRootDir(c.MkDir())
+	snaptest.MockSnap(c, string(mockYaml), &snap.SideInfo{
+		Revision: snap.R("42"),
+	})
+
+	restore := snapExec.MockOsReadlink(func(p string) (string, error) {
+		c.Assert(p, Equals, "/proc/self/exe")
+		return "", fmt.Errorf("fail")
+	})
+	defer restore()
+
+	// setup env
+	os.Setenv("SNAP_DATA", "/var/snap/snapname/42")
+	defer os.Unsetenv("SNAP_DATA")
+
+	// launch and verify its run the right way
+	err := snapExec.ExecApp("snapname.app", "42", "complete", []string{"foo"})
+	c.Assert(err, ErrorMatches, "cannot find completion helper: fail")
+}
+
+func (s *snapExecSuite) TestSnapExecCompleteConfined(c *C) {
+	dirs.SetRootDir(c.MkDir())
+	snaptest.MockSnap(c, string(mockYaml), &snap.SideInfo{
+		Revision: snap.R("42"),
+	})
+
+	execArgv0 := ""
+	execArgs := []string{}
+	restore := snapExec.MockSyscallExec(func(argv0 string, argv []string, env []string) error {
+		execArgv0 = argv0
+		execArgs = argv
+		return nil
+	})
+	defer restore()
+
+	restore = snapExec.MockOsReadlink(func(p string) (string, error) {
+		c.Assert(p, Equals, "/proc/self/exe")
+		// as if running inside the snap mount namespace
+		return "/usr/lib/snapd/snap-exec", nil
+	})
+	defer restore()
+
+	// setup env
+	os.Setenv("SNAP_DATA", "/var/snap/snapname/42")
+	defer os.Unsetenv("SNAP_DATA")
+
+	// launch and verify its run the right way
+	err := snapExec.ExecApp("snapname.app", "42", "complete", []string{"foo"})
+	c.Assert(err, IsNil)
+	c.Check(execArgv0, Equals, "/bin/bash")
+	c.Check(execArgs, DeepEquals, []string{execArgv0,
+		dirs.CompletionHelperInCore,
+		filepath.Join(dirs.SnapMountDir, "snapname/42/you/complete/me"),
+		"foo"})
+}
+
+func (s *snapExecSuite) TestSnapExecCompleteClassicReexec(c *C) {
+	restore := release.MockReleaseInfo(&release.OS{ID: "ubuntu"})
+	defer restore()
+	dirs.SetRootDir(c.MkDir())
+	snaptest.MockSnap(c, string(mockClassicYaml), &snap.SideInfo{
+		Revision: snap.R("42"),
+	})
+
+	execArgv0 := ""
+	execArgs := []string{}
+	restore = snapExec.MockSyscallExec(func(argv0 string, argv []string, env []string) error {
+		execArgv0 = argv0
+		execArgs = argv
+		return nil
+	})
+	defer restore()
+
+	restore = snapExec.MockOsReadlink(func(p string) (string, error) {
+		c.Assert(p, Equals, "/proc/self/exe")
+		// as if it's reexeced from the snap
+		return filepath.Join(dirs.SnapMountDir, "core/current", dirs.CoreLibExecDir, "snap-exec"), nil
+	})
+	defer restore()
+
+	// setup env
+	os.Setenv("SNAP_DATA", "/var/snap/snapname/42")
+	defer os.Unsetenv("SNAP_DATA")
+
+	// launch and verify its run the right way
+	err := snapExec.ExecApp("snapname.app", "42", "complete", []string{"foo"})
+	c.Assert(err, IsNil)
+	c.Check(execArgv0, Equals, "/bin/bash")
+	c.Check(execArgs, DeepEquals, []string{execArgv0,
+		filepath.Join(dirs.SnapMountDir, "core/current", dirs.CompletionHelperInCore),
+		filepath.Join(dirs.SnapMountDir, "snapname/42/you/complete/me"),
+		"foo"})
+}
+
+func (s *snapExecSuite) TestSnapExecCompleteClassicNoReexec(c *C) {
+	restore := release.MockReleaseInfo(&release.OS{ID: "centos"})
+	defer restore()
+	dirs.SetRootDir(c.MkDir())
+	snaptest.MockSnap(c, string(mockClassicYaml), &snap.SideInfo{
+		Revision: snap.R("42"),
+	})
+
+	execArgv0 := ""
+	execArgs := []string{}
+	execEnv := []string{}
+	restore = snapExec.MockSyscallExec(func(argv0 string, argv []string, env []string) error {
+		execArgv0 = argv0
+		execArgs = argv
+		execEnv = env
+		return nil
+	})
+	defer restore()
+
+	restore = snapExec.MockOsReadlink(func(p string) (string, error) {
+		c.Assert(p, Equals, "/proc/self/exe")
+		// running from distro libexecdir
+		return filepath.Join(dirs.DistroLibExecDir, "snap-exec"), nil
+	})
+	defer restore()
+
+	// setup env
+	os.Setenv("SNAP_DATA", "/var/snap/snapname/42")
+	defer os.Unsetenv("SNAP_DATA")
+	os.Setenv("SNAP_SAVED_TMPDIR", "/var/tmp99")
+	defer os.Unsetenv("SNAP_SAVED_TMPDIR")
+
+	// launch and verify its run the right way
+	err := snapExec.ExecApp("snapname.app", "42", "complete", []string{"foo"})
+	c.Assert(err, IsNil)
+	c.Check(execArgv0, Equals, "/bin/bash")
+	c.Check(execArgs, DeepEquals, []string{execArgv0,
+		filepath.Join(dirs.DistroLibExecDir, "etelpmoc.sh"),
+		filepath.Join(dirs.SnapMountDir, "snapname/42/you/complete/me"),
+		"foo"})
+	c.Check(execEnv, testutil.Contains, "SNAP_DATA=/var/snap/snapname/42")
+	c.Check(execEnv, testutil.Contains, "TMPDIR=/var/tmp99")
+}
