@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2015-2020 Canonical Ltd
+ * Copyright (C) 2015-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -27,6 +27,7 @@ import (
 	"mime"
 	"net/http"
 	"strings"
+	"time"
 
 	"github.com/snapcore/snapd/asserts/snapasserts"
 	"github.com/snapcore/snapd/client"
@@ -123,9 +124,9 @@ func postSnap(c *Command, r *http.Request, user *auth.UserState) Response {
 	}
 	inst.ctx = r.Context()
 
-	state := c.d.overlord.State()
-	state.Lock()
-	defer state.Unlock()
+	st := c.d.overlord.State()
+	st.Lock()
+	defer st.Unlock()
 
 	if user != nil {
 		inst.userID = user.ID
@@ -143,17 +144,21 @@ func postSnap(c *Command, r *http.Request, user *auth.UserState) Response {
 		return BadRequest("unknown action %s", inst.Action)
 	}
 
-	msg, tsets, err := impl(&inst, state)
+	msg, tsets, err := impl(&inst, st)
 	if err != nil {
 		return inst.errToResponse(err)
 	}
 
-	chg := newChange(state, inst.Action+"-snap", msg, tsets, inst.Snaps)
+	chg := newChange(st, inst.Action+"-snap", msg, tsets, inst.Snaps)
+	if len(tsets) == 0 {
+		chg.SetStatus(state.DoneStatus)
+	}
+
 	if inst.SystemRestartImmediate {
 		chg.Set("system-restart-immediate", true)
 	}
 
-	ensureStateSoon(state)
+	ensureStateSoon(st)
 
 	return AsyncResponse(nil, chg.ID())
 }
@@ -191,18 +196,22 @@ type snapInstruction struct {
 	Action string `json:"action"`
 	Amend  bool   `json:"amend"`
 	snapRevisionOptions
-	DevMode                bool                   `json:"devmode"`
-	JailMode               bool                   `json:"jailmode"`
-	Classic                bool                   `json:"classic"`
-	IgnoreValidation       bool                   `json:"ignore-validation"`
-	IgnoreRunning          bool                   `json:"ignore-running"`
-	Unaliased              bool                   `json:"unaliased"`
-	Purge                  bool                   `json:"purge,omitempty"`
-	SystemRestartImmediate bool                   `json:"system-restart-immediate"`
-	Transaction            client.TransactionType `json:"transaction"`
-	Snaps                  []string               `json:"snaps"`
-	Users                  []string               `json:"users"`
-	ValidationSets         []string               `json:"validation-sets"`
+	DevMode                bool                             `json:"devmode"`
+	JailMode               bool                             `json:"jailmode"`
+	Classic                bool                             `json:"classic"`
+	IgnoreValidation       bool                             `json:"ignore-validation"`
+	IgnoreRunning          bool                             `json:"ignore-running"`
+	Unaliased              bool                             `json:"unaliased"`
+	Purge                  bool                             `json:"purge,omitempty"`
+	SystemRestartImmediate bool                             `json:"system-restart-immediate"`
+	Transaction            client.TransactionType           `json:"transaction"`
+	Snaps                  []string                         `json:"snaps"`
+	Users                  []string                         `json:"users"`
+	SnapshotOptions        map[string]*snap.SnapshotOptions `json:"snapshot-options"`
+	ValidationSets         []string                         `json:"validation-sets"`
+	QuotaGroupName         string                           `json:"quota-group"`
+	Time                   string                           `json:"time"`
+	HoldLevel              string                           `json:"hold-level"`
 
 	// The fields below should not be unmarshalled into. Do not export them.
 	userID int
@@ -236,10 +245,59 @@ func (inst *snapInstruction) installFlags() (snapstate.Flags, error) {
 	if inst.IgnoreValidation {
 		flags.IgnoreValidation = true
 	}
+	flags.QuotaGroupName = inst.QuotaGroupName
 
 	return flags, nil
 }
 
+func (inst *snapInstruction) holdLevel() snapstate.HoldLevel {
+	switch inst.HoldLevel {
+	case "auto-refresh":
+		return snapstate.HoldAutoRefresh
+	case "general":
+		return snapstate.HoldGeneral
+	default:
+		panic("not validated hold level")
+	}
+}
+
+// cleanSnapshotOptions cleans the snapshot options.
+//
+// With default marshalling, some permutations of valid JSON definitions of snapshot-options e.g.
+//   - `"snapshot-options": { "snap1": {} }`
+//   - `"snapshot-options": { "snap1": {exclude: []} }`
+//
+// results in a pointer to SnapshotOptions object with a nil or zero length exclusion list
+// which in turn will be marshalled to JSON as `options: {}` when we rather want it omitted.
+// The cleaning step ensures that we only populate map entries for snapshot options that contain
+// usable content that we want to be marshalled downstream.
+func (inst *snapInstruction) cleanSnapshotOptions() {
+	for name, options := range inst.SnapshotOptions {
+		if options.Unset() {
+			delete(inst.SnapshotOptions, name)
+		}
+	}
+}
+
+func (inst *snapInstruction) validateSnapshotOptions() error {
+	if inst.SnapshotOptions == nil {
+		return nil
+	}
+	if inst.Action != "snapshot" {
+		return fmt.Errorf("snapshot-options can only be specified for snapshot action")
+	}
+	for name, options := range inst.SnapshotOptions {
+		if !strutil.ListContains(inst.Snaps, name) {
+			return fmt.Errorf("cannot use snapshot-options for snap %q that is not listed in snaps", name)
+		}
+		if err := options.Validate(); err != nil {
+			return fmt.Errorf("invalid snapshot-options for snap %q: %v", name, err)
+		}
+	}
+
+	return nil
+}
+
 func (inst *snapInstruction) validate() error {
 	if inst.CohortKey != "" {
 		if inst.Action != "install" && inst.Action != "refresh" && inst.Action != "switch" {
@@ -269,6 +327,41 @@ func (inst *snapInstruction) validate() error {
 	default:
 		return fmt.Errorf("invalid value for transaction type: %s", inst.Transaction)
 	}
+	if inst.QuotaGroupName != "" && inst.Action != "install" {
+		return fmt.Errorf("quota-group can only be specified on install")
+	}
+
+	if inst.Action == "hold" {
+		if inst.Time == "" {
+			return errors.New("hold action requires a non-empty time value")
+		} else if inst.Time != "forever" {
+			if _, err := time.Parse(time.RFC3339, inst.Time); err != nil {
+				return fmt.Errorf(`hold action requires time to be "forever" or in RFC3339 format: %v`, err)
+			}
+		}
+		if inst.HoldLevel == "" {
+			return errors.New("hold action requires a non-empty hold-level value")
+		} else if !(inst.HoldLevel == "auto-refresh" || inst.HoldLevel == "general") {
+			return errors.New(`hold action requires hold-level to be either "auto-refresh" or "general"`)
+		}
+	}
+
+	if inst.Action != "hold" {
+		if inst.Time != "" {
+			return errors.New(`time can only be specified for the "hold" action`)
+		}
+		if inst.HoldLevel != "" {
+			return errors.New(`hold-level can only be specified for the "hold" action`)
+		}
+	}
+
+	if err := inst.validateSnapshotOptions(); err != nil {
+		return err
+	}
+
+	if inst.Action == "snapshot" {
+		inst.cleanSnapshotOptions()
+	}
 
 	return inst.snapRevisionOptions.validate()
 }
@@ -452,6 +545,26 @@ func snapSwitch(inst *snapInstruction, st *state.State) (string, []*state.TaskSe
 	return msg, []*state.TaskSet{ts}, nil
 }
 
+// snapHold holds refreshes for one snap.
+func snapHold(inst *snapInstruction, st *state.State) (string, []*state.TaskSet, error) {
+	res, err := snapHoldMany(inst, st)
+	if err != nil {
+		return "", nil, err
+	}
+
+	return res.Summary, res.Tasksets, nil
+}
+
+// snapUnhold removes the hold on refreshes for one snap.
+func snapUnhold(inst *snapInstruction, st *state.State) (string, []*state.TaskSet, error) {
+	res, err := snapUnholdMany(inst, st)
+	if err != nil {
+		return "", nil, err
+	}
+
+	return res.Summary, res.Tasksets, nil
+}
+
 type snapActionFunc func(*snapInstruction, *state.State) (string, []*state.TaskSet, error)
 
 var snapInstructionDispTable = map[string]snapActionFunc{
@@ -462,6 +575,8 @@ var snapInstructionDispTable = map[string]snapActionFunc{
 	"enable":  snapEnable,
 	"disable": snapDisable,
 	"switch":  snapSwitch,
+	"hold":    snapHold,
+	"unhold":  snapUnhold,
 }
 
 func (inst *snapInstruction) dispatch() snapActionFunc {
@@ -539,19 +654,17 @@ func snapOpMany(c *Command, r *http.Request, user *auth.UserState) Response {
 		return inst.errToResponse(err)
 	}
 
-	var chg *state.Change
+	chg := newChange(st, inst.Action+"-snap", res.Summary, res.Tasksets, res.Affected)
 	if len(res.Tasksets) == 0 {
-		chg = st.NewChange(inst.Action+"-snap", res.Summary)
 		chg.SetStatus(state.DoneStatus)
-	} else {
-		chg = newChange(st, inst.Action+"-snap", res.Summary, res.Tasksets, res.Affected)
-		ensureStateSoon(st)
 	}
 
 	if inst.SystemRestartImmediate {
 		chg.Set("system-restart-immediate", true)
 	}
 
+	ensureStateSoon(st)
+
 	chg.Set("api-data", map[string]interface{}{"snap-names": res.Affected})
 
 	return AsyncResponse(res.Result, chg.ID())
@@ -574,6 +687,10 @@ func (inst *snapInstruction) dispatchForMany() (op snapManyActionFunc) {
 	case "snapshot":
 		// see api_snapshots.go
 		op = snapshotMany
+	case "hold":
+		op = snapHoldMany
+	case "unhold":
+		op = snapUnholdMany
 	}
 	return op
 }
@@ -585,7 +702,7 @@ func snapInstallMany(inst *snapInstruction, st *state.State) (*snapInstructionRe
 		}
 	}
 	transaction := inst.Transaction
-	installed, tasksets, err := snapstateInstallMany(st, inst.Snaps, inst.userID, &snapstate.Flags{Transaction: transaction})
+	installed, tasksets, err := snapstateInstallMany(st, inst.Snaps, nil, inst.userID, &snapstate.Flags{Transaction: transaction})
 	if err != nil {
 		return nil, err
 	}
@@ -623,7 +740,7 @@ func snapUpdateMany(inst *snapInstruction, st *state.State) (*snapInstructionRes
 
 	transaction := inst.Transaction
 	// TODO: use a per-request context
-	updated, tasksets, err := snapstateUpdateMany(context.TODO(), st, inst.Snaps, inst.userID, &snapstate.Flags{
+	updated, tasksets, err := snapstateUpdateMany(context.TODO(), st, inst.Snaps, nil, inst.userID, &snapstate.Flags{
 		IgnoreRunning: inst.IgnoreRunning,
 		Transaction:   transaction,
 	})
@@ -679,29 +796,55 @@ func snapEnforceValidationSets(inst *snapInstruction, st *state.State) (*snapIns
 		return nil, err
 	}
 
-	var validationErr *snapasserts.ValidationSetsValidationError
-	err = assertstateTryEnforceValidationSets(st, inst.ValidationSets, inst.userID, snaps, ignoreValidationSnaps)
+	var tss []*state.TaskSet
+	var affected []string
+	err = assertstateTryEnforcedValidationSets(st, inst.ValidationSets, inst.userID, snaps, ignoreValidationSnaps)
 	if err != nil {
-		var ok bool
-		validationErr, ok = err.(*snapasserts.ValidationSetsValidationError)
+		vErr, ok := err.(*snapasserts.ValidationSetsValidationError)
 		if !ok {
 			return nil, err
 		}
+
+		tss, affected, err = meetSnapConstraintsForEnforce(inst, st, vErr)
+		if err != nil {
+			return nil, err
+		}
 	}
-	tss, affected, err := snapstateEnforceSnaps(context.TODO(), st, inst.ValidationSets, validationErr, inst.userID)
-	if err != nil {
-		return nil, err
+
+	summary := fmt.Sprintf("Enforce validation sets %s", strutil.Quoted(inst.ValidationSets))
+	if len(affected) != 0 {
+		summary = fmt.Sprintf("%s for snaps %s", summary, strutil.Quoted(affected))
 	}
 
 	return &snapInstructionResult{
-		Summary:  fmt.Sprintf("Enforced validation sets: %s", strutil.Quoted(inst.ValidationSets)),
+		Summary:  summary,
 		Affected: affected,
 		Tasksets: tss,
 	}, nil
 }
 
+func meetSnapConstraintsForEnforce(inst *snapInstruction, st *state.State, vErr *snapasserts.ValidationSetsValidationError) ([]*state.TaskSet, []string, error) {
+	// Save the sequence numbers so we can pin them later when enforcing the sets again
+	pinnedSeqs := make(map[string]int, len(inst.ValidationSets))
+	for _, vsStr := range inst.ValidationSets {
+		account, name, sequence, err := snapasserts.ParseValidationSet(vsStr)
+		if err != nil {
+			return nil, nil, err
+		}
+
+		if sequence == 0 {
+			continue
+		}
+
+		pinnedSeqs[fmt.Sprintf("%s/%s", account, name)] = sequence
+	}
+
+	return snapstateResolveValSetsEnforcementError(context.TODO(), st, vErr, pinnedSeqs, inst.userID)
+}
+
 func snapRemoveMany(inst *snapInstruction, st *state.State) (*snapInstructionResult, error) {
-	removed, tasksets, err := snapstateRemoveMany(st, inst.Snaps)
+	flags := &snapstate.RemoveFlags{Purge: inst.Purge}
+	removed, tasksets, err := snapstateRemoveMany(st, inst.Snaps, flags)
 	if err != nil {
 		return nil, err
 	}
@@ -811,3 +954,65 @@ func shouldSearchStore(r *http.Request) bool {
 
 	return false
 }
+
+func snapHoldMany(inst *snapInstruction, st *state.State) (res *snapInstructionResult, err error) {
+	var msg string
+	var tss []*state.TaskSet
+	if len(inst.Snaps) == 0 {
+		if inst.holdLevel() == snapstate.HoldGeneral {
+			return nil, errors.New("holding general refreshes for all snaps is not supported")
+		}
+		patchValues := map[string]interface{}{"refresh.hold": inst.Time}
+		ts, err := configstateConfigureInstalled(st, "core", patchValues, 0)
+		if err != nil {
+			return nil, err
+		}
+
+		tss = []*state.TaskSet{ts}
+		msg = i18n.G("Hold auto-refreshes for all snaps")
+	} else {
+		holdLevel := inst.holdLevel()
+		if err := snapstateHoldRefreshesBySystem(st, holdLevel, inst.Time, inst.Snaps); err != nil {
+			return nil, err
+		}
+		msgFmt := i18n.G("Hold general refreshes for %s")
+		if holdLevel == snapstate.HoldAutoRefresh {
+			msgFmt = i18n.G("Hold auto-refreshes for %s")
+		}
+		msg = fmt.Sprintf(msgFmt, strutil.Quoted(inst.Snaps))
+	}
+
+	return &snapInstructionResult{
+		Summary:  msg,
+		Affected: inst.Snaps,
+		Tasksets: tss,
+	}, nil
+}
+
+func snapUnholdMany(inst *snapInstruction, st *state.State) (res *snapInstructionResult, err error) {
+	var msg string
+	var tss []*state.TaskSet
+
+	if len(inst.Snaps) == 0 {
+		patchValues := map[string]interface{}{"refresh.hold": nil}
+		ts, err := configstateConfigureInstalled(st, "core", patchValues, 0)
+		if err != nil {
+			return nil, err
+		}
+
+		tss = []*state.TaskSet{ts}
+		msg = i18n.G("Remove auto-refresh hold on all snaps")
+	} else {
+		if err := snapstateProceedWithRefresh(st, "system", inst.Snaps); err != nil {
+			return nil, err
+		}
+
+		msg = fmt.Sprintf(i18n.G("Remove refresh hold on %s"), strutil.Quoted(inst.Snaps))
+	}
+
+	return &snapInstructionResult{
+		Summary:  msg,
+		Affected: inst.Snaps,
+		Tasksets: tss,
+	}, nil
+}
