@@ -19,9 +19,12 @@
 
 package gadget
 
+import "github.com/snapcore/snapd/gadget/quantity"
+
 type (
 	MountedFilesystemUpdater = mountedFilesystemUpdater
 	RawStructureUpdater      = rawStructureUpdater
+	InvalidOffsetError       = invalidOffsetError
 )
 
 var (
@@ -29,6 +32,7 @@ var (
 	ValidateVolumeStructure = validateVolumeStructure
 	ValidateRole            = validateRole
 	ValidateVolume          = validateVolume
+	ValidateOffsetWrite     = validateOffsetWrite
 
 	SetImplicitForVolumeStructure = setImplicitForVolumeStructure
 
@@ -64,7 +68,8 @@ var (
 
 	OnDiskStructureIsLikelyImplicitSystemDataRole = onDiskStructureIsLikelyImplicitSystemDataRole
 
-	SearchForVolumeWithTraits = searchForVolumeWithTraits
+	SearchVolumeWithTraitsAndMatchParts = searchVolumeWithTraitsAndMatchParts
+	OrderStructuresByOffset             = orderStructuresByOffset
 )
 
 func MockEvalSymlinks(mock func(path string) (string, error)) (restore func()) {
@@ -83,3 +88,7 @@ func (m *MountedFilesystemWriter) WriteDirectory(volumeRoot, src, dst string, pr
 func (s *StructureEncryptionParameters) SetUnknownKeys(m map[string]string) {
 	s.unknownKeys = m
 }
+
+func NewInvalidOffsetError(offset, lowerBound, upperBound quantity.Offset) *InvalidOffsetError {
+	return &invalidOffsetError{offset: offset, lowerBound: lowerBound, upperBound: upperBound}
+}
