@@ -23,16 +23,18 @@ import (
 	"errors"
 	"fmt"
 	"io/ioutil"
-	"os"
 	"path/filepath"
+	"syscall"
 
 	. "gopkg.in/check.v1"
 
 	"github.com/snapcore/snapd/boot"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/gadget"
+	"github.com/snapcore/snapd/gadget/gadgettest"
 	"github.com/snapcore/snapd/gadget/install"
 	"github.com/snapcore/snapd/gadget/quantity"
+	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/testutil"
 )
 
@@ -64,7 +66,7 @@ func (s *contentTestSuite) SetUpTest(c *C) {
 	s.mockUnmountCalls = nil
 
 	s.gadgetRoot = c.MkDir()
-	err := makeMockGadget(s.gadgetRoot, gadgetContent)
+	err := gadgettest.MakeMockGadget(s.gadgetRoot, gadgetContent)
 	c.Assert(err, IsNil)
 
 	s.mockMountPoint = c.MkDir()
@@ -82,15 +84,9 @@ func (s *contentTestSuite) SetUpTest(c *C) {
 	s.AddCleanup(restore)
 }
 
-var mockOnDiskStructureSystemSeed = gadget.OnDiskStructure{
-	Node: "/dev/node2",
-	LaidOutStructure: gadget.LaidOutStructure{
+func mockOnDiskStructureSystemSeed(gadgetRoot string) *gadget.LaidOutStructure {
+	return &gadget.LaidOutStructure{
 		VolumeStructure: &gadget.VolumeStructure{
-			Name:       "Recovery",
-			Size:       1258291200,
-			Type:       "EF,C12A7328-F81F-11D2-BA4B-00A0C93EC93B",
-			Role:       "system-seed",
-			Label:      "ubuntu-seed",
 			Filesystem: "vfat",
 			Content: []gadget.VolumeContent{
 				{
@@ -98,31 +94,18 @@ var mockOnDiskStructureSystemSeed = gadget.OnDiskStructure{
 					Target:           "EFI/boot/grubx64.efi",
 				},
 			},
+			YamlIndex: 1000, // to demonstrate we do not use the laid out index
+		},
+		ResolvedContent: []gadget.ResolvedContent{
+			{
+				VolumeContent: &gadget.VolumeContent{
+					UnresolvedSource: "grubx64.efi",
+					Target:           "EFI/boot/grubx64.efi",
+				},
+				ResolvedSource: filepath.Join(gadgetRoot, "grubx64.efi"),
+			},
 		},
-		StartOffset: 2097152,
-		YamlIndex:   1000, // to demonstrate we do not use the laid out index
-	},
-	DiskIndex: 2,
-}
-
-func makeMockGadget(gadgetRoot, gadgetContent string) error {
-	if err := os.MkdirAll(filepath.Join(gadgetRoot, "meta"), 0755); err != nil {
-		return err
 	}
-	if err := ioutil.WriteFile(filepath.Join(gadgetRoot, "meta", "gadget.yaml"), []byte(gadgetContent), 0644); err != nil {
-		return err
-	}
-	if err := ioutil.WriteFile(filepath.Join(gadgetRoot, "pc-boot.img"), []byte("pc-boot.img content"), 0644); err != nil {
-		return err
-	}
-	if err := ioutil.WriteFile(filepath.Join(gadgetRoot, "pc-core.img"), []byte("pc-core.img content"), 0644); err != nil {
-		return err
-	}
-	if err := ioutil.WriteFile(filepath.Join(gadgetRoot, "grubx64.efi"), []byte("grubx64.efi content"), 0644); err != nil {
-		return err
-	}
-
-	return nil
 }
 
 const gadgetContent = `volumes:
@@ -163,21 +146,20 @@ type mockContentChange struct {
 }
 
 type mockWriteObserver struct {
-	content        map[string][]*mockContentChange
-	observeErr     error
-	expectedStruct *gadget.LaidOutStructure
-	c              *C
+	content      map[string][]*mockContentChange
+	observeErr   error
+	expectedRole string
+	c            *C
 }
 
-func (m *mockWriteObserver) Observe(op gadget.ContentOperation, sourceStruct *gadget.LaidOutStructure,
+func (m *mockWriteObserver) Observe(op gadget.ContentOperation, partRole,
 	targetRootDir, relativeTargetPath string, data *gadget.ContentChange) (gadget.ContentChangeAction, error) {
 	if m.content == nil {
 		m.content = make(map[string][]*mockContentChange)
 	}
 	m.content[targetRootDir] = append(m.content[targetRootDir],
 		&mockContentChange{path: relativeTargetPath, change: data})
-	m.c.Assert(sourceStruct, NotNil)
-	m.c.Check(sourceStruct, DeepEquals, m.expectedStruct)
+	m.c.Check(partRole, Equals, m.expectedRole)
 	return gadget.ChangeApply, m.observeErr
 }
 
@@ -197,11 +179,11 @@ func (s *contentTestSuite) TestWriteFilesystemContent(c *C) {
 		}, {
 			mountErr:   errors.New("mount error"),
 			unmountErr: nil,
-			err:        "cannot mount filesystem .*: mount error",
+			err:        "cannot mount .* at .*: mount error",
 		}, {
 			mountErr:   nil,
 			unmountErr: errors.New("unmount error"),
-			err:        "unmount error",
+			err:        "cannot unmount /dev/node2 after writing filesystem content: unmount error",
 		}, {
 			observeErr: errors.New("observe error"),
 			err:        "cannot create filesystem image: cannot write filesystem content of source:grubx64.efi: cannot observe file write: observe error",
@@ -212,7 +194,7 @@ func (s *contentTestSuite) TestWriteFilesystemContent(c *C) {
 		restore := install.MockSysMount(func(source, target, fstype string, flags uintptr, data string) error {
 			c.Check(source, Equals, "/dev/node2")
 			c.Check(fstype, Equals, "vfat")
-			c.Check(target, Equals, filepath.Join(dirs.SnapRunDir, "gadget-install/2"))
+			c.Check(target, Equals, filepath.Join(dirs.SnapRunDir, "gadget-install/dev-node2"))
 			return tc.mountErr
 		})
 		defer restore()
@@ -223,22 +205,13 @@ func (s *contentTestSuite) TestWriteFilesystemContent(c *C) {
 		defer restore()
 
 		// copy existing mock
-		m := mockOnDiskStructureSystemSeed
-		m.ResolvedContent = []gadget.ResolvedContent{
-			{
-				VolumeContent: &gadget.VolumeContent{
-					UnresolvedSource: "grubx64.efi",
-					Target:           "EFI/boot/grubx64.efi",
-				},
-				ResolvedSource: filepath.Join(s.gadgetRoot, "grubx64.efi"),
-			},
-		}
+		m := mockOnDiskStructureSystemSeed(s.gadgetRoot)
 		obs := &mockWriteObserver{
-			c:              c,
-			observeErr:     tc.observeErr,
-			expectedStruct: &m.LaidOutStructure,
+			c:            c,
+			observeErr:   tc.observeErr,
+			expectedRole: m.Role(),
 		}
-		err := install.WriteFilesystemContent(&m, "/dev/node2", obs)
+		err := install.WriteFilesystemContent(m, "/dev/node2", obs)
 		if tc.err == "" {
 			c.Assert(err, IsNil)
 		} else {
@@ -247,11 +220,11 @@ func (s *contentTestSuite) TestWriteFilesystemContent(c *C) {
 
 		if err == nil {
 			// the target file system is mounted on a directory named after the structure index
-			content, err := ioutil.ReadFile(filepath.Join(dirs.SnapRunDir, "gadget-install/2", "EFI/boot/grubx64.efi"))
+			content, err := ioutil.ReadFile(filepath.Join(dirs.SnapRunDir, "gadget-install/dev-node2", "EFI/boot/grubx64.efi"))
 			c.Assert(err, IsNil)
 			c.Check(string(content), Equals, "grubx64.efi content")
 			c.Assert(obs.content, DeepEquals, map[string][]*mockContentChange{
-				filepath.Join(dirs.SnapRunDir, "gadget-install/2"): {
+				filepath.Join(dirs.SnapRunDir, "gadget-install/dev-node2"): {
 					{
 						path:   "EFI/boot/grubx64.efi",
 						change: &gadget.ContentChange{After: filepath.Join(s.gadgetRoot, "grubx64.efi")},
@@ -262,6 +235,85 @@ func (s *contentTestSuite) TestWriteFilesystemContent(c *C) {
 	}
 }
 
+func (s *contentTestSuite) TestWriteFilesystemContentUnmountErrHandling(c *C) {
+	dirs.SetRootDir(c.MkDir())
+	defer dirs.SetRootDir(dirs.GlobalRootDir)
+
+	log, restore := logger.MockLogger()
+	defer restore()
+
+	type unmountArgs struct {
+		target string
+		flags  int
+	}
+
+	restore = install.MockSysMount(func(source, target, fstype string, flags uintptr, data string) error {
+		return nil
+	})
+	defer restore()
+
+	// copy existing mock
+	m := mockOnDiskStructureSystemSeed(s.gadgetRoot)
+	obs := &mockWriteObserver{
+		c:            c,
+		observeErr:   nil,
+		expectedRole: m.Role(),
+	}
+
+	for _, tc := range []struct {
+		unmountErr     error
+		lazyUnmountErr error
+
+		expectedErr string
+	}{
+		{
+			nil,
+			nil,
+			"",
+		}, {
+			errors.New("umount error"),
+			nil,
+			"", // no error as lazy unmount succeeded
+		}, {
+			errors.New("umount error"),
+			errors.New("lazy umount err"),
+			`cannot unmount /dev/node2 after writing filesystem content: lazy umount err`},
+	} {
+		log.Reset()
+
+		var unmountCalls []unmountArgs
+		restore = install.MockSysUnmount(func(target string, flags int) error {
+			unmountCalls = append(unmountCalls, unmountArgs{target, flags})
+			switch flags {
+			case 0:
+				return tc.unmountErr
+			case syscall.MNT_DETACH:
+				return tc.lazyUnmountErr
+			default:
+				return fmt.Errorf("unexpected mount flag %v", flags)
+			}
+		})
+		defer restore()
+
+		err := install.WriteFilesystemContent(m, "/dev/node2", obs)
+		if tc.expectedErr == "" {
+			c.Assert(err, IsNil)
+		} else {
+			c.Assert(err, ErrorMatches, tc.expectedErr)
+		}
+		if tc.unmountErr == nil {
+			c.Check(unmountCalls, HasLen, 1)
+			c.Check(unmountCalls[0].flags, Equals, 0)
+			c.Check(log.String(), Equals, "")
+		} else {
+			c.Check(unmountCalls, HasLen, 2)
+			c.Check(unmountCalls[0].flags, Equals, 0)
+			c.Check(unmountCalls[1].flags, Equals, syscall.MNT_DETACH)
+			c.Check(log.String(), Matches, `(?sm).* cannot unmount /.*/run/snapd/gadget-install/dev-node2 after writing filesystem content, trying lazy unmount next: umount error`)
+		}
+	}
+}
+
 func (s *contentTestSuite) TestMakeFilesystem(c *C) {
 	mockUdevadm := testutil.MockCommand(c, "udevadm", "")
 	defer mockUdevadm.Restore()
@@ -277,9 +329,9 @@ func (s *contentTestSuite) TestMakeFilesystem(c *C) {
 	defer restore()
 
 	err := install.MakeFilesystem(install.MkfsParams{
-		Type:       mockOnDiskStructureWritable.Filesystem,
+		Type:       mockOnDiskStructureWritable.PartitionFSType,
 		Device:     mockOnDiskStructureWritable.Node,
-		Label:      mockOnDiskStructureWritable.Label,
+		Label:      mockOnDiskStructureWritable.PartitionFSLabel,
 		Size:       mockOnDiskStructureWritable.Size,
 		SectorSize: quantity.Size(512),
 	})
@@ -298,9 +350,9 @@ func (s *contentTestSuite) TestMakeFilesystemRealMkfs(c *C) {
 	defer mockMkfsExt4.Restore()
 
 	err := install.MakeFilesystem(install.MkfsParams{
-		Type:       mockOnDiskStructureWritable.Filesystem,
+		Type:       mockOnDiskStructureWritable.PartitionFSType,
 		Device:     mockOnDiskStructureWritable.Node,
-		Label:      mockOnDiskStructureWritable.Label,
+		Label:      mockOnDiskStructureWritable.PartitionFSLabel,
 		Size:       mockOnDiskStructureWritable.Size,
 		SectorSize: quantity.Size(512),
 	})
@@ -320,7 +372,7 @@ func (s *contentTestSuite) TestMountFilesystem(c *C) {
 	defer dirs.SetRootDir("")
 
 	// mount a filesystem...
-	err := install.MountFilesystem("/dev/node2", "vfat", "ubuntu-seed", boot.InitramfsRunMntDir)
+	err := install.MountFilesystem("/dev/node2", "vfat", filepath.Join(boot.InitramfsRunMntDir, "ubuntu-seed"))
 	c.Assert(err, IsNil)
 
 	// ...and check if it was mounted at the right mount point
@@ -331,6 +383,6 @@ func (s *contentTestSuite) TestMountFilesystem(c *C) {
 
 	// try again with mocked error
 	s.mockMountErr = fmt.Errorf("mock mount error")
-	err = install.MountFilesystem("/dev/node2", "vfat", "ubuntu-seed", boot.InitramfsRunMntDir)
+	err = install.MountFilesystem("/dev/node2", "vfat", filepath.Join(boot.InitramfsRunMntDir, "ubuntu-seed"))
 	c.Assert(err, ErrorMatches, `cannot mount filesystem "/dev/node2" at ".*/run/mnt/ubuntu-seed": mock mount error`)
 }
