@@ -33,6 +33,7 @@ import (
 	"github.com/snapcore/snapd/gadget"
 	"github.com/snapcore/snapd/gadget/quantity"
 	"github.com/snapcore/snapd/kernel"
+	"github.com/snapcore/snapd/secboot"
 )
 
 type layoutTestSuite struct {
@@ -45,30 +46,32 @@ func (p *layoutTestSuite) SetUpTest(c *C) {
 	p.dir = c.MkDir()
 }
 
-var defaultConstraints = gadget.LayoutConstraints{
-	NonMBRStartOffset: 1 * quantity.OffsetMiB,
-}
-
 func (p *layoutTestSuite) TestVolumeSize(c *C) {
 	vol := gadget.Volume{
 		Structure: []gadget.VolumeStructure{
-			{Size: 2 * quantity.SizeMiB},
+			{Offset: asOffsetPtr(quantity.OffsetMiB), Size: 2 * quantity.SizeMiB, EnclosingVolume: &gadget.Volume{}},
 		},
 	}
-	v, err := gadget.LayoutVolume(p.dir, "", &vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(&vol, opts)
 	c.Assert(err, IsNil)
 
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
 		Volume: &gadget.Volume{
 			Structure: []gadget.VolumeStructure{
-				{Size: 2 * quantity.SizeMiB},
+				{Offset: asOffsetPtr(quantity.OffsetMiB), Size: 2 * quantity.SizeMiB, EnclosingVolume: &gadget.Volume{}},
 			},
 		},
-		Size:    3 * quantity.SizeMiB,
-		RootDir: p.dir,
 		LaidOutStructure: []gadget.LaidOutStructure{{
-			VolumeStructure: &gadget.VolumeStructure{Size: 2 * quantity.SizeMiB},
-			StartOffset:     1 * quantity.OffsetMiB,
+			OnDiskStructure: gadget.OnDiskStructure{
+				Size:        2 * quantity.SizeMiB,
+				StartOffset: 1 * quantity.OffsetMiB,
+			},
+			VolumeStructure: &gadget.VolumeStructure{
+				Offset:          asOffsetPtr(quantity.OffsetMiB),
+				Size:            2 * quantity.SizeMiB,
+				EnclosingVolume: &gadget.Volume{},
+			},
 		}},
 	})
 }
@@ -96,28 +99,105 @@ volumes:
 	vol := mustParseVolume(c, gadgetYaml, "first-image")
 	c.Assert(vol.Structure, HasLen, 2)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(err, IsNil)
 
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
-		Volume:  vol,
-		Size:    501 * quantity.SizeMiB,
-		RootDir: p.dir,
+		Volume: vol,
 		LaidOutStructure: []gadget.LaidOutStructure{
 			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:        "00000000-0000-0000-0000-0000deadbeef",
+					Size:        400 * quantity.SizeMiB,
+					StartOffset: 1 * quantity.OffsetMiB,
+				},
 				VolumeStructure: &vol.Structure[0],
-				StartOffset:     1 * quantity.OffsetMiB,
-				YamlIndex:       0,
 			},
 			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:        "83,00000000-0000-0000-0000-0000feedface",
+					Size:        100 * quantity.SizeMiB,
+					StartOffset: 401 * quantity.OffsetMiB,
+				},
 				VolumeStructure: &vol.Structure[1],
-				StartOffset:     401 * quantity.OffsetMiB,
-				YamlIndex:       1,
 			},
 		},
 	})
 }
 
+func (p *layoutTestSuite) testLayoutVolumeWithDataPartitions(c *C, encType secboot.EncryptionType) {
+	gadgetYaml := `
+volumes:
+  first:
+    schema: gpt
+    bootloader: grub
+    structure:
+        - type: 00000000-0000-0000-0000-dd00deadbeef
+          filesystem: ext4
+          filesystem-label: ubuntu-save
+          role: system-save
+          size: 1M
+        - type: 00000000-0000-0000-0000-cc00deadbeef
+          filesystem: ext4
+          filesystem-label: ubuntu-data
+          role: system-data
+          size: 500M
+`
+	vol := mustParseVolume(c, gadgetYaml, "first")
+	c.Assert(vol.Structure, HasLen, 2)
+
+	opts := &gadget.LayoutOptions{
+		GadgetRootDir: p.dir,
+		EncType:       encType}
+	v, err := gadget.LayoutVolume(vol, opts)
+	c.Assert(err, IsNil)
+
+	saveFsLabel := "ubuntu-save"
+	saveFsType := "ext4"
+	dataFsLabel := "ubuntu-data"
+	dataFsType := "ext4"
+	if encType == secboot.EncryptionTypeLUKS {
+		saveFsLabel = "ubuntu-save-enc"
+		saveFsType = "crypto_LUKS"
+		dataFsLabel = "ubuntu-data-enc"
+		dataFsType = "crypto_LUKS"
+	}
+	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
+		Volume: vol,
+		LaidOutStructure: []gadget.LaidOutStructure{
+			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:             "00000000-0000-0000-0000-dd00deadbeef",
+					Size:             1 * quantity.SizeMiB,
+					StartOffset:      1 * quantity.OffsetMiB,
+					PartitionFSType:  saveFsType,
+					PartitionFSLabel: saveFsLabel,
+				},
+				VolumeStructure: &vol.Structure[0],
+			},
+			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:             "00000000-0000-0000-0000-cc00deadbeef",
+					Size:             500 * quantity.SizeMiB,
+					StartOffset:      2 * quantity.OffsetMiB,
+					PartitionFSType:  dataFsType,
+					PartitionFSLabel: dataFsLabel,
+				},
+				VolumeStructure: &vol.Structure[1],
+			},
+		},
+	})
+}
+
+func (p *layoutTestSuite) TestLayoutVolumeWithDataPartitions(c *C) {
+	p.testLayoutVolumeWithDataPartitions(c, secboot.EncryptionTypeNone)
+}
+
+func (p *layoutTestSuite) TestLayoutVolumeWithDataPartitionsEncrypted(c *C) {
+	p.testLayoutVolumeWithDataPartitions(c, secboot.EncryptionTypeLUKS)
+}
+
 func (p *layoutTestSuite) TestLayoutVolumeImplicitOrdering(c *C) {
 	gadgetYaml := `
 volumes:
@@ -138,33 +218,44 @@ volumes:
 	vol := mustParseVolume(c, gadgetYaml, "first")
 	c.Assert(vol.Structure, HasLen, 4)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(err, IsNil)
 
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
-		Volume:  vol,
-		Size:    1101 * quantity.SizeMiB,
-		RootDir: p.dir,
+		Volume: vol,
 		LaidOutStructure: []gadget.LaidOutStructure{
 			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:        "00000000-0000-0000-0000-dd00deadbeef",
+					Size:        400 * quantity.SizeMiB,
+					StartOffset: 1 * quantity.OffsetMiB,
+				},
 				VolumeStructure: &vol.Structure[0],
-				StartOffset:     1 * quantity.OffsetMiB,
-				YamlIndex:       0,
 			},
 			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:        "00000000-0000-0000-0000-cc00deadbeef",
+					Size:        500 * quantity.SizeMiB,
+					StartOffset: 401 * quantity.OffsetMiB,
+				},
 				VolumeStructure: &vol.Structure[1],
-				StartOffset:     401 * quantity.OffsetMiB,
-				YamlIndex:       1,
 			},
 			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:        "00000000-0000-0000-0000-bb00deadbeef",
+					Size:        100 * quantity.SizeMiB,
+					StartOffset: 901 * quantity.OffsetMiB,
+				},
 				VolumeStructure: &vol.Structure[2],
-				StartOffset:     901 * quantity.OffsetMiB,
-				YamlIndex:       2,
 			},
 			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:        "00000000-0000-0000-0000-aa00deadbeef",
+					Size:        100 * quantity.SizeMiB,
+					StartOffset: 1001 * quantity.OffsetMiB,
+				},
 				VolumeStructure: &vol.Structure[3],
-				StartOffset:     1001 * quantity.OffsetMiB,
-				YamlIndex:       3,
 			},
 		},
 	})
@@ -194,33 +285,44 @@ volumes:
 	vol := mustParseVolume(c, gadgetYaml, "first")
 	c.Assert(vol.Structure, HasLen, 4)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(err, IsNil)
 
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
-		Volume:  vol,
-		Size:    1300 * quantity.SizeMiB,
-		RootDir: p.dir,
+		Volume: vol,
 		LaidOutStructure: []gadget.LaidOutStructure{
 			{
-				VolumeStructure: &vol.Structure[3],
-				StartOffset:     1 * quantity.OffsetMiB,
-				YamlIndex:       3,
-			},
-			{
-				VolumeStructure: &vol.Structure[1],
-				StartOffset:     200 * quantity.OffsetMiB,
-				YamlIndex:       1,
-			},
-			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:        "00000000-0000-0000-0000-aa00deadbeef",
+					Size:        100 * quantity.SizeMiB,
+					StartOffset: 1 * quantity.OffsetMiB,
+				},
 				VolumeStructure: &vol.Structure[0],
-				StartOffset:     800 * quantity.OffsetMiB,
-				YamlIndex:       0,
 			},
 			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:        "00000000-0000-0000-0000-cc00deadbeef",
+					Size:        500 * quantity.SizeMiB,
+					StartOffset: 200 * quantity.OffsetMiB,
+				},
+				VolumeStructure: &vol.Structure[1],
+			},
+			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:        "00000000-0000-0000-0000-dd00deadbeef",
+					Size:        400 * quantity.SizeMiB,
+					StartOffset: 800 * quantity.OffsetMiB,
+				},
 				VolumeStructure: &vol.Structure[2],
-				StartOffset:     1200 * quantity.OffsetMiB,
-				YamlIndex:       2,
+			},
+			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:        "00000000-0000-0000-0000-bb00deadbeef",
+					Size:        100 * quantity.SizeMiB,
+					StartOffset: 1200 * quantity.OffsetMiB,
+				},
+				VolumeStructure: &vol.Structure[3],
 			},
 		},
 	})
@@ -249,33 +351,44 @@ volumes:
 	vol := mustParseVolume(c, gadgetYaml, "first")
 	c.Assert(vol.Structure, HasLen, 4)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(err, IsNil)
 
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
-		Volume:  vol,
-		Size:    1200 * quantity.SizeMiB,
-		RootDir: p.dir,
+		Volume: vol,
 		LaidOutStructure: []gadget.LaidOutStructure{
 			{
-				VolumeStructure: &vol.Structure[3],
-				StartOffset:     1 * quantity.OffsetMiB,
-				YamlIndex:       3,
-			},
-			{
-				VolumeStructure: &vol.Structure[1],
-				StartOffset:     200 * quantity.OffsetMiB,
-				YamlIndex:       1,
-			},
-			{
-				VolumeStructure: &vol.Structure[2],
-				StartOffset:     700 * quantity.OffsetMiB,
-				YamlIndex:       2,
-			},
-			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:        "00000000-0000-0000-0000-aa00deadbeef",
+					Size:        100 * quantity.SizeMiB,
+					StartOffset: 1 * quantity.OffsetMiB,
+				},
 				VolumeStructure: &vol.Structure[0],
-				StartOffset:     800 * quantity.OffsetMiB,
-				YamlIndex:       0,
+			},
+			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:        "00000000-0000-0000-0000-cc00deadbeef",
+					Size:        500 * quantity.SizeMiB,
+					StartOffset: 200 * quantity.OffsetMiB,
+				},
+				VolumeStructure: &vol.Structure[1],
+			},
+			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:        "00000000-0000-0000-0000-bb00deadbeef",
+					Size:        100 * quantity.SizeMiB,
+					StartOffset: 700 * quantity.OffsetMiB,
+				},
+				VolumeStructure: &vol.Structure[2],
+			},
+			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:        "00000000-0000-0000-0000-dd00deadbeef",
+					Size:        400 * quantity.SizeMiB,
+					StartOffset: 800 * quantity.OffsetMiB,
+				},
+				VolumeStructure: &vol.Structure[3],
 			},
 		},
 	})
@@ -295,11 +408,53 @@ volumes:
               - image: foo.img
 `
 	vol := mustParseVolume(c, gadgetYaml, "first")
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(v, IsNil)
 	c.Assert(err, ErrorMatches, `cannot lay out structure #0: content "foo.img":.*no such file or directory`)
 }
 
+func (p *layoutTestSuite) TestLayoutVolumeContentCanIgnoreContent(c *C) {
+	gadgetYaml := `
+volumes:
+  first:
+    schema: gpt
+    bootloader: grub
+    structure:
+        - type: 00000000-0000-0000-0000-dd00deadbeef
+          size: 400M
+          offset: 800M
+          content:
+              - image: foo.img
+`
+	vol := mustParseVolume(c, gadgetYaml, "first")
+	opts := &gadget.LayoutOptions{
+		GadgetRootDir: p.dir,
+	}
+	// LayoutVolume fails with default options because the foo.img
+	// file is missing
+	_, err := gadget.LayoutVolume(vol, opts)
+	c.Assert(err, ErrorMatches, `cannot lay out structure #0: content "foo.img":.*no such file or directory`)
+
+	// But LayoutVolume works with the IgnoreContent works
+	opts.IgnoreContent = true
+	v, err := gadget.LayoutVolume(vol, opts)
+	c.Assert(err, IsNil)
+	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
+		Volume: vol,
+		LaidOutStructure: []gadget.LaidOutStructure{
+			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:        "00000000-0000-0000-0000-dd00deadbeef",
+					Size:        400 * quantity.SizeMiB,
+					StartOffset: 800 * quantity.OffsetMiB,
+				},
+				VolumeStructure: &vol.Structure[0],
+			},
+		},
+	})
+}
+
 func makeSizedFile(c *C, path string, size quantity.Size, content []byte) {
 	err := os.MkdirAll(filepath.Dir(path), 0755)
 	c.Assert(err, IsNil)
@@ -333,7 +488,8 @@ volumes:
 
 	vol := mustParseVolume(c, gadgetYaml, "first")
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(v, IsNil)
 	c.Assert(err, ErrorMatches, `cannot lay out structure #0: content "foo.img" does not fit in the structure`)
 }
@@ -356,10 +512,8 @@ volumes:
 
 	vol := mustParseVolume(c, gadgetYaml, "first")
 
-	constraints := gadget.LayoutConstraints{
-		NonMBRStartOffset: 1 * quantity.OffsetMiB,
-	}
-	v, err := gadget.LayoutVolume(p.dir, "", vol, constraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(v, IsNil)
 	c.Assert(err, ErrorMatches, `cannot lay out structure #0: content "bar.img" does not fit in the structure`)
 }
@@ -382,7 +536,8 @@ volumes:
 
 	vol := mustParseVolume(c, gadgetYaml, "first")
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(v, IsNil)
 	c.Assert(err, ErrorMatches, `cannot lay out structure #0: content "foo.img" does not fit in the structure`)
 }
@@ -404,7 +559,8 @@ volumes:
 
 	vol := mustParseVolume(c, gadgetYaml, "first")
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(v, IsNil)
 	c.Assert(err, ErrorMatches, fmt.Sprintf(`cannot lay out structure #0: content "foo.img" size %v is larger than declared %v`, quantity.SizeMiB+1, quantity.SizeMiB))
 }
@@ -432,7 +588,8 @@ volumes:
 
 	vol := mustParseVolume(c, gadgetYaml, "first")
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(v, IsNil)
 	c.Assert(err, ErrorMatches, `cannot lay out structure #0: content "foo.img" overlaps with preceding image "bar.img"`)
 }
@@ -461,16 +618,19 @@ volumes:
 	c.Assert(vol.Structure, HasLen, 1)
 	c.Assert(vol.Structure[0].Content, HasLen, 2)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(err, IsNil)
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
-		Volume:  vol,
-		Size:    3 * quantity.SizeMiB,
-		RootDir: p.dir,
+		Volume: vol,
 		LaidOutStructure: []gadget.LaidOutStructure{
 			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:        "00000000-0000-0000-0000-0000deadbeef",
+					Size:        2 * quantity.SizeMiB,
+					StartOffset: 1 * quantity.OffsetMiB,
+				},
 				VolumeStructure: &vol.Structure[0],
-				StartOffset:     1 * quantity.OffsetMiB,
 				LaidOutContent: []gadget.LaidOutContent{
 					{
 						VolumeContent: &vol.Structure[0].Content[1],
@@ -512,16 +672,19 @@ volumes:
 	c.Assert(vol.Structure, HasLen, 1)
 	c.Assert(vol.Structure[0].Content, HasLen, 2)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(err, IsNil)
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
-		Volume:  vol,
-		Size:    3 * quantity.SizeMiB,
-		RootDir: p.dir,
+		Volume: vol,
 		LaidOutStructure: []gadget.LaidOutStructure{
 			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:        "00000000-0000-0000-0000-0000deadbeef",
+					Size:        2 * quantity.SizeMiB,
+					StartOffset: 1 * quantity.OffsetMiB,
+				},
 				VolumeStructure: &vol.Structure[0],
-				StartOffset:     1 * quantity.OffsetMiB,
 				LaidOutContent: []gadget.LaidOutContent{
 					{
 						VolumeContent: &vol.Structure[0].Content[0],
@@ -560,16 +723,19 @@ volumes:
 	c.Assert(vol.Structure, HasLen, 1)
 	c.Assert(vol.Structure[0].Content, HasLen, 1)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(err, IsNil)
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
-		Volume:  vol,
-		Size:    3 * quantity.SizeMiB,
-		RootDir: p.dir,
+		Volume: vol,
 		LaidOutStructure: []gadget.LaidOutStructure{
 			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:        "00000000-0000-0000-0000-0000deadbeef",
+					Size:        2 * quantity.SizeMiB,
+					StartOffset: 1 * quantity.OffsetMiB,
+				},
 				VolumeStructure: &vol.Structure[0],
-				StartOffset:     1 * quantity.OffsetMiB,
 				LaidOutContent: []gadget.LaidOutContent{
 					{
 						VolumeContent: &vol.Structure[0].Content[0],
@@ -602,15 +768,19 @@ volumes:
 	c.Assert(vol.Structure, HasLen, 1)
 	c.Assert(vol.Structure[0].Content, HasLen, 1)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(err, IsNil)
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
-		Volume:  vol,
-		Size:    3 * quantity.SizeMiB,
-		RootDir: p.dir,
+		Volume: vol,
 		LaidOutStructure: []gadget.LaidOutStructure{
 			{
-				StartOffset:     1 * quantity.OffsetMiB,
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:            "00000000-0000-0000-0000-0000deadbeef",
+					PartitionFSType: "ext4",
+					Size:            2 * quantity.SizeMiB,
+					StartOffset:     1 * quantity.OffsetMiB,
+				},
 				VolumeStructure: &vol.Structure[0],
 				ResolvedContent: []gadget.ResolvedContent{
 					{
@@ -626,115 +796,7 @@ volumes:
 	})
 }
 
-func (p *layoutTestSuite) TestLayoutVolumeConstraintsChange(c *C) {
-	gadgetYaml := `
-volumes:
-  first:
-    schema: gpt
-    bootloader: grub
-    structure:
-        - role: mbr
-          type: bare
-          size: 446
-          offset: 0
-        - type: 00000000-0000-0000-0000-0000deadbeef
-          filesystem: ext4
-          size: 2M
-          content:
-              - source: foo.txt
-                target: /boot
-`
-	resolvedContent := []gadget.ResolvedContent{
-		{
-			VolumeContent: &gadget.VolumeContent{
-				UnresolvedSource: "foo.txt",
-				Target:           "/boot",
-			},
-			ResolvedSource: filepath.Join(p.dir, "foo.txt"),
-		},
-	}
-
-	makeSizedFile(c, filepath.Join(p.dir, "foo.txt"), 0, []byte("foobar\n"))
-
-	vol := mustParseVolume(c, gadgetYaml, "first")
-	c.Assert(vol.Structure, HasLen, 2)
-	c.Assert(vol.Structure[1].Content, HasLen, 1)
-
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
-	c.Assert(err, IsNil)
-	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
-		Volume:  vol,
-		Size:    3 * quantity.SizeMiB,
-		RootDir: p.dir,
-		LaidOutStructure: []gadget.LaidOutStructure{
-			{
-				VolumeStructure: &vol.Structure[0],
-				StartOffset:     0,
-				YamlIndex:       0,
-			},
-			{
-				VolumeStructure: &vol.Structure[1],
-				StartOffset:     1 * quantity.OffsetMiB,
-				YamlIndex:       1,
-				ResolvedContent: resolvedContent,
-			},
-		},
-	})
-
-	// still valid
-	constraints := gadget.LayoutConstraints{
-		// 512kiB
-		NonMBRStartOffset: 512 * quantity.OffsetKiB,
-	}
-	v, err = gadget.LayoutVolume(p.dir, "", vol, constraints)
-	c.Assert(err, IsNil)
-	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
-		Volume:  vol,
-		Size:    2*quantity.SizeMiB + 512*quantity.SizeKiB,
-		RootDir: p.dir,
-		LaidOutStructure: []gadget.LaidOutStructure{
-			{
-				VolumeStructure: &vol.Structure[0],
-				StartOffset:     0,
-				YamlIndex:       0,
-			},
-			{
-				VolumeStructure: &vol.Structure[1],
-				StartOffset:     512 * quantity.OffsetKiB,
-				YamlIndex:       1,
-				ResolvedContent: resolvedContent,
-			},
-		},
-	})
-
-	// constraints would make a non MBR structure overlap with MBR, but
-	// structures start one after another unless offset is specified
-	// explicitly
-	constraintsBad := gadget.LayoutConstraints{
-		NonMBRStartOffset: 400,
-	}
-	v, err = gadget.LayoutVolume(p.dir, "", vol, constraintsBad)
-	c.Assert(err, IsNil)
-	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
-		Volume:  vol,
-		Size:    2*quantity.SizeMiB + 446,
-		RootDir: p.dir,
-		LaidOutStructure: []gadget.LaidOutStructure{
-			{
-				VolumeStructure: &vol.Structure[0],
-				YamlIndex:       0,
-			},
-			{
-				VolumeStructure: &vol.Structure[1],
-				StartOffset:     446,
-				YamlIndex:       1,
-				ResolvedContent: resolvedContent,
-			},
-		},
-	})
-}
-
-func (p *layoutTestSuite) TestLayoutVolumeMBRImplicitConstraints(c *C) {
+func (p *layoutTestSuite) TestLayoutVolumeMBR(c *C) {
 	gadgetYaml := `
 volumes:
   first:
@@ -752,22 +814,29 @@ volumes:
 	vol := mustParseVolume(c, gadgetYaml, "first")
 	c.Assert(vol.Structure, HasLen, 2)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(err, IsNil)
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
-		Volume:  vol,
-		Size:    2 * quantity.SizeMiB,
-		RootDir: p.dir,
+		Volume: vol,
 		LaidOutStructure: []gadget.LaidOutStructure{
 			{
 				// MBR
+				OnDiskStructure: gadget.OnDiskStructure{
+					Name:        "mbr",
+					Type:        "bare",
+					Size:        quantity.Size(446),
+					StartOffset: 0,
+				},
 				VolumeStructure: &vol.Structure[0],
-				StartOffset:     0,
-				YamlIndex:       0,
 			}, {
+				OnDiskStructure: gadget.OnDiskStructure{
+					Name:        "other",
+					Type:        "00000000-0000-0000-0000-0000deadbeef",
+					Size:        quantity.SizeMiB,
+					StartOffset: 1 * quantity.OffsetMiB,
+				},
 				VolumeStructure: &vol.Structure[1],
-				StartOffset:     1 * quantity.OffsetMiB,
-				YamlIndex:       1,
 			},
 		},
 	})
@@ -804,48 +873,51 @@ volumes:
 	vol := mustParseVolume(c, gadgetYaml, "pc")
 	c.Assert(vol.Structure, HasLen, 3)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(err, IsNil)
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
-		Volume:  vol,
-		Size:    3 * quantity.SizeMiB,
-		RootDir: p.dir,
+		Volume: vol,
 		LaidOutStructure: []gadget.LaidOutStructure{
 			{
 				// mbr
+				OnDiskStructure: gadget.OnDiskStructure{
+					Name:        "mbr",
+					Type:        "mbr",
+					Size:        quantity.Size(440),
+					StartOffset: 0,
+				},
 				VolumeStructure: &vol.Structure[0],
-				StartOffset:     0,
-				YamlIndex:       0,
 			}, {
 				// foo
+				OnDiskStructure: gadget.OnDiskStructure{
+					Name:        "foo",
+					Type:        "DA,21686148-6449-6E6F-744E-656564454649",
+					Size:        quantity.SizeMiB,
+					StartOffset: 1 * quantity.OffsetMiB,
+				},
 				VolumeStructure: &vol.Structure[1],
-				StartOffset:     1 * quantity.OffsetMiB,
-				YamlIndex:       1,
-				// break for gofmt < 1.11
-				AbsoluteOffsetWrite: asOffsetPtr(92),
 				LaidOutContent: []gadget.LaidOutContent{
 					{
 						VolumeContent: &vol.Structure[1].Content[0],
 						Size:          200 * quantity.SizeKiB,
 						StartOffset:   1 * quantity.OffsetMiB,
-						// offset-write: bar+10
-						AbsoluteOffsetWrite: asOffsetPtr(2*quantity.OffsetMiB + 10),
 					},
 				},
 			}, {
 				// bar
+				OnDiskStructure: gadget.OnDiskStructure{
+					Name:        "bar",
+					Type:        "DA,21686148-6449-6E6F-744E-656564454649",
+					Size:        quantity.SizeMiB,
+					StartOffset: 2 * quantity.OffsetMiB,
+				},
 				VolumeStructure: &vol.Structure[2],
-				StartOffset:     2 * quantity.OffsetMiB,
-				YamlIndex:       2,
-				// break for gofmt < 1.11
-				AbsoluteOffsetWrite: asOffsetPtr(600),
 				LaidOutContent: []gadget.LaidOutContent{
 					{
 						VolumeContent: &vol.Structure[2].Content[0],
 						Size:          150 * quantity.SizeKiB,
 						StartOffset:   2 * quantity.OffsetMiB,
-						// offset-write: bar+10
-						AbsoluteOffsetWrite: asOffsetPtr(450),
 					},
 				},
 			},
@@ -858,9 +930,10 @@ func (p *layoutTestSuite) TestLayoutVolumeOffsetWriteBadRelativeTo(c *C) {
 	volBadStructure := gadget.Volume{
 		Structure: []gadget.VolumeStructure{
 			{
-				Name: "foo",
-				Type: "DA,21686148-6449-6E6F-744E-656564454649",
-				Size: 1 * quantity.SizeMiB,
+				Name:   "foo",
+				Type:   "DA,21686148-6449-6E6F-744E-656564454649",
+				Offset: asOffsetPtr(0),
+				Size:   1 * quantity.SizeMiB,
 				OffsetWrite: &gadget.RelativeOffset{
 					RelativeTo: "bar",
 					Offset:     10,
@@ -868,94 +941,13 @@ func (p *layoutTestSuite) TestLayoutVolumeOffsetWriteBadRelativeTo(c *C) {
 			},
 		},
 	}
-	volBadContent := gadget.Volume{
-		Structure: []gadget.VolumeStructure{
-			{
-				Name: "foo",
-				Type: "DA,21686148-6449-6E6F-744E-656564454649",
-				Size: 1 * quantity.SizeMiB,
-				Content: []gadget.VolumeContent{
-					{
-						Image: "foo.img",
-						OffsetWrite: &gadget.RelativeOffset{
-							RelativeTo: "bar",
-							Offset:     10,
-						},
-					},
-				},
-			},
-		},
-	}
 
 	makeSizedFile(c, filepath.Join(p.dir, "foo.img"), 200*quantity.SizeKiB, []byte(""))
 
-	v, err := gadget.LayoutVolume(p.dir, "", &volBadStructure, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(&volBadStructure, opts)
 	c.Check(v, IsNil)
-	c.Check(err, ErrorMatches, `cannot resolve offset-write of structure #0 \("foo"\): refers to an unknown structure "bar"`)
-
-	v, err = gadget.LayoutVolume(p.dir, "", &volBadContent, defaultConstraints)
-	c.Check(v, IsNil)
-	c.Check(err, ErrorMatches, `cannot resolve offset-write of structure #0 \("foo"\) content "foo.img": refers to an unknown structure "bar"`)
-}
-
-func (p *layoutTestSuite) TestLayoutVolumeOffsetWriteEnlargesVolume(c *C) {
-	var gadgetYamlStructure = `
-volumes:
-  pc:
-    bootloader: grub
-    structure:
-      - name: mbr
-        type: mbr
-        size: 440
-      - name: foo
-        type: DA,21686148-6449-6E6F-744E-656564454649
-        size: 1M
-        offset: 1M
-        # 1GB
-        offset-write: mbr+1073741824
-
-`
-	vol := mustParseVolume(c, gadgetYamlStructure, "pc")
-
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
-	c.Assert(err, IsNil)
-	// offset-write is at 1GB
-	c.Check(v.Size, Equals, 1*quantity.SizeGiB+gadget.SizeLBA48Pointer)
-
-	var gadgetYamlContent = `
-volumes:
-  pc:
-    bootloader: grub
-    structure:
-      - name: mbr
-        type: mbr
-        size: 440
-      - name: foo
-        type: DA,21686148-6449-6E6F-744E-656564454649
-        size: 1M
-        offset: 1M
-        content:
-          - image: foo.img
-            # 2GB
-            offset-write: mbr+2147483648
-          - image: bar.img
-            # 1GB
-            offset-write: mbr+1073741824
-          - image: baz.img
-            # 3GB
-            offset-write: mbr+3221225472
-
-`
-	makeSizedFile(c, filepath.Join(p.dir, "foo.img"), 200*quantity.SizeKiB, []byte(""))
-	makeSizedFile(c, filepath.Join(p.dir, "bar.img"), 150*quantity.SizeKiB, []byte(""))
-	makeSizedFile(c, filepath.Join(p.dir, "baz.img"), 100*quantity.SizeKiB, []byte(""))
-
-	vol = mustParseVolume(c, gadgetYamlContent, "pc")
-
-	v, err = gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
-	c.Assert(err, IsNil)
-	// foo.img offset-write is at 3GB
-	c.Check(v.Size, Equals, 3*quantity.SizeGiB+gadget.SizeLBA48Pointer)
+	c.Check(err, ErrorMatches, `structure "foo" refers to an unexpected structure "bar"`)
 }
 
 func (p *layoutTestSuite) TestLayoutVolumePartialNoSuchFile(c *C) {
@@ -974,14 +966,17 @@ volumes:
 	vol := mustParseVolume(c, gadgetYaml, "first")
 	c.Assert(vol.Structure, HasLen, 1)
 
-	v, err := gadget.LayoutVolumePartially(vol, defaultConstraints)
+	v, err := gadget.LayoutVolumePartially(vol)
 	c.Assert(v, DeepEquals, &gadget.PartiallyLaidOutVolume{
 		Volume: vol,
 		LaidOutStructure: []gadget.LaidOutStructure{
 			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					Type:        "00000000-0000-0000-0000-dd00deadbeef",
+					Size:        400 * quantity.SizeMiB,
+					StartOffset: 800 * quantity.OffsetMiB,
+				},
 				VolumeStructure: &vol.Structure[0],
-				StartOffset:     800 * quantity.OffsetMiB,
-				YamlIndex:       0,
 			},
 		},
 	})
@@ -1010,7 +1005,8 @@ volumes:
 
 	vol := mustParseVolume(c, gadgetYamlContent, "pc")
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(err, IsNil)
 	c.Assert(v.LaidOutStructure, HasLen, 1)
 	c.Assert(v.LaidOutStructure[0].LaidOutContent, HasLen, 2)
@@ -1019,9 +1015,13 @@ volumes:
 
 	c.Assert(ps, DeepEquals, gadget.LaidOutStructure{
 		// foo
+		OnDiskStructure: gadget.OnDiskStructure{
+			Name:        "foo",
+			Type:        "DA,21686148-6449-6E6F-744E-656564454649",
+			Size:        quantity.SizeMiB,
+			StartOffset: 1 * quantity.OffsetMiB,
+		},
 		VolumeStructure: &vol.Structure[0],
-		StartOffset:     1 * quantity.OffsetMiB,
-		YamlIndex:       0,
 		LaidOutContent: []gadget.LaidOutContent{
 			{
 				VolumeContent: &vol.Structure[0].Content[0],
@@ -1040,9 +1040,13 @@ volumes:
 	shiftedTo0 := gadget.ShiftStructureTo(ps, 0)
 	c.Assert(shiftedTo0, DeepEquals, gadget.LaidOutStructure{
 		// foo
+		OnDiskStructure: gadget.OnDiskStructure{
+			Name:        "foo",
+			Type:        "DA,21686148-6449-6E6F-744E-656564454649",
+			Size:        quantity.SizeMiB,
+			StartOffset: 0,
+		},
 		VolumeStructure: &vol.Structure[0],
-		StartOffset:     0,
-		YamlIndex:       0,
 		LaidOutContent: []gadget.LaidOutContent{
 			{
 				VolumeContent: &vol.Structure[0].Content[0],
@@ -1061,9 +1065,13 @@ volumes:
 	shiftedTo2M := gadget.ShiftStructureTo(ps, 2*quantity.OffsetMiB)
 	c.Assert(shiftedTo2M, DeepEquals, gadget.LaidOutStructure{
 		// foo
+		OnDiskStructure: gadget.OnDiskStructure{
+			Name:        "foo",
+			Type:        "DA,21686148-6449-6E6F-744E-656564454649",
+			Size:        quantity.SizeMiB,
+			StartOffset: 2 * quantity.OffsetMiB,
+		},
 		VolumeStructure: &vol.Structure[0],
-		StartOffset:     2 * quantity.OffsetMiB,
-		YamlIndex:       0,
 		LaidOutContent: []gadget.LaidOutContent{
 			{
 				VolumeContent: &vol.Structure[0].Content[0],
@@ -1131,10 +1139,39 @@ func (p *layoutTestSuite) TestResolveContentPathsNotInWantedAssets(c *C) {
 	c.Assert(vol.Structure, HasLen, 1)
 
 	kernelSnapDir := c.MkDir()
-	_, err := gadget.LayoutVolume(p.dir, kernelSnapDir, vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir, KernelRootDir: kernelSnapDir}
+	_, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(err, ErrorMatches, `cannot resolve content for structure #0 at index 0: cannot find "dtbs" in kernel info from "/.*"`)
 }
 
+func (p *layoutTestSuite) TestResolveContentPathsSkipResolveContent(c *C) {
+	vol := mustParseVolume(c, gadgetYamlWithKernelRef, "pi")
+	c.Assert(vol.Structure, HasLen, 1)
+
+	kernelSnapDir := c.MkDir()
+	defaultOpts := &gadget.LayoutOptions{GadgetRootDir: p.dir, KernelRootDir: kernelSnapDir}
+
+	opts := defaultOpts
+	_, err := gadget.LayoutVolume(vol, opts)
+	c.Assert(err, ErrorMatches, `cannot resolve content for structure #0 at index 0: cannot find "dtbs" in kernel info from "/.*"`)
+
+	// SkipResolveContent will allow to layout the volume even if
+	// files are missing
+	opts = defaultOpts
+	opts.SkipResolveContent = true
+	v, err := gadget.LayoutVolume(vol, opts)
+	c.Assert(err, IsNil)
+	c.Assert(v.Structure, HasLen, 1)
+
+	// As does IgnoreContent will allow to layout the volume even if
+	// files are missing
+	opts = defaultOpts
+	opts.IgnoreContent = true
+	v, err = gadget.LayoutVolume(vol, opts)
+	c.Assert(err, IsNil)
+	c.Assert(v.Structure, HasLen, 1)
+}
+
 func (p *layoutTestSuite) TestResolveContentPathsErrorInKernelRef(c *C) {
 	// create invalid kernel ref
 	s := strings.Replace(gadgetYamlWithKernelRef, "$kernel:dtbs", "$kernel:-invalid-kernel-ref", -1)
@@ -1144,7 +1181,8 @@ func (p *layoutTestSuite) TestResolveContentPathsErrorInKernelRef(c *C) {
 	c.Assert(vol.Structure, HasLen, 1)
 
 	kernelSnapDir := c.MkDir()
-	_, err := gadget.LayoutVolume(p.dir, kernelSnapDir, vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir, KernelRootDir: kernelSnapDir}
+	_, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(err, ErrorMatches, `cannot resolve content for structure #0 at index 0: cannot parse kernel ref: invalid asset name in kernel ref "\$kernel:-invalid-kernel-ref/boot-assets/"`)
 }
 
@@ -1163,7 +1201,8 @@ assets:
 	kernelSnapDir := mockKernel(c, kernelYaml, map[string]string{
 		"dtbs/foo.dtb": "foo.dtb content",
 	})
-	_, err := gadget.LayoutVolume(p.dir, kernelSnapDir, vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir, KernelRootDir: kernelSnapDir}
+	_, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(err, ErrorMatches, `cannot resolve content for structure #0 at index 0: cannot find wanted kernel content "boot-assets/" in "/.*"`)
 }
 
@@ -1184,7 +1223,8 @@ assets:
 		"some-file":       "some-file content",
 	}
 	kernelSnapDir := mockKernel(c, kernelYaml, kernelSnapFiles)
-	lv, err := gadget.LayoutVolume(p.dir, kernelSnapDir, vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir, KernelRootDir: kernelSnapDir}
+	lv, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(err, IsNil)
 	// Volume.Content is unchanged
 	c.Assert(lv.Structure, HasLen, 1)
@@ -1264,7 +1304,8 @@ assets:
 	kernelSnapDir := mockKernel(c, kernelYaml, map[string]string{
 		"dtbs/foo.dtb": "foo.dtb content",
 	})
-	lv, err := gadget.LayoutVolume(p.dir, kernelSnapDir, vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir, KernelRootDir: kernelSnapDir}
+	lv, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(err, IsNil)
 	// Volume.Content is unchanged
 	c.Assert(lv.Structure, HasLen, 1)
@@ -1333,6 +1374,48 @@ assets:
 	kernelSnapDir := mockKernel(c, kernelYaml, map[string]string{
 		"a/foo.dtb": "foo.dtb content",
 	})
-	_, err := gadget.LayoutVolume(p.dir, kernelSnapDir, vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir, KernelRootDir: kernelSnapDir}
+	_, err := gadget.LayoutVolume(vol, opts)
 	c.Assert(err, ErrorMatches, `.*: cannot find wanted kernel content "ab" in.*`)
 }
+
+func (p *layoutTestSuite) TestLayoutWithMinSize(c *C) {
+	vol := gadget.Volume{
+		Structure: []gadget.VolumeStructure{
+			{
+				Offset:          asOffsetPtr(quantity.OffsetMiB),
+				MinSize:         quantity.SizeMiB,
+				Size:            2 * quantity.SizeMiB,
+				EnclosingVolume: &gadget.Volume{},
+			},
+			{
+				MinSize:         quantity.SizeMiB,
+				Size:            2 * quantity.SizeMiB,
+				EnclosingVolume: &gadget.Volume{},
+			},
+		},
+	}
+	v, err := gadget.LayoutVolume(&vol, nil)
+	c.Assert(err, IsNil)
+
+	// Check StartOffset and Size is well defined even if using min-size
+	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
+		Volume: &vol,
+		LaidOutStructure: []gadget.LaidOutStructure{
+			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					StartOffset: 1 * quantity.OffsetMiB,
+					Size:        2 * quantity.SizeMiB,
+				},
+				VolumeStructure: &vol.Structure[0],
+			},
+			{
+				OnDiskStructure: gadget.OnDiskStructure{
+					StartOffset: 3 * quantity.OffsetMiB,
+					Size:        2 * quantity.SizeMiB,
+				},
+				VolumeStructure: &vol.Structure[1],
+			},
+		},
+	})
+}
