@@ -104,13 +104,17 @@ var privKey, _ = rsa.GenerateKey(rand.Reader, 768)
 
 // see crypto/tls/generate_cert.go
 func generateTestCert(c *check.C, certpath, keypath string) {
+	generateTestCertWithDates(c, certpath, keypath, time.Now(), time.Now().Add(24*time.Hour))
+}
+
+func generateTestCertWithDates(c *check.C, certpath, keypath string, notBefore, notAfter time.Time) {
 	template := x509.Certificate{
 		SerialNumber: big.NewInt(123456789),
 		Subject: pkix.Name{
 			Organization: []string{"Snapd testers"},
 		},
-		NotBefore:   time.Now(),
-		NotAfter:    time.Now().Add(24 * time.Hour),
+		NotBefore:   notBefore,
+		NotAfter:    notAfter,
 		IPAddresses: []net.IP{net.IPv4(127, 0, 0, 1)},
 		DNSNames:    []string{"localhost"},
 		IsCA:        true,
@@ -255,6 +259,7 @@ func (s *tlsSuite) TestClientMaxTLS11Error(c *check.C) {
 	// - golang < 1.12: tls: server selected unsupported protocol version 302
 	// - golang >= 1.12: tls: protocol version not supported
 	c.Assert(err, check.ErrorMatches, ".* tls: (server selected unsupported protocol version 302|protocol version not supported)")
+	c.Check(httputil.IsCertExpiredOrNotValidYetError(err), check.Equals, false)
 }
 
 func (s *tlsSuite) TestClientMaxTLS12Ok(c *check.C) {
@@ -282,4 +287,30 @@ func (s *tlsSuite) TestClientMaxTLS12Ok(c *check.C) {
 	// this is testing the protocol, the self-signed certificate error is
 	// fine and expected.
 	c.Assert(err, check.ErrorMatches, ".* certificate signed by unknown authority")
+	c.Check(httputil.IsCertExpiredOrNotValidYetError(err), check.Equals, false)
+}
+
+func (s *tlsSuite) TestCertExpireOrNotValidYet(c *check.C) {
+	generateTestCertWithDates(c, s.certpath, s.keypath, time.Time{}, time.Time{})
+
+	// create a server that uses our certs
+	srv := httptest.NewUnstartedServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		io.WriteString(w, `all good`)
+	}))
+	cert, err := tls.LoadX509KeyPair(s.certpath, s.keypath)
+	c.Assert(err, check.IsNil)
+	srv.TLS = &tls.Config{
+		Certificates: []tls.Certificate{cert},
+	}
+	srv.StartTLS()
+	s.AddCleanup(srv.Close)
+
+	// Server running only TLS1.1 doesn't work
+	cli := httputil.NewHTTPClient(nil)
+	c.Assert(cli, check.NotNil)
+	c.Assert(s.logbuf.String(), check.Equals, "")
+
+	_, err = cli.Get(srv.URL)
+	c.Assert(err, check.ErrorMatches, ".*: x509: certificate has expired or is not yet valid.*")
+	c.Check(httputil.IsCertExpiredOrNotValidYetError(err), check.Equals, true)
 }
