@@ -118,9 +118,9 @@ var u2fDevices = []u2fDevice{
 		ProductIDPattern: "5026",
 	},
 	{
-		Name:             "Tomu board + chopstx U2F + SoloKeys",
+		Name:             "Tomu board + chopstx U2F + SoloKeys + Flipper zero",
 		VendorIDPattern:  "0483",
-		ProductIDPattern: "cdab|a2ca",
+		ProductIDPattern: "cdab|a2ca|5741",
 	},
 	{
 		Name:             "SoloKeys",
@@ -132,6 +132,11 @@ var u2fDevices = []u2fDevice{
 		VendorIDPattern:  "1d50",
 		ProductIDPattern: "60fc",
 	},
+	{
+		Name:             "Thetis U2F BT Fido2 Key",
+		VendorIDPattern:  "1ea8",
+		ProductIDPattern: "fc25",
+	},
 	{
 		Name:             "MIRKey",
 		VendorIDPattern:  "0483",
