@@ -51,6 +51,27 @@ var abstractCalculateLastUsableLBA = func(device string, diskSize uint64, sector
 	return CalculateLastUsableLBA(device, diskSize, sectorSize)
 }
 
+func MockUdevPropertiesForDevice(new func(string, string) (map[string]string, error)) (restore func()) {
+	old := udevadmProperties
+	// for better testing we mock the udevadm command output so that we still
+	// test the parsing
+	udevadmProperties = func(typeOpt, dev string) ([]byte, error) {
+		props, err := new(typeOpt, dev)
+		if err != nil {
+			return []byte(err.Error()), err
+		}
+		// put it into udevadm format output, i.e. "KEY=VALUE\n"
+		output := ""
+		for k, v := range props {
+			output += fmt.Sprintf("%s=%s\n", k, v)
+		}
+		return []byte(output), nil
+	}
+	return func() {
+		udevadmProperties = old
+	}
+}
+
 func parseDeviceMajorMinor(s string) (int, int, error) {
 	errMsg := fmt.Errorf("invalid device number format: (expected <int>:<int>)")
 	devNums := strings.SplitN(s, ":", 2)
@@ -856,7 +877,7 @@ func (d *disk) FindMatchingPartitionWithFsLabel(label string) (Partition, error)
 	}
 
 	for _, p := range d.partitions {
-		if p.FilesystemLabel == encodedLabel {
+		if p.hasFilesystemLabel(encodedLabel) {
 			return p, nil
 		}
 	}
@@ -983,3 +1004,15 @@ func PartitionUUID(node string) (string, error) {
 func SectorSize(devname string) (uint64, error) {
 	return blockDeviceSectorSize(devname)
 }
+
+// filesystemTypeForPartition returns the filesystem type for a
+// partition passed by device name. The type might be an empty string
+// if no filesystem has been detected.
+func filesystemTypeForPartition(devname string) (string, error) {
+	props, err := udevPropertiesForName(devname)
+	if err != nil {
+		return "", err
+	}
+
+	return props["ID_FS_TYPE"], nil
+}
