@@ -20,6 +20,8 @@
 package devicestate_test
 
 import (
+	"crypto/x509"
+	"errors"
 	"fmt"
 	"io/ioutil"
 	"net"
@@ -110,7 +112,13 @@ func (s *deviceMgrSerialSuite) mockServer(c *C, reqID string, bhv *devicestatete
 	bhv.SignSerial = s.signSerial
 	bhv.ExpectedCapabilities = "serial-stream"
 
-	return devicestatetest.MockDeviceService(c, bhv)
+	mockServer, extraCerts := devicestatetest.MockDeviceService(c, bhv)
+	fname := filepath.Join(dirs.SnapdStoreSSLCertsDir, "test-server-certs.pem")
+	err := os.MkdirAll(filepath.Dir(fname), 0755)
+	c.Assert(err, IsNil)
+	err = ioutil.WriteFile(fname, extraCerts, 0644)
+	c.Assert(err, IsNil)
+	return mockServer
 }
 
 func (s *deviceMgrSerialSuite) findBecomeOperationalChange(skipIDs ...string) *state.Change {
@@ -677,7 +685,7 @@ func (s *deviceMgrSerialSuite) TestDoRequestSerialIdempotentAfterGotSerial(c *C)
 		"model":    "pc",
 		"serial":   "9999",
 	})
-	c.Assert(asserts.IsNotFound(err), Equals, true)
+	c.Assert(errors.Is(err, &asserts.NotFoundError{}), Equals, true)
 
 	s.state.Unlock()
 	s.se.Ensure()
@@ -846,30 +854,55 @@ func (s *deviceMgrSerialSuite) TestDoRequestSerialNoReachableDNS(c *C) {
 }
 
 func (s *deviceMgrSerialSuite) testDoRequestSerialKeepsRetrying(c *C, rt http.RoundTripper) {
+	chg, t := s.makeRequestChangeWithTransport(c, rt)
+
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	// ensure we keep trying even if we are well above maxTentative
+	for i := 0; i < 10; i++ {
+		s.state.Unlock()
+		s.se.Ensure()
+		s.se.Wait()
+		s.state.Lock()
+
+		c.Check(chg.Status(), Equals, state.DoingStatus)
+		c.Assert(chg.Err(), IsNil)
+	}
+
+	c.Check(chg.Status(), Equals, state.DoingStatus)
+
+	var nTentatives int
+	err := t.Get("pre-poll-tentatives", &nTentatives)
+	c.Assert(err, IsNil)
+	c.Check(nTentatives, Equals, 0)
+}
+
+func (s *deviceMgrSerialSuite) makeRequestChangeWithTransport(c *C, rt http.RoundTripper) (*state.Change, *state.Task) {
 	privKey, _ := assertstest.GenerateKey(testKeyLength)
 
 	// immediate
 	r := devicestate.MockRetryInterval(0)
-	defer r()
+	s.AddCleanup(r)
 
 	// set a low maxRetry value
 	r = devicestate.MockMaxTentatives(3)
-	defer r()
+	s.AddCleanup(r)
 
 	mockServer := s.mockServer(c, "REQID-1", nil)
-	defer mockServer.Close()
+	s.AddCleanup(mockServer.Close)
 
 	restore := devicestate.MockBaseStoreURL(mockServer.URL)
-	defer restore()
+	s.AddCleanup(restore)
 
 	restore = devicestate.MockRepeatRequestSerial("after-add-serial")
-	defer restore()
+	s.AddCleanup(restore)
 
 	restore = devicestate.MockHttputilNewHTTPClient(func(opts *httputil.ClientOptions) *http.Client {
 		c.Check(opts.ProxyConnectHeader, NotNil)
 		return &http.Client{Transport: rt}
 	})
-	defer restore()
+	s.AddCleanup(restore)
 
 	// setup state as done by first-boot/Ensure/doGenerateDeviceKey
 	s.state.Lock()
@@ -897,23 +930,44 @@ func (s *deviceMgrSerialSuite) testDoRequestSerialKeepsRetrying(c *C, rt http.Ro
 	// avoid full seeding
 	s.seeding()
 
-	// ensure we keep trying even if we are well above maxTentative
-	for i := 0; i < 10; i++ {
+	return chg, t
+}
+
+type simulateCertExpiredErrorRoundTripper struct{}
+
+func (s *simulateCertExpiredErrorRoundTripper) RoundTrip(r *http.Request) (*http.Response, error) {
+	return nil, x509.CertificateInvalidError{
+		Reason: x509.Expired,
+	}
+}
+
+func (s *deviceMgrSerialSuite) TestDoRequestSerialCertExpired(c *C) {
+	chg, t := s.makeRequestChangeWithTransport(c, &simulateCertExpiredErrorRoundTripper{})
+
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	// keep trying well beyond the 21 retry attempts we do
+	for i := 0; i < 100; i++ {
 		s.state.Unlock()
 		s.se.Ensure()
 		s.se.Wait()
 		s.state.Lock()
 
-		c.Check(chg.Status(), Equals, state.DoingStatus)
-		c.Check(chg.Err(), IsNil)
+		if chg.Status() == state.ErrorStatus {
+			break
+		}
 	}
 
-	c.Check(chg.Status(), Equals, state.DoingStatus)
+	c.Check(chg.Status(), Equals, state.ErrorStatus)
+	c.Assert(chg.Err(), ErrorMatches, `(?ms).*cannot retrieve request-id for making a request for a serial: Post \"?https://.*/request-id\"?: x509: certificate has expired or is not yet valid.*`)
 
 	var nTentatives int
 	err := t.Get("pre-poll-tentatives", &nTentatives)
 	c.Assert(err, IsNil)
-	c.Check(nTentatives, Equals, 0)
+	// this is one above maxTentativesCertExpired (35)
+	c.Check(nTentatives, Equals, 21)
+
 }
 
 func (s *deviceMgrSerialSuite) TestFullDeviceRegistrationPollHappy(c *C) {
@@ -1093,35 +1147,45 @@ func (s *deviceMgrSerialSuite) TestFullDeviceRegistrationHappyPrepareDeviceHook(
 }
 
 func (s *deviceMgrSerialSuite) TestFullDeviceRegistrationHappyWithHookAndNewProxy(c *C) {
-	s.testFullDeviceRegistrationHappyWithHookAndProxy(c, true)
+	s.testFullDeviceRegistrationHappyWithHookAndProxy(c, "new-enough")
 }
 
 func (s *deviceMgrSerialSuite) TestFullDeviceRegistrationHappyWithHookAndOldProxy(c *C) {
-	s.testFullDeviceRegistrationHappyWithHookAndProxy(c, false)
+	s.testFullDeviceRegistrationHappyWithHookAndProxy(c, "old-proxy")
 }
 
-func (s *deviceMgrSerialSuite) testFullDeviceRegistrationHappyWithHookAndProxy(c *C, newEnough bool) {
+func (s *deviceMgrSerialSuite) TestFullDeviceRegistrationHappyWithHookAndBrokenProxy(c *C) {
+	s.testFullDeviceRegistrationHappyWithHookAndProxy(c, "error-from-proxy")
+}
+
+func (s *deviceMgrSerialSuite) testFullDeviceRegistrationHappyWithHookAndProxy(c *C, proxyBehavior string) {
 	r1 := devicestate.MockKeyLength(testKeyLength)
 	defer r1()
 
 	var reqID string
 	var storeVersion string
 	head := func(c *C, bhv *devicestatetest.DeviceServiceBehavior, w http.ResponseWriter, r *http.Request) {
-		w.Header().Set("Snap-Store-Version", storeVersion)
+		switch proxyBehavior {
+		case "error-from-proxy":
+			w.WriteHeader(500)
+		default:
+			w.Header().Set("Snap-Store-Version", storeVersion)
+		}
 	}
 	bhv := &devicestatetest.DeviceServiceBehavior{
 		Head: head,
 	}
 	svcPath := "/svc/"
-	if newEnough {
+	switch proxyBehavior {
+	case "new-enough":
 		reqID = "REQID-42"
 		storeVersion = "6"
 		bhv.PostPreflight = func(c *C, bhv *devicestatetest.DeviceServiceBehavior, w http.ResponseWriter, r *http.Request) {
-			c.Check(r.Header.Get("X-Snap-Device-Service-URL"), Matches, "http://[^/]*/bad/svc/")
+			c.Check(r.Header.Get("X-Snap-Device-Service-URL"), Matches, "https://[^/]*/bad/svc/")
 			c.Check(r.Header.Get("X-Extra-Header"), Equals, "extra")
 		}
 		svcPath = "/bad/svc/"
-	} else {
+	case "old-proxy", "error-from-proxy":
 		reqID = "REQID-41"
 		storeVersion = "5"
 		bhv.RequestIDURLPath = "/svc/request-id"
@@ -1129,6 +1193,8 @@ func (s *deviceMgrSerialSuite) testFullDeviceRegistrationHappyWithHookAndProxy(c
 		bhv.PostPreflight = func(c *C, bhv *devicestatetest.DeviceServiceBehavior, w http.ResponseWriter, r *http.Request) {
 			c.Check(r.Header.Get("X-Extra-Header"), Equals, "extra")
 		}
+	default:
+		c.Fatalf("unknown proxy behavior %v", proxyBehavior)
 	}
 
 	mockServer := s.mockServer(c, reqID, bhv)
@@ -1642,14 +1708,13 @@ func (s *deviceMgrSerialSuite) TestNewEnoughProxyParse(c *C) {
 	s.state.Lock()
 	defer s.state.Unlock()
 
-	log, restore := logger.MockLogger()
-	defer restore()
 	os.Setenv("SNAPD_DEBUG", "1")
 	defer os.Unsetenv("SNAPD_DEBUG")
 
 	badURL := &url.URL{Opaque: "%a"} // url.Parse(badURL.String()) needs to fail, which isn't easy :-)
-	c.Check(devicestate.NewEnoughProxy(s.state, badURL, http.DefaultClient), Equals, false)
-	c.Check(log.String(), Matches, "(?m).* DEBUG: Cannot check whether proxy store supports a custom serial vault: parse .*")
+	newEnoughProxy, err := devicestate.NewEnoughProxy(s.state, badURL, http.DefaultClient)
+	c.Check(err, ErrorMatches, "cannot check whether proxy store supports a custom serial vault: parse .*")
+	c.Check(newEnoughProxy, Equals, false)
 }
 
 func (s *deviceMgrSerialSuite) TestNewEnoughProxy(c *C) {
@@ -1698,18 +1763,19 @@ func (s *deviceMgrSerialSuite) TestNewEnoughProxy(c *C) {
 	u, err := url.Parse(server.URL)
 	c.Assert(err, IsNil)
 	for _, expected := range expecteds {
-		log.Reset()
-		c.Check(devicestate.NewEnoughProxy(s.state, u, http.DefaultClient), Equals, false)
-		if len(expected) > 0 {
-			expected = "(?m).* DEBUG: Cannot check whether proxy store supports a custom serial vault: " + expected
+		newEnoughProxy, err := devicestate.NewEnoughProxy(s.state, u, http.DefaultClient)
+		if expected != "" {
+			expected = "cannot check whether proxy store supports a custom serial vault: " + expected
+			c.Check(err, ErrorMatches, expected)
 		}
-		c.Check(log.String(), Matches, expected)
+		c.Check(newEnoughProxy, Equals, false)
 	}
 	c.Check(n, Equals, len(expecteds))
 
 	// and success at last
-	log.Reset()
-	c.Check(devicestate.NewEnoughProxy(s.state, u, http.DefaultClient), Equals, true)
+	newEnoughProxy, err := devicestate.NewEnoughProxy(s.state, u, http.DefaultClient)
+	c.Check(err, IsNil)
+	c.Check(newEnoughProxy, Equals, true)
 	c.Check(log.String(), Equals, "")
 	c.Check(n, Equals, len(expecteds)+1)
 }
@@ -1919,6 +1985,8 @@ func (s *deviceMgrSerialSuite) TestFullDeviceRegistrationUC20Happy(c *C) {
 	r2 := devicestate.MockBaseStoreURL(mockServer.URL)
 	defer r2()
 
+	s.setUC20PCModelInState(c)
+
 	// setup state as will be done by first-boot
 	s.state.Lock()
 	defer s.state.Unlock()
