@@ -74,6 +74,45 @@ func criticalTaskEdges(ts *state.TaskSet) (beginEdge, beforeHooksEdge, hooksEdge
 	return beginEdge, beforeHooksEdge, hooksEdge, nil
 }
 
+// maybeEnforceValidationSetsTask returns a task for tracking validation-sets. This may
+// return nil if no validation-sets are present.
+func maybeEnforceValidationSetsTask(st *state.State, model *asserts.Model) (*state.Task, error) {
+	vsKey := func(accountID, name string) string {
+		return fmt.Sprintf("%s/%s", accountID, name)
+	}
+
+	// Encode validation-sets included in the seed
+	db := assertstate.DB(st)
+	as, err := db.FindMany(asserts.ValidationSetType, nil)
+	if err != nil {
+		// If none are included, then skip this
+		if errors.Is(err, &asserts.NotFoundError{}) {
+			return nil, nil
+		}
+		return nil, err
+	}
+
+	vsKeys := make(map[string][]string)
+	for _, a := range as {
+		vsa := a.(*asserts.ValidationSet)
+		vsKeys[vsKey(vsa.AccountID(), vsa.Name())] = a.Ref().PrimaryKey
+	}
+
+	// Set up pins from the model
+	pins := make(map[string]int)
+	for _, vs := range model.ValidationSets() {
+		key := vsKey(vs.AccountID, vs.Name)
+		if vs.Sequence > 0 {
+			pins[key] = vs.Sequence
+		}
+	}
+
+	t := st.NewTask("enforce-validation-sets", i18n.G("Track validation sets"))
+	t.Set("validation-set-keys", vsKeys)
+	t.Set("pinned-sequence-numbers", pins)
+	return t, nil
+}
+
 func markSeededTask(st *state.State) *state.Task {
 	return st.NewTask("mark-seeded", i18n.G("Mark system seeded"))
 }
@@ -87,24 +126,8 @@ func trivialSeeding(st *state.State) []*state.TaskSet {
 	return []*state.TaskSet{configTs, state.NewTaskSet(markSeeded)}
 }
 
-type populateStateFromSeedOptions struct {
-	Label   string
-	Mode    string
-	Preseed bool
-}
-
-func populateStateFromSeedImpl(st *state.State, opts *populateStateFromSeedOptions, tm timings.Measurer) ([]*state.TaskSet, error) {
-	mode := "run"
-	sysLabel := ""
-	preseed := false
-	if opts != nil {
-		if opts.Mode != "" {
-			mode = opts.Mode
-		}
-		sysLabel = opts.Label
-		preseed = opts.Preseed
-	}
-
+func (m *DeviceManager) populateStateFromSeedImpl(tm timings.Measurer) ([]*state.TaskSet, error) {
+	st := m.state
 	// check that the state is empty
 	var seeded bool
 	err := st.Get("seeded", &seeded)
@@ -115,10 +138,23 @@ func populateStateFromSeedImpl(st *state.State, opts *populateStateFromSeedOptio
 		return nil, fmt.Errorf("cannot populate state: already seeded")
 	}
 
+	preseed := m.preseed
+	sysLabel, mode, err := m.seedLabelAndMode()
+	if err != nil {
+		return nil, err
+	}
+	hasModeenv := false
+	if mode != "" {
+		hasModeenv = true
+	} else {
+		mode = "run"
+	}
+
 	var deviceSeed seed.Seed
 	// ack all initial assertions
 	timings.Run(tm, "import-assertions[finish]", "finish importing assertions from seed", func(nested timings.Measurer) {
-		deviceSeed, err = importAssertionsFromSeed(st, sysLabel)
+		isCoreBoot := hasModeenv || !release.OnClassic
+		deviceSeed, err = m.importAssertionsFromSeed(isCoreBoot)
 	})
 	if err != nil && err != errNothingToDo {
 		return nil, err
@@ -131,7 +167,8 @@ func populateStateFromSeedImpl(st *state.State, opts *populateStateFromSeedOptio
 	timings.Run(tm, "load-verified-snap-metadata", "load verified snap metadata from seed", func(nested timings.Measurer) {
 		err = deviceSeed.LoadMeta(mode, nil, nested)
 	})
-	if release.OnClassic && err == seed.ErrNoMeta {
+	// ErrNoMeta can happen only with Core 16/18-style seeds
+	if err == seed.ErrNoMeta && release.OnClassic {
 		if preseed {
 			return nil, fmt.Errorf("no snaps to preseed")
 		}
@@ -150,9 +187,6 @@ func populateStateFromSeedImpl(st *state.State, opts *populateStateFromSeedOptio
 		return nil, err
 	}
 
-	// optimistically forget the deviceSeed here
-	unloadDeviceSeed(st)
-
 	tsAll := []*state.TaskSet{}
 	configTss := []*state.TaskSet{}
 
@@ -227,6 +261,7 @@ func populateStateFromSeedImpl(st *state.State, opts *populateStateFromSeedOptio
 		configTss = chainTs(configTss, snapstate.ConfigureSnap(st, "core", snapstate.UseConfigDefaults))
 	}
 
+	modelIsDangerous := model.Grade() == asserts.ModelDangerous
 	for _, seedSnap := range essentialSeedSnaps {
 		flags := snapstate.Flags{
 			SkipConfigure: true,
@@ -237,7 +272,7 @@ func populateStateFromSeedImpl(st *state.State, opts *populateStateFromSeedOptio
 			// XXX: eventually we may need to allow specific snaps to be devmode for
 			// non-dangerous models, we can do that here since that information will
 			// probably be in the model assertion which we have here
-			ApplySnapDevMode: model.Grade() == asserts.ModelDangerous,
+			ApplySnapDevMode: modelIsDangerous,
 		}
 
 		ts, info, err := installSeedSnap(st, seedSnap, flags)
@@ -275,7 +310,10 @@ func populateStateFromSeedImpl(st *state.State, opts *populateStateFromSeedOptio
 			// XXX: eventually we may need to allow specific snaps to be devmode for
 			// non-dangerous models, we can do that here since that information will
 			// probably be in the model assertion which we have here
-			ApplySnapDevMode: model.Grade() == asserts.ModelDangerous,
+			ApplySnapDevMode: modelIsDangerous,
+			// for non-dangerous models snaps need to opt-in explicitly
+			// Classic is simply ignored for non-classic snaps, so we do not need to check further
+			Classic: release.OnClassic && modelIsDangerous,
 		}
 
 		ts, info, err := installSeedSnap(st, seedSnap, flags)
@@ -305,10 +343,18 @@ func populateStateFromSeedImpl(st *state.State, opts *populateStateFromSeedOptio
 	ts := tsAll[len(tsAll)-1]
 	endTs := state.NewTaskSet()
 
+	// Start tracking any validation sets included in the seed after
+	// installing the included snaps.
+	if trackVss, err := maybeEnforceValidationSetsTask(st, model); err != nil {
+		return nil, err
+	} else if trackVss != nil {
+		trackVss.WaitAll(ts)
+		endTs.AddTask(trackVss)
+	}
+
 	markSeeded := markSeededTask(st)
 	if preseed {
 		endTs.AddTask(preseedDoneTask)
-		markSeeded.WaitFor(preseedDoneTask)
 	}
 	whatSeeds := &seededSystem{
 		System:    sysLabel,
@@ -319,15 +365,19 @@ func populateStateFromSeedImpl(st *state.State, opts *populateStateFromSeedOptio
 	}
 	markSeeded.Set("seed-system", whatSeeds)
 
-	// mark-seeded waits for the taskset of last snap
+	// mark-seeded waits for the taskset of last snap, and
+	// for all the tasks in the endTs as well.
 	markSeeded.WaitAll(ts)
+	markSeeded.WaitAll(endTs)
 	endTs.AddTask(markSeeded)
 	tsAll = append(tsAll, endTs)
 
 	return tsAll, nil
 }
 
-func importAssertionsFromSeed(st *state.State, sysLabel string) (seed.Seed, error) {
+func (m *DeviceManager) importAssertionsFromSeed(isCoreBoot bool) (seed.Seed, error) {
+	st := m.state
+
 	// TODO: use some kind of context fo Device/SetDevice?
 	device, err := internal.Device(st)
 	if err != nil {
@@ -336,9 +386,9 @@ func importAssertionsFromSeed(st *state.State, sysLabel string) (seed.Seed, erro
 
 	// collect and
 	// set device,model from the model assertion
-	deviceSeed, err := loadDeviceSeed(st, sysLabel)
-	if err == seed.ErrNoAssertions && release.OnClassic {
-		// on classic seeding is optional
+	_, deviceSeed, err := m.earlyLoadDeviceSeed(nil)
+	if err == seed.ErrNoAssertions && !isCoreBoot {
+		// if classic boot seeding is optional
 		// set the fallback model
 		err := setClassicFallbackModel(st, device)
 		if err != nil {
@@ -352,6 +402,8 @@ func importAssertionsFromSeed(st *state.State, sysLabel string) (seed.Seed, erro
 	modelAssertion := deviceSeed.Model()
 
 	classicModel := modelAssertion.Classic()
+	// FIXME this will not be correct on classic with modes system when
+	// mode is not "run".
 	if release.OnClassic != classicModel {
 		var msg string
 		if classicModel {
@@ -370,29 +422,32 @@ func importAssertionsFromSeed(st *state.State, sysLabel string) (seed.Seed, erro
 	return deviceSeed, nil
 }
 
-// loadDeviceSeed loads and caches the device seed based on sysLabel,
-// it is meant to be used before and during seeding.
-// It is an error to call it with different sysLabel values once one
-// seed has been loaded and cached.
-var loadDeviceSeed = func(st *state.State, sysLabel string) (deviceSeed seed.Seed, err error) {
-	cached := st.Cached(loadedDeviceSeedKey{})
-	if cached != nil {
-		loaded := cached.(*loadedDeviceSeed)
-		if loaded.sysLabel != sysLabel {
-			return nil, fmt.Errorf("internal error: requested inconsistent device seed: %s (was %s)", sysLabel, loaded.sysLabel)
-		}
-		return loaded.seed, loaded.err
+// processAutoImportAssertions attempts to load the auto import assertions
+// and create all knows system users, if and only if the model grade is dangerous.
+// Processing of the auto-import assertion is opportunistic and can fail
+// for example if system-user-as is serial bound and there is no serial-as yet
+func processAutoImportAssertions(st *state.State, deviceSeed seed.Seed, db asserts.RODatabase, commitTo func(batch *asserts.Batch) error) error {
+	// only proceed for dangerous model
+	if deviceSeed.Model().Grade() != asserts.ModelDangerous {
+		return nil
 	}
+	seed20AssertionsLoader, ok := deviceSeed.(seed.AutoImportAssertionsLoaderSeed)
+	if !ok {
+		return fmt.Errorf("failed to auto-import assertions, invalid loader")
+	}
+	err := seed20AssertionsLoader.LoadAutoImportAssertions(commitTo)
+	if err != nil {
+		return err
+	}
+	// automatic user creation is meant to imply sudoers
+	const sudoer = true
+	_, err = createAllKnownSystemUsers(st, db, deviceSeed.Model(), nil, sudoer)
+	return err
+}
 
-	// cache the outcome, both success and errors
-	defer func() {
-		st.Cache(loadedDeviceSeedKey{}, &loadedDeviceSeed{
-			sysLabel: sysLabel,
-			seed:     deviceSeed,
-			err:      err,
-		})
-	}()
-
+// loadDeviceSeed loads the seed based on sysLabel,
+// It is meant to be called by DeviceManager.earlyLoadDeviceSeed.
+var loadDeviceSeed = func(st *state.State, sysLabel string) (deviceSeed seed.Seed, err error) {
 	deviceSeed, err = seed.Open(dirs.SnapSeedDir, sysLabel)
 	if err != nil {
 		return nil, err
@@ -416,17 +471,3 @@ var loadDeviceSeed = func(st *state.State, sysLabel string) (deviceSeed seed.See
 
 	return deviceSeed, nil
 }
-
-// unloadDeviceSeed forgets the cached outcomes of loadDeviceSeed.
-// Its main reason is to avoid using memory past the point where the deviceSeed
-// isn't needed anymore.
-func unloadDeviceSeed(st *state.State) {
-	st.Cache(loadedDeviceSeedKey{}, nil)
-}
-
-type loadedDeviceSeedKey struct{}
-type loadedDeviceSeed struct {
-	sysLabel string
-	seed     seed.Seed
-	err      error
-}
