@@ -57,7 +57,7 @@ var isConnectedTests = []struct {
 	exitCode            int
 }{{
 	args: []string{"is-connected"},
-	err:  "the required argument `<plug|slot>` was not provided",
+	err:  "must specify either a plug/slot name or --list",
 }, {
 	args: []string{"is-connected", "plug1"},
 }, {
@@ -287,3 +287,66 @@ plugs:
 	c.Check(string(stdout), Equals, "")
 	c.Check(string(stderr), Equals, "")
 }
+
+func (s *isConnectedSuite) TestIsConnectedList(c *C) {
+	s.st.Lock()
+	defer s.st.Unlock()
+
+	task := s.st.NewTask("test-task", "my test task")
+	setup := &hookstate.HookSetup{Snap: "snap1", Revision: snap.R(1), Hook: "test-hook"}
+
+	mockContext, err := hookstate.NewContext(task, s.st, setup, s.mockHandler, "")
+	c.Check(err, IsNil)
+
+	mockInstalledSnap(c, s.st, `name: snap1
+plugs:
+  plug1a:
+    interface: x11
+  plug1b:
+    interface: x11
+  plug1c:
+    interface: x11
+  plug1d:
+    interface: x11
+slots:
+  slot1a:
+    interface: x11
+  slot1b:
+    interface: x11`, "")
+	mockInstalledSnap(c, s.st, `name: snap2
+plugs:
+  plug2a:
+    interface: x11
+  plug2b:
+    interface: x11
+slots:
+  slot2a:
+    interface: x11
+  slot2b:
+    interface: x11`, "")
+	mockInstalledSnap(c, s.st, `name: snap3
+plugs:
+  plug3a:
+    interface: x11
+slots:
+  slot3a:
+    interface: x11
+  slot3b:
+    interface: x11`, "")
+	s.st.Set("conns", map[string]interface{}{
+		"snap1:plug1a snap2:slot2a": map[string]interface{}{},
+		"snap2:plug2a snap1:slot1a": map[string]interface{}{},
+		"snap3:plug3a snap1:slot1a": map[string]interface{}{},
+		"snap1:plug1c snap3:slot3a": map[string]interface{}{"undesired": true},
+		"snap1:plug1d snap3:slot3b": map[string]interface{}{"hotplug-gone": true},
+	})
+
+	s.st.Unlock()
+	defer s.st.Lock()
+
+	stdout, stderr, err := ctlcmd.Run(mockContext, []string{"is-connected", "--list"}, 0)
+	c.Check(err, IsNil)
+
+	c.Check(string(stdout), Equals, "plug1a\nslot1a\n")
+	c.Check(string(stderr), Equals, "")
+}
