@@ -70,10 +70,13 @@ func (s *snapshotSuite) SetUpTest(c *check.C) {
 	dirs.SetRootDir(c.MkDir())
 	os.MkdirAll(dirs.SnapshotsDir, os.ModePerm)
 
-	restore := snapstate.MockEnforcedValidationSets(func(st *state.State, extraVss ...*asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
-		return nil, nil
+	old := snapstate.EnforcedValidationSets
+	s.AddCleanup(func() {
+		snapstate.EnforcedValidationSets = old
 	})
-	s.AddCleanup(restore)
+	snapstate.EnforcedValidationSets = func(st *state.State, extraVss ...*asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
+		return nil, nil
+	}
 }
 
 func (s *snapshotSuite) TearDownTest(c *check.C) {
@@ -356,7 +359,7 @@ func (snapshotSuite) TestSaveChecksSnapnamesError(c *check.C) {
 	st := state.New(nil)
 	st.Lock()
 	defer st.Unlock()
-	_, _, _, err := snapshotstate.Save(st, nil, nil)
+	_, _, _, err := snapshotstate.Save(st, nil, nil, nil)
 	c.Check(err, check.ErrorMatches, "bzzt")
 }
 
@@ -421,7 +424,7 @@ func (s snapshotSuite) TestSaveChecksSnapstateConflict(c *check.C) {
 	st, restore := s.createConflictingChange(c)
 	defer restore()
 
-	_, _, _, err := snapshotstate.Save(st, []string{"foo"}, nil)
+	_, _, _, err := snapshotstate.Save(st, []string{"foo"}, nil, nil)
 	c.Assert(err, check.NotNil)
 	c.Check(err, check.FitsTypeOf, &snapstate.ChangeConflictError{})
 }
@@ -457,7 +460,7 @@ func (snapshotSuite) TestSaveConflictsWithSnapstate(c *check.C) {
 	})
 
 	chg := st.NewChange("snapshot-save", "...")
-	_, _, saveTasks, err := snapshotstate.Save(st, nil, nil)
+	_, _, saveTasks, err := snapshotstate.Save(st, nil, nil, nil)
 	c.Assert(err, check.IsNil)
 	chg.AddAll(saveTasks)
 
@@ -473,7 +476,7 @@ func (snapshotSuite) TestSaveChecksSnapstateConflictError(c *check.C) {
 	st := state.New(nil)
 	st.Lock()
 	defer st.Unlock()
-	_, _, _, err := snapshotstate.Save(st, nil, nil)
+	_, _, _, err := snapshotstate.Save(st, nil, nil, nil)
 	c.Check(err, check.ErrorMatches, "bzzt")
 }
 
@@ -484,7 +487,7 @@ func (snapshotSuite) TestSaveChecksSetIDError(c *check.C) {
 
 	st.Set("last-snapshot-set-id", "3/4")
 
-	_, _, _, err := snapshotstate.Save(st, nil, nil)
+	_, _, _, err := snapshotstate.Save(st, nil, nil, nil)
 	c.Check(err, check.ErrorMatches, ".* could not unmarshal .*")
 }
 
@@ -493,7 +496,7 @@ func (snapshotSuite) TestSaveNoSnapsInState(c *check.C) {
 	st.Lock()
 	defer st.Unlock()
 
-	setID, saved, taskset, err := snapshotstate.Save(st, nil, nil)
+	setID, saved, taskset, err := snapshotstate.Save(st, nil, nil, nil)
 	c.Assert(err, check.IsNil)
 	c.Check(setID, check.Equals, uint64(1))
 	c.Check(saved, check.HasLen, 0)
@@ -505,7 +508,7 @@ func (snapshotSuite) TestSaveSnapNotInstalled(c *check.C) {
 	st.Lock()
 	defer st.Unlock()
 
-	setID, saved, taskset, err := snapshotstate.Save(st, []string{"foo"}, nil)
+	setID, saved, taskset, err := snapshotstate.Save(st, []string{"foo"}, nil, nil)
 	c.Assert(err, check.ErrorMatches, `snap "foo" is not installed`)
 	c.Check(setID, check.Equals, uint64(0))
 	c.Check(saved, check.HasLen, 0)
@@ -527,7 +530,11 @@ func (snapshotSuite) TestSaveSomeSnaps(c *check.C) {
 	st.Lock()
 	defer st.Unlock()
 
-	setID, saved, taskset, err := snapshotstate.Save(st, nil, nil)
+	snapshotOptions := map[string]*snap.SnapshotOptions{
+		"a-snap": {Exclude: []string{"$SNAP_COMMON/exclude", "$SNAP_DATA/exclude"}},
+	}
+
+	setID, saved, taskset, err := snapshotstate.Save(st, nil, nil, snapshotOptions)
 	c.Assert(err, check.IsNil)
 	c.Check(setID, check.Equals, uint64(1))
 	c.Check(saved, check.DeepEquals, []string{"a-snap", "c-snap"})
@@ -535,8 +542,24 @@ func (snapshotSuite) TestSaveSomeSnaps(c *check.C) {
 	c.Assert(tasks, check.HasLen, 2)
 	c.Check(tasks[0].Kind(), check.Equals, "save-snapshot")
 	c.Check(tasks[0].Summary(), check.Equals, `Save data of snap "a-snap" in snapshot set #1`)
+
+	var snapshot [2]map[string]interface{}
+	c.Assert(tasks[0].Get("snapshot-setup", &snapshot[0]), check.IsNil)
+	c.Check(snapshot[0], check.DeepEquals, map[string]interface{}{
+		"set-id":  1.,
+		"snap":    "a-snap",
+		"options": map[string]interface{}{"exclude": []interface{}{"$SNAP_COMMON/exclude", "$SNAP_DATA/exclude"}},
+		"current": "unset",
+	})
+
 	c.Check(tasks[1].Kind(), check.Equals, "save-snapshot")
 	c.Check(tasks[1].Summary(), check.Equals, `Save data of snap "c-snap" in snapshot set #1`)
+	c.Assert(tasks[1].Get("snapshot-setup", &snapshot[1]), check.IsNil)
+	c.Check(snapshot[1], check.DeepEquals, map[string]interface{}{
+		"set-id":  1.,
+		"snap":    "c-snap",
+		"current": "unset",
+	})
 }
 
 func (s snapshotSuite) TestSaveOneSnap(c *check.C) {
@@ -557,7 +580,7 @@ func (s snapshotSuite) TestSaveOneSnap(c *check.C) {
 		Current: snap.R(1),
 	})
 
-	setID, saved, taskset, err := snapshotstate.Save(st, []string{"a-snap"}, []string{"a-user"})
+	setID, saved, taskset, err := snapshotstate.Save(st, []string{"a-snap"}, []string{"a-user"}, nil)
 	c.Assert(err, check.IsNil)
 	c.Check(setID, check.Equals, uint64(1))
 	c.Check(saved, check.DeepEquals, []string{"a-snap"})
@@ -607,6 +630,11 @@ func (snapshotSuite) TestSaveIntegration(c *check.C) {
 	st.Lock()
 	defer st.Unlock()
 
+	snapshotOptions := map[string]*snap.SnapshotOptions{
+		"one-snap": {Exclude: []string{"$SNAP_COMMON/exclude-a-snap", "$SNAP_DATA/exclude-a-snap"}},
+		"tri-snap": {Exclude: []string{"$SNAP_COMMON/exclude-b-snap", "$SNAP_DATA/exclude-b-snap"}},
+	}
+
 	snapshots := make(map[string]*client.Snapshot, 3)
 	for i, name := range []string{"one-snap", "too-snap", "tri-snap"} {
 		sideInfo := &snap.SideInfo{RealName: name, Revision: snap.R(i + 1)}
@@ -627,10 +655,11 @@ func (snapshotSuite) TestSaveIntegration(c *check.C) {
 			Version:  "v1",
 			Revision: sideInfo.Revision,
 			Epoch:    snap.E("0"),
+			Options:  snapshotOptions[name],
 		}
 	}
 
-	setID, saved, taskset, err := snapshotstate.Save(st, nil, []string{"a-user"})
+	setID, saved, taskset, err := snapshotstate.Save(st, nil, []string{"a-user"}, snapshotOptions)
 	c.Assert(err, check.IsNil)
 	c.Check(setID, check.Equals, uint64(1))
 	c.Check(saved, check.DeepEquals, []string{"one-snap", "too-snap", "tri-snap"})
@@ -737,7 +766,7 @@ exec /bin/tar "$@"
 		c.Assert(os.Mkdir(filepath.Join(homedir, "snap", name, "common", "common-"+name), mode), check.IsNil)
 	}
 
-	setID, saved, taskset, err := snapshotstate.Save(st, nil, []string{"a-user"})
+	setID, saved, taskset, err := snapshotstate.Save(st, nil, []string{"a-user"}, nil)
 	c.Assert(err, check.IsNil)
 	c.Check(setID, check.Equals, uint64(1))
 	c.Check(saved, check.DeepEquals, []string{"one-snap", "too-snap", "tri-snap"})
@@ -837,7 +866,7 @@ exec /bin/tar "$@"
 	// these dir permissions (000) make tar unhappy
 	c.Assert(os.Mkdir(filepath.Join(homedir, "snap/tar-fail-snap/common/common-tar-fail-snap"), 00), check.IsNil)
 
-	setID, saved, taskset, err := snapshotstate.Save(st, nil, []string{"a-user"})
+	setID, saved, taskset, err := snapshotstate.Save(st, nil, []string{"a-user"}, nil)
 	c.Assert(err, check.IsNil)
 	c.Check(setID, check.Equals, uint64(1))
 	c.Check(saved, check.DeepEquals, []string{"tar-fail-snap"})
@@ -1233,7 +1262,7 @@ func testRestoreIntegration(c *check.C, snapDataDir string, opts *dirs.SnapDirOp
 			c.Assert(os.MkdirAll(filepath.Join(home, snapDataDir, name, "common", "common-"+name), 0755), check.IsNil)
 		}
 
-		_, err := backend.Save(context.TODO(), 42, snapInfo, nil, []string{"a-user", "b-user"}, opts)
+		_, err := backend.Save(context.TODO(), 42, snapInfo, nil, []string{"a-user", "b-user"}, nil, opts)
 		c.Assert(err, check.IsNil)
 	}
 
@@ -1312,7 +1341,7 @@ func (snapshotSuite) TestRestoreIntegrationFails(c *check.C) {
 		c.Assert(os.MkdirAll(filepath.Join(homedir, "snap", name, fmt.Sprint(i+1), "canary-"+name), 0755), check.IsNil)
 		c.Assert(os.MkdirAll(filepath.Join(homedir, "snap", name, "common", "common-"+name), 0755), check.IsNil)
 
-		_, err := backend.Save(context.TODO(), 42, snapInfo, nil, []string{"a-user"}, nil)
+		_, err := backend.Save(context.TODO(), 42, snapInfo, nil, []string{"a-user"}, nil, nil)
 		c.Assert(err, check.IsNil)
 	}
 
