@@ -30,7 +30,6 @@ import (
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/progress"
 	"github.com/snapcore/snapd/snap"
-	"github.com/snapcore/snapd/snap/quota"
 	"github.com/snapcore/snapd/timings"
 	"github.com/snapcore/snapd/wrappers"
 )
@@ -206,21 +205,14 @@ func (b Backend) generateWrappers(s *snap.Info, linkCtx LinkContext) error {
 	}
 	cleanupFuncs = append(cleanupFuncs, wrappers.RemoveSnapBinaries)
 
-	vitalityRank := 0
-	var quotaGrp *quota.Group
-	if linkCtx.ServiceOptions != nil {
-		vitalityRank = linkCtx.ServiceOptions.VitalityRank
-		quotaGrp = linkCtx.ServiceOptions.QuotaGroup
-	}
 	// add the daemons from the snap.yaml
-	opts := &wrappers.AddSnapServicesOptions{
-		VitalityRank:            vitalityRank,
+	ensureOpts := &wrappers.EnsureSnapServicesOptions{
 		Preseeding:              b.preseed,
 		RequireMountedSnapdSnap: linkCtx.RequireMountedSnapdSnap,
-		QuotaGroup:              quotaGrp,
 	}
-	// TODO: switch to EnsureSnapServices
-	if err = wrappers.AddSnapServices(s, opts, progress.Null); err != nil {
+	if err = wrappers.EnsureSnapServices(map[*snap.Info]*wrappers.SnapServiceOptions{
+		s: linkCtx.ServiceOptions,
+	}, ensureOpts, nil, progress.Null); err != nil {
 		return err
 	}
 	cleanupFuncs = append(cleanupFuncs, func(s *snap.Info) error {
