@@ -20,6 +20,7 @@
 package snapstate
 
 import (
+	"errors"
 	"fmt"
 	"regexp"
 	"strconv"
@@ -35,6 +36,7 @@ import (
 	seccomp_compiler "github.com/snapcore/snapd/sandbox/seccomp"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snapdtool"
+	"github.com/snapcore/snapd/strutil"
 )
 
 // featureSet contains the flag values that can be listed in assumes entries
@@ -46,134 +48,13 @@ var featureSet = map[string]bool{
 	"snap-env": true,
 	// Support for the "command-chain" feature for apps and hooks in snap.yaml
 	"command-chain": true,
-}
-
-// supportedSystemUsernames for now contains the hardcoded list of system
-// users (and implied system group of same name) that snaps may specify. This
-// will eventually be moved out of here into the store.
-//
-// Since the snap is mounted read-only and to avoid problems associated with
-// different systems using different uids and gids for the same user name and
-// group name, snapd will create system-usernames where 'scope' is not
-// 'external' (currently snapd only supports 'scope: shared') with the
-// following characteristics:
-//
-// - uid and gid shall match for the specified system-username
-// - a snapd-allocated [ug]id for a user/group name shall never change
-// - snapd should avoid [ug]ids that are known to overlap with uid ranges of
-//   common use cases and user namespace container managers so that DAC and
-//   AppArmor owner match work as intended.
-// - [ug]id shall be < 2^31 to avoid (at least) broken devpts kernel code
-// - [ug]id shall be >= 524288 (0x00080000) to give plenty of room for large
-//   sites, default uid/gid ranges for docker (231072-296608), LXD installs
-//   that setup a default /etc/sub{uid,gid} (100000-165536) and podman whose
-//   tutorials reference setting up a specific default user and range
-//   (100000-165536)
-// - [ug]id shall be < 1,000,000 and > 1,001,000,000 (ie, 1,000,000 subordinate
-//   uid with 1,000,000,000 range) to avoid overlapping with LXD's minimum and
-//   maximum id ranges. LXD allows for any id range >= 65536 and doesn't
-//   perform any [ug]id overlap detection with current users
-// - [ug]ids assigned by snapd initially will fall within a 65536 (2^16) range
-//   (see below) where the first [ug]id in the range has the 16 lower bits all
-//   set to zero. This allows snapd to conveniently be bitwise aligned, follows
-//   sensible conventions (see https://systemd.io/UIDS-GIDS.html) but also
-//   potentially discoverable by systemd-nspawn (it assigns a different 65536
-//   range to each container. Its allocation algorithm is not sequential and
-//   may choose anything within its range that isn't already allocated. It's
-//   detection algorithm includes (effectively) performing a getpwent()
-//   operation on CANDIDATE_UID & 0XFFFF0000 and selecting another range if it
-//   is assigned).
-//
-// What [ug]id range(s) should snapd use?
-//
-// While snapd does not employ user namespaces, it will operate on systems with
-// container managers that do and will assign from a range of [ug]ids. It is
-// desirable that snapd assigns [ug]ids that minimally conflict with the system
-// and other software (potential conflicts with admin-assigned ranges in
-// /etc/subuid and /etc/subgid cannot be avoided, but can be documented as well
-// as detected/logged). Overlapping with container managers is non-fatal for
-// snapd and the container, but introduces the possibility that a uid in the
-// container matches a uid a snap is using, which is undesirable in terms of
-// security (eg, DoS via ulimit, same ownership of files between container and
-// snap (even if the other's files are otherwise inaccessible), etc).
-//
-// snapd shall assign [ug]ids from range(s) of 65536 where the lowest value in
-// the range has the 16 lower bits all set to zero (initially just one range,
-// but snapd can add more as needed).
-//
-// To avoid [ug]id overlaps, snapd shall only assign [ug]ids >= 524288
-// (0x00080000) and <= 983040 (0x000F0000, ie the first 65536 range under LXD's
-// minimum where the lower 16 bits are all zeroes). While [ug]ids >= 1001062400
-// (0x3BAB0000, the first 65536 range above LXD's maximum where the lower 16
-// bits are all zeroes) would also avoid overlap, considering nested containers
-// (eg, LXD snap runs a container that runs a container that runs snapd),
-// choosing >= 1001062400 would mean that the admin would need to increase the
-// LXD id range for these containers for snapd to be allowed to create its
-// [ug]ids in the deeply nested containers. The requirements would both be an
-// administrative burden and artificially limit the number of deeply nested
-// containers the host could have.
-//
-// Looking at the LSB and distribution defaults for login.defs, we can observe
-// uids and gids in the system's initial 65536 range (ie, 0-65536):
-//
-// - 0-99        LSB-suggested statically assigned range (eg, root, daemon,
-//               etc)
-// - 0           mandatory 'root' user
-// - 100-499     LSB-suggested dynamically assigned range for system users
-//               (distributions often prefer a higher range, see below)
-// - 500-999     typical distribution default for dynamically assigned range
-//               for system users (some distributions use a smaller
-//               SYS_[GU]ID_MIN)
-// - 1000-60000  typical distribution default for dynamically assigned range
-//               for regular users
-// - 65535 (-1)  should not be assigned since '-1' might be evaluated as this
-//               with set[ug]id* and chown families of functions
-// - 65534 (-2)  nobody/nogroup user for NFS/etc [ug]id anonymous squashing
-// - 65519-65533 systemd recommended reserved range for site-local anonymous
-//               additions, etc
-//
-// To facilitate potential future use cases within the 65536 range snapd will
-// assign from, snapd will only assign from the following subset of ranges
-// relative to the range minimum (ie, its 'base' which has the lower 16 bits
-// all set to zero):
-//
-// - 60500-60999 'scope: shared' system-usernames
-// - 61000-65519 'scope: private' system-usernames
-//
-// Since the first [ug]id range must be >= 524288 and <= 983040 (see above) and
-// following the above guide for system-usernames [ug]ids within this 65536
-// range, the lowest 'scope: shared' user in this range is 584788 (0x0008EC54).
-//
-// Since this number is within systemd-nspawn's range of 524288-1879048191
-// (0x00080000-0x6FFFFFFF), the number's lower 16 bits are not all zeroes so
-// systemd-nspawn won't detect this allocation and could potentially assign the
-// 65536 range starting at 0x00080000 to a container. snapd will therefore also
-// create the 'snapd-range-524288-root' user and group with [ug]id 524288 to
-// work within systemd-nspawn's collision detection. This user/group will not
-// be assigned to snaps at this time.
-//
-// In short (phew!), use the following:
-//
-// $ snappy-debug.id-range 524288 # 0x00080000
-// Host range:              524288-589823 (00080000-0008ffff; 0-65535)
-// LSB static range:        524288-524387 (00080000-00080063; 0-99)
-// Useradd system range:    524788-525287 (000801f4-000803e7; 500-999)
-// Useradd regular range:   525288-584288 (000803e8-0008ea60; 1000-60000)
-// Snapd system range:      584788-585287 (0008ec54-0008ee47; 60500-60999)
-// Snapd private range:     585288-589807 (0008ee48-0008ffef; 61000-65519)
-//
-// Snapd is of course free to add more ranges (eg, 589824 (0x00090000)) with
-// new snapd-range-<base>-root users, or to allocate differently within its
-// 65536 range in the future (sequentially assigned [ug]ids are not required),
-// but for now start very regimented to avoid as many problems as possible.
-//
-// References:
-// https://forum.snapcraft.io/t/multiple-users-and-groups-in-snaps/
-// https://systemd.io/UIDS-GIDS.html
-// https://docs.docker.com/engine/security/userns-remap/
-// https://github.com/lxc/lxd/blob/master/doc/userns-idmap.md
-var supportedSystemUsernames = map[string]uint32{
-	"snap_daemon": 584788,
+	// Support for "kernel-assets" in gadget.yaml. I.e. having volume
+	// content of the style $kernel:ref`
+	"kernel-assets": true,
+	// Support for "refresh-mode: ignore-running" in snap.yaml
+	"app-refresh-mode": true,
+	// Support for "SNAP_UID" and "SNAP_EUID" environment variables
+	"snap-uid-envvars": true,
 }
 
 func checkAssumes(si *snap.Info) error {
@@ -187,23 +68,31 @@ func checkAssumes(si *snap.Info) error {
 		}
 	}
 	if len(missing) > 0 {
-		hint := "try to refresh the core or snapd snaps"
-		if release.OnClassic {
-			hint = "try to update snapd and refresh the core snap"
-		}
-		return fmt.Errorf("snap %q assumes unsupported features: %s (%s)", si.InstanceName(), strings.Join(missing, ", "), hint)
+		return fmt.Errorf("snap %q assumes unsupported features: %s (try to refresh snapd)", si.InstanceName(), strings.Join(missing, ", "))
 	}
 	return nil
 }
 
-var versionExp = regexp.MustCompile(`^([1-9][0-9]*)(?:\.([0-9]+)(?:\.([0-9]+))?)?`)
+// regular expression which matches a version expressed as groups of digits
+// separated with dots, with optional non-numbers afterwards
+var versionExp = regexp.MustCompile(`^(?:[1-9][0-9]*)(?:\.(?:[0-9]+))*`)
 
 func checkVersion(version string) bool {
 	// double check that the input looks like a snapd version
-	req := versionExp.FindStringSubmatch(version)
-	if req == nil || req[0] != version {
+	reqVersionNumMatch := versionExp.FindStringSubmatch(version)
+	if reqVersionNumMatch == nil {
 		return false
 	}
+	// this check ensures that no one can use an assumes like snapd2.48.3~pre2
+	// or snapd2.48.5+20.10, as modifiers past the version number are not meant
+	// to be relied on for snaps via assumes, however the check against the real
+	// snapd version number below allows such non-numeric modifiers since real
+	// snapds do have versions like that (for example debian pkg of snapd)
+	if reqVersionNumMatch[0] != version {
+		return false
+	}
+
+	req := strings.Split(reqVersionNumMatch[0], ".")
 
 	if snapdtool.Version == "unknown" {
 		return true // Development tree.
@@ -213,16 +102,19 @@ func checkVersion(version string) bool {
 	// this code (see PR#7344). However this would change current
 	// behavior, i.e. "2.41~pre1" would *not* match [snapd2.41] anymore
 	// (which the code below does).
-	cur := versionExp.FindStringSubmatch(snapdtool.Version)
-	if cur == nil {
+	curVersionNumMatch := versionExp.FindStringSubmatch(snapdtool.Version)
+	if curVersionNumMatch == nil {
 		return false
 	}
+	cur := strings.Split(curVersionNumMatch[0], ".")
 
-	for i := 1; i < len(req); i++ {
-		if req[i] == "" {
-			return true
-		}
-		if cur[i] == "" {
+	for i := range req {
+		if i == len(cur) {
+			// we hit the end of the elements of the current version number and have
+			// more required version numbers left, so this doesn't match, if the
+			// previous element was higher we would have broken out already, so the
+			// only case left here is where we have version requirements that are
+			// not met
 			return false
 		}
 		reqN, err1 := strconv.Atoi(req[i])
@@ -426,7 +318,7 @@ func checkCoreName(st *state.State, snapInfo, curInfo *snap.Info, _ snap.Contain
 		return nil
 	}
 	core, err := coreInfo(st)
-	if err == state.ErrNoState {
+	if errors.Is(err, state.ErrNoState) {
 		return nil
 	}
 	if err != nil {
@@ -467,7 +359,6 @@ func checkGadgetOrKernel(st *state.State, snapInfo, curInfo *snap.Info, snapf sn
 		// not a relevant check
 		return nil
 	}
-
 	ok, err := HasSnapOfType(st, typ)
 	if err != nil {
 		return fmt.Errorf("cannot detect original %s snap: %v", kind, err)
@@ -478,8 +369,8 @@ func checkGadgetOrKernel(st *state.State, snapInfo, curInfo *snap.Info, snapf sn
 		return nil
 	}
 
-	currentSnap, err := infoForDeviceSnap(st, deviceCtx, kind, whichName)
-	if err == state.ErrNoState {
+	currentSnap, err := infoForDeviceSnap(st, deviceCtx, whichName)
+	if errors.Is(err, state.ErrNoState) {
 		// check if we are in the remodel case
 		if deviceCtx != nil && deviceCtx.ForRemodeling() {
 			if whichName(deviceCtx.Model()) == snapInfo.InstanceName() {
@@ -492,7 +383,7 @@ func checkGadgetOrKernel(st *state.State, snapInfo, curInfo *snap.Info, snapf sn
 		return fmt.Errorf("cannot find original %s snap: %v", kind, err)
 	}
 
-	if currentSnap.SnapID != "" && snapInfo.SnapID == "" {
+	if currentSnap.SnapID != "" && snapInfo.SnapID == "" && deviceCtx.Model().Grade() != asserts.ModelDangerous {
 		return fmt.Errorf("cannot replace signed %s snap with an unasserted one", kind)
 	}
 
@@ -560,7 +451,7 @@ func checkEpochs(_ *state.State, snapInfo, curInfo *snap.Info, _ snap.Container,
 }
 
 // check that the snap installed in the system (via snapst) can be
-// upgraded to info (i.e. that info's epoch can read sanpst's epoch)
+// upgraded to info (i.e. that info's epoch can read snapst's epoch)
 func earlyEpochCheck(info *snap.Info, snapst *SnapState) error {
 	if snapst == nil {
 		// no snapst, no problem
@@ -578,15 +469,37 @@ func earlyEpochCheck(info *snap.Info, snapst *SnapState) error {
 	return checkEpochs(nil, info, cur, nil, Flags{}, nil)
 }
 
+func earlyChecks(st *state.State, snapst *SnapState, update *snap.Info, flags Flags) (Flags, error) {
+	flags, err := ensureInstallPreconditions(st, update, flags, snapst)
+	if err != nil {
+		return flags, err
+	}
+
+	if err := earlyEpochCheck(update, snapst); err != nil {
+		return flags, err
+	}
+	return flags, nil
+}
+
 // check that the listed system users are valid
 var osutilEnsureUserGroup = osutil.EnsureUserGroup
 
 func validateSystemUsernames(si *snap.Info) error {
 	for _, user := range si.SystemUsernames {
-		if _, ok := supportedSystemUsernames[user.Name]; !ok {
+		systemUserName, ok := snap.SupportedSystemUsernames[user.Name]
+		if !ok {
 			return fmt.Errorf(`snap %q requires unsupported system username "%s"`, si.InstanceName(), user.Name)
 		}
 
+		if systemUserName.AllowedSnapIds != nil && si.SnapID != "" {
+			// Only certain snaps can use this user; let's check whether ours
+			// is one of these
+			if !strutil.ListContains(systemUserName.AllowedSnapIds, si.SnapID) {
+				return fmt.Errorf(`snap %q is not allowed to use the system user %q`,
+					si.InstanceName(), user.Name)
+			}
+		}
+
 		switch user.Scope {
 		case "shared":
 			// this is supported
@@ -631,7 +544,7 @@ func checkAndCreateSystemUsernames(si *snap.Info) error {
 	// TODO: move user creation to a more appropriate place like "link-snap"
 	extrausers := !release.OnClassic
 	for _, user := range si.SystemUsernames {
-		id := supportedSystemUsernames[user.Name]
+		id := snap.SupportedSystemUsernames[user.Name].Id
 		switch user.Scope {
 		case "shared":
 			// Create the snapd-range-<base>-root user and group so
@@ -653,10 +566,21 @@ func checkAndCreateSystemUsernames(si *snap.Info) error {
 	return nil
 }
 
+func checkConfigureHooks(_ *state.State, snapInfo, curInfo *snap.Info, _ snap.Container, _ Flags, deviceCtx DeviceContext) error {
+	hasDefaultConfigureHook := snapInfo.Hooks["default-configure"] != nil
+	hasConfigureHook := snapInfo.Hooks["configure"] != nil
+
+	if hasDefaultConfigureHook && !hasConfigureHook {
+		return fmt.Errorf(`cannot specify "default-configure" hook without "configure" hook`)
+	}
+	return nil
+}
+
 func init() {
 	AddCheckSnapCallback(checkCoreName)
 	AddCheckSnapCallback(checkSnapdName)
 	AddCheckSnapCallback(checkGadgetOrKernel)
 	AddCheckSnapCallback(checkBases)
 	AddCheckSnapCallback(checkEpochs)
+	AddCheckSnapCallback(checkConfigureHooks)
 }
