@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016 Canonical Ltd
+ * Copyright (C) 2016-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -69,7 +69,7 @@ func (cs *changeSuite) TestReadyTime(c *C) {
 }
 
 func (cs *changeSuite) TestStatusString(c *C) {
-	for s := state.Status(0); s < state.ErrorStatus+1; s++ {
+	for s := state.Status(0); s < state.WaitStatus+1; s++ {
 		c.Assert(s.String(), Matches, ".+")
 	}
 }
@@ -89,6 +89,21 @@ func (cs *changeSuite) TestGetSet(c *C) {
 	c.Check(v, Equals, 1)
 }
 
+func (cs *changeSuite) TestHas(c *C) {
+	st := state.New(nil)
+	st.Lock()
+	defer st.Unlock()
+
+	chg := st.NewChange("install", "...")
+	c.Check(chg.Has("a"), Equals, false)
+
+	chg.Set("a", 1)
+	c.Check(chg.Has("a"), Equals, true)
+
+	chg.Set("a", nil)
+	c.Check(chg.Has("a"), Equals, false)
+}
+
 // TODO Better testing of full change roundtripping via JSON.
 
 func (cs *changeSuite) TestNewTaskAddTaskAndTasks(c *C) {
@@ -228,9 +243,13 @@ func (cs *changeSuite) TestStatusDerivedFromTasks(c *C) {
 
 	tasks := make(map[state.Status]*state.Task)
 
-	for s := state.DefaultStatus + 1; s < state.ErrorStatus+1; s++ {
+	for s := state.DefaultStatus + 1; s < state.WaitStatus+1; s++ {
 		t := st.NewTask("download", s.String())
-		t.SetStatus(s)
+		if s == state.WaitStatus {
+			t.SetToWait(state.DoneStatus)
+		} else {
+			t.SetStatus(s)
+		}
 		chg.AddTask(t)
 		tasks[s] = t
 	}
@@ -241,6 +260,7 @@ func (cs *changeSuite) TestStatusDerivedFromTasks(c *C) {
 		state.UndoStatus,
 		state.DoingStatus,
 		state.DoStatus,
+		state.WaitStatus,
 		state.ErrorStatus,
 		state.UndoneStatus,
 		state.DoneStatus,
@@ -253,7 +273,11 @@ func (cs *changeSuite) TestStatusDerivedFromTasks(c *C) {
 			if s == s2 {
 				break
 			}
-			tasks[s2].SetStatus(s)
+			if s == state.WaitStatus {
+				tasks[s2].SetToWait(state.DoneStatus)
+			} else {
+				tasks[s2].SetStatus(s)
+			}
 		}
 		c.Assert(chg.Status(), Equals, s)
 	}
@@ -433,9 +457,13 @@ func (cs *changeSuite) TestAbort(c *C) {
 
 	chg := st.NewChange("install", "...")
 
-	for s := state.DefaultStatus + 1; s < state.ErrorStatus+1; s++ {
+	for s := state.DefaultStatus + 1; s < state.WaitStatus+1; s++ {
 		t := st.NewTask("download", s.String())
-		t.SetStatus(s)
+		if s == state.WaitStatus {
+			t.SetToWait(state.DoneStatus)
+		} else {
+			t.SetStatus(s)
+		}
 		t.Set("old-status", s)
 		chg.AddTask(t)
 	}
@@ -452,7 +480,7 @@ func (cs *changeSuite) TestAbort(c *C) {
 		switch s {
 		case state.DoStatus:
 			c.Assert(t.Status(), Equals, state.HoldStatus)
-		case state.DoneStatus:
+		case state.DoneStatus, state.WaitStatus:
 			c.Assert(t.Status(), Equals, state.UndoStatus)
 		case state.DoingStatus:
 			c.Assert(t.Status(), Equals, state.AbortStatus)
@@ -527,16 +555,17 @@ func (cs *changeSuite) TestAbortKⁿ(c *C) {
 
 // Task wait order:
 //
-//             => t21 => t22
-//           /               \
+//	  => t21 => t22
+//	/               \
+//
 // t11 => t12                 => t41 => t42
-//           \               /
-//             => t31 => t32
+//
+//	\               /
+//	  => t31 => t32
 //
 // setup and result lines are <task>:<status>[:<lane>,...]
 //
 // "*" as task name means "all remaining".
-//
 var abortLanesTests = []struct {
 	setup  string
 	abort  []int
@@ -579,6 +608,10 @@ var abortLanesTests = []struct {
 		setup:  "t11:do:1 t12:do:1 t21:do:2 t22:do:2 t31:do:3 t32:do:3 t41:do:4 t42:do:4",
 		abort:  []int{2},
 		result: "t21:hold t22:hold t41:hold t42:hold *:do",
+	}, {
+		setup:  "t11:done:1 t12:wait:1 t21:do:2 t22:do:2 t31:do:3 t32:do:3 t41:do:4 t42:do:4",
+		abort:  []int{2},
+		result: "t21:hold t22:hold t41:hold t42:hold t11:done t12:wait *:do",
 	}, {
 		setup:  "t11:do:1 t12:do:1 t21:do:2 t22:do:2 t31:do:3 t32:do:3 t41:do:4 t42:do:4",
 		abort:  []int{3},
@@ -662,7 +695,7 @@ func (ts *taskRunnerSuite) TestAbortLanes(c *C) {
 		c.Logf("Testing setup: %s", test.setup)
 
 		statuses := make(map[string]state.Status)
-		for s := state.DefaultStatus; s <= state.ErrorStatus; s++ {
+		for s := state.DefaultStatus; s <= state.WaitStatus; s++ {
 			statuses[strings.ToLower(s.String())] = s
 		}
 
@@ -682,7 +715,11 @@ func (ts *taskRunnerSuite) TestAbortLanes(c *C) {
 			}
 			seen[parts[0]] = true
 			task := tasks[parts[0]]
-			task.SetStatus(statuses[parts[1]])
+			if statuses[parts[1]] == state.WaitStatus {
+				task.SetToWait(state.DoneStatus)
+			} else {
+				task.SetStatus(statuses[parts[1]])
+			}
 			if len(parts) > 2 {
 				lanes := strings.Split(parts[2], ",")
 				for _, lane := range lanes {
@@ -726,11 +763,9 @@ func (ts *taskRunnerSuite) TestAbortLanes(c *C) {
 	}
 }
 
-//
 // setup and result lines are <task>:<status>[:<lane>,...]
 // order is <task1>-><task2> (implies task2 waits for task 1)
 // "*" as task name means "all remaining".
-//
 var abortUnreadyLanesTests = []struct {
 	setup  string
 	order  string
@@ -741,6 +776,9 @@ var abortUnreadyLanesTests = []struct {
 	{
 		setup:  "*:do",
 		result: "*:hold",
+	}, {
+		setup:  "*:wait",
+		result: "*:undo",
 	}, {
 		setup:  "*:done",
 		result: "*:done",
@@ -778,6 +816,11 @@ var abortUnreadyLanesTests = []struct {
 		order: "t11->t12 t12->t21 t12->t31 t21->t22 t31->t32 t22->t41 t32->t41 t41->t42",
 		// lane 2 is fully complete so it does not get aborted
 		result: "t11:done t12:done t21:done t22:done t31:abort t32:hold t41:hold t42:hold *:undo",
+	}, {
+		setup: "t11:done:2,3 t12:done:2,3 t21:done:2 t22:done:2 t31:wait:3 t32:do:3 t41:do:4 t42:do:4",
+		order: "t11->t12 t12->t21 t12->t31 t21->t22 t31->t32 t22->t41 t32->t41 t41->t42",
+		// lane 2 is fully complete so it does not get aborted
+		result: "t11:done t12:done t21:done t22:done t31:undo t32:hold t41:hold t42:hold *:undo",
 	}, {
 		setup:  "t11:done:2,3 t12:done:2,3 t21:doing:2 t22:do:2 t31:doing:3 t32:do:3 t41:do:4 t42:do:4",
 		order:  "t11->t12 t12->t21 t12->t31 t21->t22 t31->t32 t22->t41 t32->t41 t41->t42",
@@ -865,7 +908,7 @@ func (ts *taskRunnerSuite) TestAbortUnreadyLanes(c *C) {
 		}
 
 		statuses := make(map[string]state.Status)
-		for s := state.DefaultStatus; s <= state.ErrorStatus; s++ {
+		for s := state.DefaultStatus; s <= state.WaitStatus; s++ {
 			statuses[strings.ToLower(s.String())] = s
 		}
 
@@ -886,7 +929,11 @@ func (ts *taskRunnerSuite) TestAbortUnreadyLanes(c *C) {
 			}
 			seen[parts[0]] = true
 			task := tasks[parts[0]]
-			task.SetStatus(statuses[parts[1]])
+			if statuses[parts[1]] == state.WaitStatus {
+				task.SetToWait(state.DoneStatus)
+			} else {
+				task.SetStatus(statuses[parts[1]])
+			}
 			if len(parts) > 2 {
 				lanes := strings.Split(parts[2], ",")
 				for _, lane := range lanes {
