@@ -1137,7 +1137,7 @@ func (s *seed16Suite) TestLoadMetaCore18StoreInfo(c *C) {
 	privateSnapSideInfo := s.AssertedSnapInfo("private-snap").SideInfo
 	privateSnapSideInfo.Private = true
 	contactableSnapSideInfo := s.AssertedSnapInfo("contactable-snap").SideInfo
-	contactableSnapSideInfo.EditedContact = "mailto:author@example.com"
+	contactableSnapSideInfo.LegacyEditedContact = "mailto:author@example.com"
 
 	// these are not sorted by type, firstboot will do that
 	c.Check(runSnaps, DeepEquals, []*seed.Snap{
