@@ -1,16 +1,10 @@
 #!/bin/bash
 
-SNAPD_STATE_PATH="$SPREAD_PATH/tests/snapd-state"
-SNAPD_STATE_FILE="$SPREAD_PATH/tests/snapd-state/snapd-state.tar"
-RUNTIME_STATE_PATH="$SPREAD_PATH/tests/runtime-state"
+SNAPD_STATE_PATH="$TESTSTMP/snapd-state"
+SNAPD_STATE_FILE="$TESTSTMP/snapd-state/snapd-state.tar"
+RUNTIME_STATE_PATH="$TESTSTMP/runtime-state"
 SNAPD_ACTIVE_UNITS="$RUNTIME_STATE_PATH/snapd-active-units"
 
-# shellcheck source=tests/lib/dirs.sh
-. "$TESTSLIB/dirs.sh"
-
-# shellcheck source=tests/lib/systemd.sh
-. "$TESTSLIB/systemd.sh"
-
 
 delete_snapd_state() {
     rm -rf "$SNAPD_STATE_PATH"
@@ -30,7 +24,12 @@ is_snapd_state_saved() {
     fi
 }
 
+get_active_snapd_units() {
+    systemctl list-units --plain --state=active | grep -Eo '^snapd\..*(socket|service|timer)' || true
+}
+
 save_snapd_state() {
+    prepare_state
     if os.query is-core; then
         boot_path="$("$TESTSTOOLS"/boot-state boot-path)"
         test -n "$boot_path" || return 1
@@ -46,6 +45,7 @@ save_snapd_state() {
         cp -a /var/snap/* "$SNAPD_STATE_PATH"/var-snap/
     else
         systemctl daemon-reload
+        SNAP_MOUNT_DIR="$(os.paths snap-mount-dir)"
         escaped_snap_mount_dir="$(systemd-escape --path "$SNAP_MOUNT_DIR")"
         units="$(systemctl list-unit-files --full | grep -e "^${escaped_snap_mount_dir}[-.].*\\.mount" -e "^${escaped_snap_mount_dir}[-.].*\\.service" | cut -f1 -d ' ')"
         for unit in $units; do
@@ -61,6 +61,7 @@ save_snapd_state() {
             /var/cache/snapd \
             "$SNAP_MOUNT_DIR" \
             /etc/systemd/system/"$escaped_snap_mount_dir"-*core*.mount \
+            /etc/systemd/system/snapd.mounts.target.wants/"$escaped_snap_mount_dir"-*core*.mount \
             /etc/systemd/system/multi-user.target.wants/"$escaped_snap_mount_dir"-*core*.mount \
             $snap_confine_profiles \
             $snapd_env \
@@ -79,7 +80,7 @@ save_snapd_state() {
     fi
 
     # Save the snapd active units
-    systemd_get_active_snapd_units > "$SNAPD_ACTIVE_UNITS"
+    get_active_snapd_units > "$SNAPD_ACTIVE_UNITS"
 }
 
 restore_snapd_state() {
@@ -123,7 +124,7 @@ restore_snapd_lib() {
     # Synchronize snaps, seed and cache directories. The this is done separately in order to avoid copying
     # the snap files due to it is a heavy task and take most of the time of the restore phase.
     rsync -av --delete "$SNAPD_STATE_PATH"/snapd-lib/snaps /var/lib/snapd
-    if os.query is-core20 ; then
+    if os.query is-core20 || os.query is-core22; then
         # TODO:UC20: /var/lib/snapd/seed is a read only bind mount, use the rw
         # mount or later mount seed as needed
         rsync -av --delete "$SNAPD_STATE_PATH"/snapd-lib/seed/ /run/mnt/ubuntu-seed/
@@ -132,3 +133,9 @@ restore_snapd_lib() {
     fi
     rsync -av --delete "$SNAPD_STATE_PATH"/snapd-lib/cache /var/lib/snapd
 }
+
+remove_disabled_snaps() {
+    snap list --all | grep disabled | while read -r name _ revision _ ; do
+        snap remove "$name" --revision="$revision"
+    done
+}
